
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is Calendar.pas.                                          }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit Calendar;

interface

uses
  Classes, JvxHTMLClasses;

type

  { TCalendar }

  TCalendar = class(TJvxHTMLCustomDiv)
  private
    FDay: TJvxHTMLSelect;
    FMonth: TJvxHTMLSelect;
    FYear: TJvxHTMLSelect;
    function GetDate: TDateTime;
    procedure SetDate(Value: TDateTime);
  public
    constructor Create(Owner: TComponent); override;
  published
    property Date: TDateTime read GetDate write SetDate;
  end;

implementation

uses
  SysUtils;

{ TCalendar }

constructor TCalendar.Create(Owner: TComponent);
var
  Day: Word;
  I: Integer;
  Month: Word;
  Table: TJvxHTMLTable;
  TableBody: TJvxHTMLTableBody;
  TableRow: TJvxHTMLTableRow;
  TableCell: TJvxHTMLTableDataCell;
  Year: Word;
begin
  inherited;

  // Create the table that holds 3 combo boxes together
  Table := TJvxHTMLTable.Create(Self);
  with Table do
  begin
    Border.Value := 0;
    CellPadding.AsInteger := 0;
    CellSpacing.AsInteger := 0;
    Parent := Self;
  end;

  TableBody := TJvxHTMLTableBody.Create(Self);
  TableBody.Parent := Table;

  TableRow := TJvxHTMLTableRow.Create(Self);
  TableRow.Parent := TableBody;

  TableCell := TJvxHTMLTableDataCell.Create(Self);
  TableCell.Parent := TableRow;

  // Create month combo box
  FMonth := TJvxHTMLSelect.Create(Self);
  with FMonth do
  begin
    for I := Low(ShortMonthNames) to High(ShortMonthNames) do
      with Options.Add do
      begin
        Value.AsInteger := I;
        AsString := ShortMonthNames[I];
      end;
    Parent := TableCell;
  end;

  TableCell := TJvxHTMLTableDataCell.Create(Self);
  TableCell.Parent := TableRow;

  // Create day combo box
  FDay := TJvxHTMLSelect.Create(Self);
  with FDay do
  begin
    for I := 1 to 31 do
      with Options.Add do
      begin
        Value.AsInteger := I;
        AsInteger := I;
      end;
    Parent := TableCell;
  end;

  TableCell := TJvxHTMLTableDataCell.Create(Self);
  TableCell.Parent := TableRow;

  // Create year combo box
  DecodeDate(Now, Year, Month, Day);
  FYear := TJvxHTMLSelect.Create(Self);
  with FYear do
  begin
    for I := Year - 10 to Year + 10 do
      with Options.Add do
      begin
        Value.AsInteger := I;
        AsInteger := I;
      end;
    Parent := TableCell;
  end;
end;

function TCalendar.GetDate: TDateTime;
var
  Day: Word;
  I: Integer;
  Month: Word;
  Year: Word;
begin
  Day := 0;
  for I := 0 to FDay.Options.Count - 1 do
    if FDay.Options[I].Selected.Assigned and FDay.Options[I].Selected.Value then
    begin
      Day := FDay.Options[I].Value.AsInteger;
      Break;
    end;

  Month := 0;
  for I := 0 to FMonth.Options.Count - 1 do
    if FMonth.Options[I].Selected.Assigned and FMonth.Options[I].Selected.Value then
    begin
      Month := FMonth.Options[I].Value.AsInteger;
      Break;
    end;

  Year := 0;
  for I := 0 to FYear.Options.Count - 1 do
    if FYear.Options[I].Selected.Assigned and FYear.Options[I].Selected.Value then
    begin
      Year := FYear.Options[I].Value.AsInteger;
      Break;
    end;

  Result := EncodeDate(Year, Month, Day);
end;

procedure TCalendar.SetDate(Value: TDateTime);
var
  Day: Word;
  I: Integer;
  Month: Word;
  Year: Word;
begin
  DecodeDate(Value, Year, Month, Day);
  for I := 0 to FDay.Options.Count - 1 do
    FDay.Options[I].Selected.Value := FDay.Options[I].Value.AsInteger = Day;
  for I := 0 to FMonth.Options.Count - 1 do
    FMonth.Options[I].Selected.Value := FMonth.Options[I].Value.AsInteger = Month;
  for I := 0 to FYear.Options.Count - 1 do
    FYear.Options[I].Selected.Value := FYear.Options[I].Value.AsInteger = Year;
end;

end.
