
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is LookupEdit.pas.                                        }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit LookupEdit;

interface

uses
  Classes, JvxStdNodes, JvxHTMLClasses;

type

  { TLookupEdit }

  TLookupEdit = class(TJvxHTMLCustomDiv)
  private
    FEdit: TJvxHTMLInput;
    FEditCell: TJvxHTMLTableDataCell;
    FButtonCell: TJvxHTMLTableDataCell;
    FButtonImage: TJvxHTMLImage;
    FButtonTable: TJvxHTMLTable;
    function GetControlName: TJvxProtectedStringAttribute;
    function GetOnButtonClick: TJvxHTMLScriptAttribute;
    function GetValue: TJvxProtectedStringAttribute;
  public
    constructor Create(Owner: TComponent); override;
  published
    property ControlName: TJvxProtectedStringAttribute read GetControlName;
    property OnButtonClick: TJvxHTMLScriptAttribute read GetOnButtonClick;
    property Value: TJvxProtectedStringAttribute read GetValue;
  end;


implementation

  // <TABLE border="0" cellpadding="0" cellspacing="0">
  //   <TR>
  //     <TD>
  //       <INPUT type="text" style="border-right-style:none">
  //     </TD>
  //     <TD height="100%" style="padding-top:1px; padding-bottom:1px">
  //       <SPAN style="height:100%; border-left:none; border-top:solid 1px ThreeDShadow; border-right:solid 1px ThreeDHighlight; border-bottom:solid 1px ThreeDHighlight">
  //         <SPAN style="height:100%; border-left:none; border-top:solid 1px ThreeDDarkShadow; border-right:solid 1px ThreeDLightShadow; border-bottom:solid 1px ThreeDLightShadow">
  //           <TABLE style="border-left:solid 1px ThreeDLightShadow; border-top:solid 1px ThreeDLightShadow; border-right:solid 1px ThreeDDarkShadow; border-bottom:solid 1px ThreeDDarkShadow" cellpadding="1" cellspacing="0" height="100%" valign="middle">
  //             <TR>
  //               <TD style="border-left:solid 1px ThreeDHighlight; border-top:solid 1px ThreeDHighlight; border-right:solid 1px ThreeDShadow; border-bottom:solid 1px ThreeDShadow;">
  //                 <IMG src="../Images/GlyphLookupInactive.gif">
  //               </TD>
  //             </TR>
  //           </TABLE>
  //         </SPAN>
  //       </SPAN>
  //     </TD>
  //   </TR>
  // </TABLE>

{ TLookupEdit }

constructor TLookupEdit.Create(Owner: TComponent);
var
  Table: TJvxHTMLTable;
  TableBody: TJvxHTMLTableBody;
  TableRow: TJvxHTMLTableRow;
  OuterButtonCell: TJvxHTMLTableDataCell;
  OuterButtonDiv: TJvxHTMLDiv;
  InnerButtonDiv: TJvxHTMLDiv;
  ButtonTableBody: TJvxHTMLTableBody;
  ButtonTableRow: TJvxHTMLTableRow;
begin
  inherited;

  Table := TJvxHTMLTable.Create(Self);
  with Table do
  begin
    Border.AsInteger := 0;
    CellPadding.AsInteger := 0;
    CellSpacing.AsInteger := 0;
    Style.AsString := 'display:inline';
    Parent := Self;
  end;

  TableBody := TJvxHTMLTableBody.Create(Self);
  TableBody.Parent := Table;

  TableRow := TJvxHTMLTableRow.Create(Self);
  TableRow.Parent := TableBody;

  FEditCell := TJvxHTMLTableDataCell.Create(Self);
  FEditCell.Parent := TableRow;

  FEdit := TJvxHTMLInput.Create(Self);
  with FEdit do
  begin
    InputType.Value := itText;
    Style.AsString := 'border-right-style:none';
    Parent := FEditCell;
  end;

  OuterButtonCell := TJvxHTMLTableDataCell.Create(Self);
  with OuterButtonCell do
  begin
    Height.AsString := '100%';
    Style.AsString := 'padding-top:1px; padding-bottom:1px';
    Parent := TableRow;
  end;

  OuterButtonDiv := TJvxHTMLDiv.Create(Self);
  with OuterButtonDiv do
  begin
    Style.AsString := 'height:100%; border:solid 1px; border-left-style:none; ' +
      'border-top-color:ThreeDShadow; border-right-color:ThreeDHighlight; ' +
      'border-bottom-color:ThreeDHighlight';
    Parent := OuterButtonCell;
  end;

  InnerButtonDiv := TJvxHTMLDiv.Create(Self);
  with InnerButtonDiv do
  begin
    Style.AsString := 'height:100%; border:solid 1px; border-left-style:none; ' +
      'border-top-color:ThreeDDarkShadow; border-right-color:ThreeDLightShadow; ' +
      'border-bottom-color:ThreeDLightShadow;';
    Parent := OuterButtonDiv;
  end;

  FButtonTable := TJvxHTMLTable.Create(Self);
  with FButtonTable do
  begin
    Height.AsString := '100%';
    CellPadding.AsInteger := 0;
    CellSpacing.AsInteger := 0;
    Style.AsString := 'background-color:ThreeDFace; border:solid 1px;' +
      'border-left-color:ThreeDLightShadow; border-top-color:ThreeDLightShadow; ' +
      'border-right-color:ThreeDDarkShadow; border-bottom-color:ThreeDDarkShadow;';
    Parent := InnerButtonDiv;
  end;

  ButtonTableBody := TJvxHTMLTableBody.Create(Self);
  ButtonTableBody.Parent := FButtonTable;

  ButtonTableRow := TJvxHTMLTableRow.Create(Self);
  ButtonTableRow.Parent := ButtonTableBody;

  FButtonCell := TJvxHTMLTableDataCell.Create(Self);
  with FButtonCell do
  begin
    Style.AsString := 'border:solid 1px; padding:2px; ' +
      'border-left-color:ThreeDHighlight; border-top-color:ThreeDHighlight; ' +
      'border-right-color:ThreeDShadow; border-bottom-color:ThreeDShadow;';
    Parent := ButtonTableRow;
  end;

  FButtonImage := TJvxHTMLImage.Create(Self);
  with FButtonImage do
  begin
    Source.AsString := 'LookupBtn.gif';
    Parent := FButtonCell;
  end;
end;

function TLookupEdit.GetControlName: TJvxProtectedStringAttribute;
begin
  Result := FEdit.ControlName;
end;

function TLookupEdit.GetOnButtonClick: TJvxHTMLScriptAttribute;
begin
  Result := FButtonTable.OnClick;
end;

function TLookupEdit.GetValue: TJvxProtectedStringAttribute;
begin
  Result := FEdit.Value;
end;

end.
