
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is Main.pas.                                              }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit Main;

interface

{ NOTE: To successfully compile this example you will need to make sure that
  XVCL source directory is included into library search path.

  See Readme.htm or start the program for more information about this example. }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OleCtrls, SHDocVw, ActnList, StdActns, ImgList, ExtCtrls, ComCtrls,
  ToolWin, Db, DBTables, JvxClasses, JvxHTMLClasses, JvxDataSources;

type
  TMainForm = class(TForm)
    ToolBar: TToolBar;
    WebBrowser: TWebBrowser;
    ToolbarImages: TImageList;
    ActionList: TActionList;
    ExecuteAction: TAction;
    ExecuteButton: TToolButton;
    CloseButton: TToolButton;
    WindowCloseAction: TAction;
    CustomerTable: TTable;
    CustomerDataSource: TDataSource;
    procedure ExecuteActionExecute(Sender: TObject);
    procedure ExecuteActionUpdate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WindowCloseActionExecute(Sender: TObject);
    procedure WindowCloseActionUpdate(Sender: TObject);
  private
    { Private declarations }
    FExeDirectory: string;
    FNodeSource: TJvxDBDataSource;
    FTable: TJvxHTMLTable;
    procedure Execute;
    function CreateTableBodyRow: TJvxHTMLTableRow;
    function CreateTableHeaderRow: TJvxHTMLTableRow;
    procedure GenerateHTML(Node: TJvxNode);
    procedure InitializeHTMLTable;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  JvxXMLClasses;

{$R *.DFM}

const
  ReadmeFileName = 'Readme.htm';
  OutputFileName = 'Output.htm';

function TMainForm.CreateTableBodyRow: TJvxHTMLTableRow;
var
  DataCell: TJvxHTMLTableDataCell;
  I: Integer;
begin
  // Create <TR> element
  Result := TJvxHTMLTableRow.Create(Self);

  // Create <TD> elements for all database fields in CUSTOMER.DB
  for I := 0 to FNodeSource.Fields.Count - 1 do
  begin
    // Create <TD> element
    DataCell := TJvxHTMLTableDataCell.Create(Self);
    // Connect it to the datafield for automatic data loading
    DataCell.DataField := FNodeSource.Fields[I];
    DataCell.Parent := Result;
  end;
end;

function TMainForm.CreateTableHeaderRow: TJvxHTMLTableRow;
var
  HeaderCell: TJvxHTMLTableHeaderCell;
  I: Integer;
begin
  // Create <TR> element
  Result := TJvxHTMLTableRow.Create(Self);

  // Create <TH> elements for all database fields in CUSTOMER.DB
  for I := 0 to FNodeSource.Fields.Count - 1 do
  begin
    // Create <TH> element
    HeaderCell := TJvxHTMLTableHeaderCell.Create(Self);
    // Insert field name into the cell, for example <TH>CustNo</TH>
    HeaderCell.AsString := FNodeSource.Fields[I].FieldName;
    HeaderCell.Parent := Result;
  end;
end;

procedure TMainForm.Execute;
begin
  // Generate HTML file from the template we have created earlier
  GenerateHTML(FTable);
end;

procedure TMainForm.ExecuteActionExecute(Sender: TObject);
begin
  // Generate HTML and display the results in the browser
  Execute;
  WebBrowser.Navigate(FExeDirectory + OutputFileName);
end;

procedure TMainForm.ExecuteActionUpdate(Sender: TObject);
begin
  ExecuteAction.Enabled := Pos(ReadmeFileName, WebBrowser.LocationURL) <> 0;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  // Display Readme.htm in the browser
  WebBrowser.Navigate(FExeDirectory + ReadmeFileName);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FExeDirectory := IncludeTrailingBackslash(ExtractFilePath(Application.ExeName));

  // Create a data source object to serve as a source of <TR> elements
  // Note that FNodeSource.Fields will be populated with fields from
  // CustomerTable.
  FNodeSource := TJvxDBDataSource.Create(Self);
  FNodeSource.DataSource := CustomerDataSource;
  FNodeSource.Active := True;

  // Create a template of HTML document that will be later used to produce HTML
  FTable := TJvxHTMLTable.Create(Self);
  InitializeHTMLTable;
end;

procedure TMainForm.GenerateHTML(Node: TJvxNode);
var
  ContentGenerator: TJvxXMLGenerator;
  OutputFile: TextFile;
begin
  // Create XML generator
  ContentGenerator := TJvxXMLGenerator.Create;
  try
    // Open output file
    AssignFile(OutputFile, FExeDirectory + OutputFileName);
    Rewrite(OutputFile);
    try
      // Write generated content into the output file
      WriteLn(OutputFile, ContentGenerator.XMLFrom(Node));
    finally
      CloseFile(OutputFile);
    end;
  finally
    ContentGenerator.Free;
  end;
end;

procedure TMainForm.InitializeHTMLTable;
var
  TableHead: TJvxHTMLTableHeader;
  TableBody: TJvxHTMLTableBody;
begin
  // Set <TABLE border="1">
  FTable.Border.Value := 1;

  // Create <THEAD>
  TableHead := TJvxHTMLTableHeader.Create(Self);
  TableHead.Parent := FTable;

  // Create <TR>
  TableHead.Elements.Add(CreateTableHeaderRow);

  // Create <TBODY>
  TableBody := TJvxHTMLTableBody.Create(Self);
  TableBody.Parent := FTable;

  // Initialize the template of <TBODY> that will be used for all <TR> elements
  // from the external data source
  TableBody.Template := CreateTableBodyRow;

  // Connect external data source
  TableBody.DataSource := FNodeSource;
end;

procedure TMainForm.WindowCloseActionExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.WindowCloseActionUpdate(Sender: TObject);
begin
  WindowCloseAction.Enabled := True;
end;

end.
