
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is Main.pas.                                              }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit Main;

interface

{ NOTE: To successfully compile this example you will need to make sure that
  XVCL source directory is included into library search path.

  See Readme.htm or start the program for more information about this example. }

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OleCtrls, SHDocVw, ActnList, StdActns, ImgList, ExtCtrls, ComCtrls,
  ToolWin, JvxClasses, JvxHTMLClasses;

type
  TMainForm = class(TForm)
    ToolBar: TToolBar;
    WebBrowser: TWebBrowser;
    ToolbarImages: TImageList;
    ActionList: TActionList;
    ExecuteAction: TAction;
    ExecuteButton: TToolButton;
    CloseButton: TToolButton;
    WindowCloseAction: TAction;
    procedure ExecuteActionExecute(Sender: TObject);
    procedure ExecuteActionUpdate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WindowCloseActionExecute(Sender: TObject);
    procedure WindowCloseActionUpdate(Sender: TObject);
  private
    { Private declarations }
    FExeDirectory: string;
    FHTML: TJvxHTMLElement;
    procedure Execute;
    procedure GenerateHTML(Node: TJvxNode);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  JvxXMLClasses;

{$R *.DFM}

const
  ReadmeFileName = 'Readme.htm';
  OutputFileName = 'Output.htm';

procedure TMainForm.Execute;
begin
  // Generate HTML file from the template we have created earlier
  GenerateHTML(FHTML);
end;

procedure TMainForm.ExecuteActionExecute(Sender: TObject);
begin
  // Generate HTML and display the results in the browser
  Execute;
  WebBrowser.Navigate(FExeDirectory + OutputFileName);
end;

procedure TMainForm.ExecuteActionUpdate(Sender: TObject);
begin
  ExecuteAction.Enabled := Pos(ReadmeFileName, WebBrowser.LocationURL) <> 0;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  // Display Readme.htm in the browser
  WebBrowser.Navigate(FExeDirectory + ReadmeFileName);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Form: TJvxHTMLForm;
begin
  FExeDirectory := IncludeTrailingBackslash(ExtractFilePath(Application.ExeName));

  // Create object representing the root HTML element
  FHTML := TJvxHTMLElement.Create(Self);

  // Create form and insert it into HTML body
  Form := TJvxHTMLForm.Create(Self);
  Form.FormName.Value := 'MyForm';
  Form.Action.Value := 'http://MySite.com/MyApp.dll/MyAction?param1=1&param2=2';
  Form.Method.Value := fmGet;
  Form.Parent := FHTML.Body;

  // Create text input

  with TJvxHTMLDiv.Create(Self) do
  begin
    AsString := 'Sample text input:';
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itText;
    ControlName.Value := 'MyTextInput';
    Size.Value := 40;
    MaxLength.Value := 25;
    Value.AsString := 'TextInput';
    Parent := Form;
  end;

  // Create select

  with TJvxHTMLDiv.Create(Self) do
  begin
    AsString := 'Sample select:';
    Parent := Form;
  end;

  with TJvxHTMLSelect.Create(Self) do
  begin
    ControlName.Value := 'MySelect';
    Multiple.Value := True;
    Parent := Form;
    with Options.Add do
    begin
      Text.Add('First Option');
      Selected.Value := True;
      Value.AsString := '1';
    end;
    with Options.Add do
    begin
      Text.Add('Second Option');
      Value.AsString := '2';
    end;
  end;

  // Create checkbox

  with TJvxHTMLDiv.Create(Self) do
  begin
    AsString := 'Sample checkbox:';
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itCheckBox;
    ControlName.Value := 'MyCheckBox';
    Checked.Value := True;
    Text.Add('check box');
    Parent := Form;
  end;

  // Create radio buttons

  with TJvxHTMLDiv.Create(Self) do
  begin
    AsString := 'Sample radio buttons:';
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itRadio;
    ControlName.Value := 'MyRadio';
    Checked.Value := True;
    Text.Add('Radio 1');
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itRadio;
    ControlName.Value := 'MyRadio';
    Text.Add('Radio 2');
    Parent := Form;
  end;

  // Create submit and reset buttons

  with TJvxHTMLDiv.Create(Self) do
  begin
    AsString := 'Sample submit and reset buttons:';
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itSubmit;
    ControlName.Value := 'MySubmit';
    Value.AsString := 'Submit';
    Parent := Form;
  end;

  with TJvxHTMLInput.Create(Self) do
  begin
    InputType.Value := itReset;
    ControlName.Value := 'MyReset';
    Value.AsString := 'Reset';
    Parent := Form;
  end;
end;

procedure TMainForm.GenerateHTML(Node: TJvxNode);
var
  ContentGenerator: TJvxXMLGenerator;
  OutputFile: TextFile;
begin
  // Create XML generator
  ContentGenerator := TJvxXMLGenerator.Create;
  try
    // Open output file
    AssignFile(OutputFile, FExeDirectory + OutputFileName);
    Rewrite(OutputFile);
    try
      // Write generated content into the output file
      WriteLn(OutputFile, ContentGenerator.XMLFrom(Node));
    finally
      CloseFile(OutputFile);
    end;
  finally
    ContentGenerator.Free;
  end;
end;

procedure TMainForm.WindowCloseActionExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.WindowCloseActionUpdate(Sender: TObject);
begin
  WindowCloseAction.Enabled := True;
end;

end.
