
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxClasses.pas.                                        }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxClasses;

{$include Jvx.inc}

interface

uses
  SysUtils, Classes, Contnrs, TypInfo, JvxUtils;

type
  TJvxNode = class;
  TJvxElement = class;
  TJvxAttribute = class;
  TJvxVisitor = class;
  TJvxNodes = class;
  TJvxDataField = class;
  TJvxDataSource = class;
  EJvxError = class;

  TJvxNodeClass = class of TJvxNode;
  TJvxNodeOperation = (noInsert, noChange, noRemove);
  TJvxNodeNotifyEvent = procedure(Sender: TJvxNode;
    Operation: TJvxNodeOperation) of object;
  TJvxLoadDataEvent = procedure(Sender: TJvxNode; DataField: TJvxDataField;
    var Done: Boolean) of object;
  TJvxDefineNodeProc = procedure(var Reference; const Name: string;
    NodeClass: TJvxNodeClass; Data: Pointer);
  TJvxNodeState = set of (nsAggregatedElements, nsPredefinedNodes);

  { TJvxNodeNotifyEvents }

  TJvxNodeNotifyEvents = class(TJvxCustomEvents)
  private
    FOperation: TJvxNodeOperation;
  protected
    procedure InvokeEvent(Event: TNotifyEvent; var Abort: Boolean); override;
  public
    procedure Add(Event: TJvxNodeNotifyEvent);
    procedure Invoke(Sender: TJvxNode; Operation: TJvxNodeOperation);
    procedure Remove(Event: TJvxNodeNotifyEvent);
  end;

  { TJvxNode }

  TJvxNode = class(TComponent)
  private
    FDataField: TJvxDataField;
    FNodeName: string;
    FNodeState: TJvxNodeState;
    FOnLoadData: TJvxLoadDataEvent;
    FOnNotify: TJvxNodeNotifyEvent;
    FOnNotifyEvents: TJvxNodeNotifyEvents;
    FParent: TJvxNode;
    procedure AssignAttributes(Node: TJvxNode);
    procedure AssignElements(Node: TJvxNode);
    procedure CheckAssigned;
    function CompareAttributes(Node: TJvxNode): Boolean;
    function CompareElements(Node: TJvxNode): Boolean;
    procedure CreateAttributes;
    procedure CreateElements;
    procedure DataFieldChanged(Sender: TObject);
    function GetAttributes: TJvxNodes; virtual;
    function GetElements: TJvxNodes; virtual;
    function GetRoot: TJvxNode;
    function IsAssignedStored: Boolean;
    procedure SetDataField(Value: TJvxDataField);
    procedure SetOnLoadData(Value: TJvxLoadDataEvent);
  protected
    procedure AssignValue(Source: TJvxNode); virtual;
    function CompareValue(Source: TJvxNode): Boolean; virtual;
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); virtual;
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); virtual;
    procedure DefineProperties(Filer: TFiler); override;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); virtual;
    function GetAsBoolean: Boolean;
    function GetAsDateTime: TDateTime;
    function GetAsFloat: Extended;
    function GetAsInteger: Integer;
    function GetAssigned: Boolean; virtual;
    function GetAsString: string;
    function GetAsVariant: Variant;
    function GetChildOwner: TComponent; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function InternalGetAsBoolean: Boolean; virtual;
    function InternalGetAsDateTime: TDateTime; virtual;
    function InternalGetAsFloat: Extended; virtual;
    function InternalGetAsInteger: Integer; virtual;
    function InternalGetAsString: string; virtual;
    function InternalGetAsVariant: Variant; virtual;
    procedure InternalLoadData(DataField: TJvxDataField); virtual;
    procedure InternalSetAsBoolean(Value: Boolean); virtual;
    procedure InternalSetAsDateTime(Value: TDateTime); virtual;
    procedure InternalSetAsFloat(Value: Extended); virtual;
    procedure InternalSetAsInteger(Value: Integer); virtual;
    procedure InternalSetAssigned(Value: Boolean); virtual;
    procedure InternalSetAsString(const Value: string); virtual;
    procedure InternalSetAsVariant(const Value: Variant); virtual;
    procedure LoadData(DataField: TJvxDataField);
    procedure Notification(Component: TComponent; Operation: TOperation); override;
    procedure ReadValue(Reader: TReader); virtual;
    procedure SetAsBoolean(Value: Boolean);
    procedure SetAsDateTime(Value: TDateTime);
    procedure SetAsFloat(Value: Extended);
    procedure SetAsInteger(Value: Integer);
    procedure SetAssigned(Value: Boolean);
    procedure SetAsString(const Value: string);
    procedure SetAsVariant(const Value: Variant);
    procedure SetNodeName(const Value: string); virtual;
    procedure SetParent(Value: TJvxNode); virtual;
    procedure ValidateAttribute(Attribute: TJvxNode); virtual;
    procedure ValidateElement(Element: TJvxNode); virtual;
    procedure ValidateParent(Parent: TJvxNode); virtual;
    procedure WriteState(Writer: TWriter); override;
    procedure WriteValue(Writer: TWriter); virtual;

    property DataField: TJvxDataField read FDataField write SetDataField;
    property OnLoadData: TJvxLoadDataEvent read FOnLoadData write SetOnLoadData;
    property OnNotify: TJvxNodeNotifyEvent read FOnNotify write FOnNotify;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    procedure Accept(Visitor: TJvxVisitor); virtual; abstract;
    procedure Assign(Source: TPersistent); override;
    function Compare(Source: TPersistent): Boolean; virtual;
    function HasAttributes: Boolean; virtual;
    function HasElements: Boolean; virtual;
    function HasParent: Boolean; override;
    function IsChildOf(Node: TJvxNode): Boolean;
    function IsParentOf(Node: TJvxNode): Boolean;
    procedure Notify(Operation: TJvxNodeOperation); virtual;
    procedure SetParentComponent(Value: TComponent); override;

    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsFloat: Extended read GetAsFloat write SetAsFloat;
    property AsInteger: Integer read GetAsInteger write SetAsInteger;
    property Assigned: Boolean read GetAssigned write SetAssigned stored IsAssignedStored;
    property AsString: string read GetAsString write SetAsString;
    property AsVariant: Variant read GetAsVariant write SetAsVariant;
    property Attributes: TJvxNodes read GetAttributes;
    property Elements: TJvxNodes read GetElements;
    property NodeName: string read FNodeName;
    property OnNotifyEvents: TJvxNodeNotifyEvents read FOnNotifyEvents;
    property Parent: TJvxNode read FParent write SetParent;
    property Root: TJvxNode read GetRoot;
  end;

  { TJvxExposedNode }

  TJvxExposedNode = class(TJvxNode)
  public
    property NodeName write SetNodeName;
  end;

  { TJvxNodes }

  TJvxGetNodeEvent = function: TJvxNode of object;
  TJvxGetDataSourceEvent = function: TJvxDataSource of object;
  TJvxValidateNodeEvent = procedure(Node: TJvxNode) of object;
  TJvxNodes = class(TComponent)
  private
    FOnGetDataSource: TJvxGetDataSourceEvent;
    FOnGetTemplate: TJvxGetNodeEvent;
    FOnValidateNode: TJvxValidateNodeEvent;
    FParent: TJvxNode;
    function GetDataSource: TJvxDataSource;
    function GetTemplate: TJvxNode;
  protected
    function GetChildOwner: TComponent; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function GetCount: Integer;
    function GetItem(Index: Integer): TJvxNode;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    procedure ValidateNode(Node: TJvxNode);

    property DataSource: TJvxDataSource read GetDataSource;
    property OnGetDataSource: TJvxGetDataSourceEvent read FOnGetDataSource write FOnGetDataSource;
    property OnGetTemplate: TJvxGetNodeEvent read FOnGetTemplate write FOnGetTemplate;
    property OnValidateNode: TJvxValidateNodeEvent read FOnValidateNode write FOnValidateNode;
    property Parent: TJvxNode read FParent write FParent;
    property Template: TJvxNode read GetTemplate;
  public
    destructor Destroy; override;

    function Add(Node: TJvxNode): Integer;
    procedure Accept(Visitor: TJvxVisitor);
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    function Compare(Source: TPersistent): Boolean;
    function Delete(Index: Integer): TJvxNode;
    function FindNode(NodeName: string): TJvxNode;
    function GetParentComponent: TComponent; override;
    function HasParent: Boolean; override;
    function IndexOf(Node: TJvxNode): Integer;
    function Insert(Index: Integer; Node: TJvxNode): Integer;
    procedure Move(FromIndex, ToIndex: Integer);
    function Remove(Node: TJvxNode): Integer;
    procedure SetParentComponent(Value: TComponent); override;

    property Count: Integer read GetCount;
    property Items[Index: Integer]: TJvxNode read GetItem; default;
  end;

  { TODO: Test TJvxNodeCollection }

  TJvxNodeCollection = class(TPersistent)
  private
    function GetTemplate: TJvxNode;
  private
    FNodes: TJvxNodes;
    function GetCount: Integer;
    function GetItem(Index: Integer): TJvxNode;
    property Template: TJvxNode read GetTemplate;
  public
    constructor Create(Nodes: TJvxNodes); virtual;

    function Add: TJvxNode;
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    procedure Delete(Index: Integer);
    function IndexOf(Node: TJvxNode): Integer;
    function Insert(Index: Integer): TJvxNode;
    procedure Move(FromIndex, ToIndex: Integer);
    function Remove(Node: TJvxNode): Integer;

    property Count: Integer read GetCount;
    property Items[Index: Integer]: TJvxNode read GetItem; default;
  end;

  { TJvxAttribute }

  TJvxAttribute = class(TJvxNode)
  protected
    procedure SetParent(Value: TJvxNode); override;
  public
    procedure Accept(Visitor: TJvxVisitor); override;
    function GetParentComponent: TComponent; override;
  end;

  { TJvxElement }

  TJvxElement = class(TJvxNode)
  private
    FAttributes: TJvxNodes;
    FAttributeSource: TJvxDataSource;
    function GetAttributes: TJvxNodes; override;
    function GetAttributeSource: TJvxDataSource;
    procedure SetAttributes(Value: TJvxNodes);
  protected
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    procedure Notification(Component: TComponent;
      Operation: TOperation); override;
    procedure SetParent(Value: TJvxNode); override;
    procedure ValidateRename(Component: TComponent;
      const CurName, NewName: string); override;
  public
    destructor Destroy; override;
    procedure Accept(Visitor: TJvxVisitor); override;
    function GetParentComponent: TComponent; override;
    function HasAttributes: Boolean; override;
  end;

  { TJvxCompositeElement }

  TJvxCompositeElement = class(TJvxElement)
  private
    FElements: TJvxNodes;
    FElementSource: TJvxDataSource;
    FTemplate: TJvxElement;
    function GetElements: TJvxNodes; override;
    function GetElementSource: TJvxDataSource;
    function GetTemplate: TJvxNode;
    procedure SetElements(Value: TJvxNodes);
  protected
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    procedure Notification(Component: TComponent;
      Operation: TOperation); override;
    procedure SetElementSource(Value: TJvxDataSource);
    procedure SetTemplate(Value: TJvxElement);
    procedure ValidateRename(Component: TComponent;
      const CurName, NewName: string); override;
    property DataSource: TJvxDataSource read GetElementSource write SetElementSource;
    property Template: TJvxElement read FTemplate;
  public
    destructor Destroy; override;
    function HasElements: Boolean; override;
  end;

  { TJvxInstruction }

  TJvxInstruction = class(TJvxElement)
  public
    procedure Accept(Visitor: TJvxVisitor); override;
  end;

  { TJvxVisitor }

  TJvxVisitor = class(TObject)
  public
    procedure Visit(Attribute: TJvxAttribute); overload; virtual;
    procedure Visit(Element: TJvxElement); overload; virtual;
    procedure Visit(Instruction: TJvxInstruction); overload; virtual;
  end;

  { TJvxDataField }

  TJvxDataField = class(TComponent)
  private
    FFieldName: string;
    FDataSource: TJvxDataSource;
    FOnChangeEvents: TJvxNotifyEvents;
    FOnChange: TNotifyEvent;
    procedure CheckAssigned;
    procedure CheckDataType(ValidTypes: TTypeKinds; AccessType: PTypeInfo);
    procedure SetFieldName(const Value: string);
    procedure SetDataSource(Value: TJvxDataSource);
  protected
    procedure Changed;
    function GetAsFloat: Extended;
    function GetAsOrdinal: TJvxOrdinal;
    function GetAssigned: Boolean;
    function GetAsString: string;
    function GetAsVariant: Variant;
    function GetTypeInfo: PTypeInfo; virtual; abstract;
    function InternalGetAsFloat: Extended; virtual;
    function InternalGetAsOrdinal: TJvxOrdinal; virtual;
    function InternalGetAssigned: Boolean; virtual; abstract;
    function InternalGetAsString: string; virtual;
    function InternalGetAsVariant: Variant; virtual;
    property DataSource: TJvxDataSource read FDataSource write SetDataSource;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    property AsFloat: Extended read GetAsFloat;
    property AsOrdinal: TJvxOrdinal read GetAsOrdinal;
    property Assigned: Boolean read GetAssigned;
    property AsString: string read GetAsString;
    property AsVariant: Variant read GetAsVariant;
    property FieldName: string read FFieldName write SetFieldName;
    property OnChangeEvents: TJvxNotifyEvents read FOnChangeEvents;
    property TypeInfo: PTypeInfo read GetTypeInfo;
  end;

  { TJvxDataFields }

  TJvxDataFields = class(TObject)
  private
    FItems: TList;
    FDataSource: TJvxDataSource;
    procedure CheckFieldName(FieldName: string);
    function GetCount: Integer;
    function GetItem(Index: Integer): TJvxDataField;
    procedure ValidateItem(Item: TJvxDataField);
  public
    constructor Create(DataSource: TJvxDataSource);
    destructor Destroy; override;
    function Add(Item: TJvxDataField): Integer;
    procedure Clear;
    function Delete(Index: Integer): TJvxDataField;
    function FieldByName(FieldName: string): TJvxDataField;
    function FindField(FieldName: string): TJvxDataField;
    function IndexOf(Item: TJvxDataField): Integer;
    procedure Insert(Index: Integer; Item: TJvxDataField);
    function Remove(Item: TJvxDataField): Integer;
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TJvxDataField read GetItem; default;
    property DataSource: TJvxDataSource read FDataSource;
  end;

  { TJvxDataSource }

  TJvxDataSource = class(TComponent)
  private
    FActive: Boolean;
    FDefaultFields: Boolean;
    FFields: TJvxDataFields;
    procedure CheckActive;
    procedure DestroyDefaultFields; 
    function GetFields: TJvxDataFields;
    procedure SetActive(Value: Boolean);
  protected
    procedure CreateDefaultFields; virtual;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); virtual;
    procedure InternalClose; virtual;
    function InternalCount: Integer; virtual;
    procedure InternalDelete(Index: Integer); virtual;
    function InternalGetNode(Index: Integer;
      Prototype: TJvxNode): TJvxNode; virtual;
    function InternalIndexOf(Node: TJvxNode): Integer; virtual;
    procedure InternalInsert(Index: Integer; Node: TJvxNode); virtual;
    procedure InternalOpen; virtual;
    procedure ValidateField(Field: TJvxDataField); virtual;
  public
    destructor Destroy; override;
    procedure Close;
    function Count: Integer;
    procedure Delete(Index: Integer);
    function GetNode(Index: Integer; Prototype: TJvxNode): TJvxNode;
    function IndexOf(Node: TJvxNode): Integer;
    procedure Insert(Index: Integer; Node: TJvxNode);
    procedure Open;
    property Active: Boolean read FActive write SetActive;
    property Fields: TJvxDataFields read GetFields;
  end;

  { TJvxListDataSource }

  TJvxListDataSource = class(TJvxDataSource)
  private
    FItems: TComponentList;
  protected
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    procedure InternalClose; override;
    function InternalCount: Integer; override;
    procedure InternalDelete(Index: Integer); override;
    function InternalGetNode(Index: Integer;
      Prototype: TJvxNode): TJvxNode; override;
    function InternalIndexOf(Node: TJvxNode): Integer; override;
    procedure InternalInsert(Index: Integer; Node: TJvxNode); override;
    procedure InternalOpen; override;
  public
    constructor Create(Onwer: TComponent); override;
    destructor Destroy; override;
  end;

  { Exceptions }

  EJvxError = class(Exception);

function JvxAccessError(TypeInfo: PTypeInfo): EInvalidOperation; overload;
function JvxAccessError(Format: string; Args: array of const): EInvalidOperation; overload;
function JvxAccessError(Message: string): EInvalidOperation; overload;
function JvxValueError(const Value: string): EJvxError; overload;
function JvxValueError(Value: Integer): EJvxError; overload;

implementation

uses
  Consts, SysConst, {$ifndef VER130} Variants, {$endif} Math;

type

  { TEmptyNodesHelper }

  TEmptyNodesHelper = class(TObject)
  private
    FEmptyNodes: TJvxNodes;
    FEmptyDataSource: TJvxDataSource;
    function GetDataSource: TJvxDataSource;
  public
    constructor Create;
    destructor Destroy; override;
    property EmptyNodes: TJvxNodes read FEmptyNodes;
  end;

  { TWriteNodeStateHelper }

  TWriteNodeStateHelper = class(TObject)
  private
    FAncestor: TJvxNode;
    FNode: TJvxNode;
    FOnFindAncestor: TFindAncestorEvent;
    FWriter: TWriter;
    procedure FindAncestor(Writer: TWriter; Component: TComponent;
      const Name: string; var Ancestor, RootAncestor: TComponent);
  public
    constructor Create(Node: TJvxNode; Writer: TWriter);
    destructor Destroy; override;
  end;

  { TJvxCreateNodeData }

  TJvxGetNodesMethod = function: TJvxNodes of object;
  TJvxCreateNodeData = record
    GetNodes: TJvxGetNodesMethod;
  end;

  { TJvxValidateNodeData }
  
  TJvxValidateNodeData = record
    Node: TJvxNode;
    NodeIsValid: Boolean;
  end;

procedure JvxCreateNode(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer); forward;
procedure JvxClearNodeReference(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer); forward;
procedure JvxValidateNode(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer); forward;

var
  EmptyNodesHelper: TEmptyNodesHelper = nil;

const
  SAttributes = 'Attributes';
  SElements = 'Elements';

resourcestring
  SAccessError = 'Cannot access value as type %s';
  SAssignedCanOnlyBeSetToFalse = 'Assigned can only be set to false';
  SDataSourceInactive = 'Cannot perform this action on inactive data source';
  SDataSourceCannotBeClosed = 'Data source cannot be closed';
  SDataSourceCannotBeOpened = 'Data source cannot be opened';
  SDuplicateDataField = 'Data field %s already exists';
  SEmptyFieldName = 'Field name cannot be empty';
  SInvalidAttribute = 'Invalid attribute: %s';
  SInvalidDataField = 'Invalid data field';
  SInvalidElement = 'Invalid element: %s';
  SInvalidNode = 'Node cannot be nil';
  SInvalidParent = 'Invalid parent';
  SInvalidTemplateChange = 'Cannot change template when child nodes exist';
  SInvalidValue = 'Invalid value: %s';
  SDataFieldNotFound = 'Data field %s not found';
  SNodeCannotHaveChildNodes = 'Node cannot have child nodes';
  SNodeDoesntMatchTemplate = 'Node is not compatible with the template';
  SNodeValueCannotBeAssigned = 'Value of node cannot be assigned';
  SUnassigned = 'Unassigned';
  SUnassignedValue = 'Cannot access unassigned value';
  SUnsupportedFieldType = 'Unsupported field type %s';

function JvxAccessError(TypeInfo: PTypeInfo): EInvalidOperation; overload;
begin
  Result := EInvalidOperation.CreateFmt(SAccessError, [TypeInfo.Name]);
end;

function JvxAccessError(Format: string; Args: array of const): EInvalidOperation; overload;
begin
  Result := EInvalidOperation.CreateFmt(Format, Args);
end;

function JvxAccessError(Message: string): EInvalidOperation; overload;
begin
  Result := EInvalidOperation.Create(Message);
end;

function JvxValueError(const Value: string): EJvxError;
begin
  Result := EJvxError.CreateFmt(SInvalidValue, [Value]);
end;

function JvxValueError(Value: Integer): EJvxError;
begin
  Result := JvxValueError(IntToStr(Value));
end;

{ TJvxNodeNotifyEvents }

procedure TJvxNodeNotifyEvents.Add(Event: TJvxNodeNotifyEvent);
begin
  inherited Add(TNotifyEvent(Event));
end;

procedure TJvxNodeNotifyEvents.Invoke(Sender: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FOperation := Operation;
  inherited Invoke(Sender);
end;

procedure TJvxNodeNotifyEvents.InvokeEvent(Event: TNotifyEvent;
  var Abort: Boolean);
begin
  TJvxNodeNotifyEvent(Event)(TJvxNode(Sender), FOperation);
end;

procedure TJvxNodeNotifyEvents.Remove(Event: TJvxNodeNotifyEvent);
begin
  inherited Remove(TNotifyEvent(Event));
end;

{ TJvxNode }

procedure TJvxNode.Assign(Source: TPersistent);
var
  Node: TJvxNode absolute Source;
begin
  if Source is TJvxNode then
  begin
    SetName(Node.Name);
    SetNodeName(Node.NodeName);
    if Node.Assigned then
      AssignValue(Node);
    Assigned := Node.Assigned;
    AssignAttributes(Node);
    AssignElements(Node);
  end
  else inherited;
end;

procedure TJvxNode.AssignAttributes(Node: TJvxNode);
begin
  if HasAttributes then
    Attributes.Clear;
  if Node.HasAttributes then
    Attributes.Assign(Node.Attributes);
end;

procedure TJvxNode.AssignElements(Node: TJvxNode);
begin
  if HasElements then
    Elements.Clear;
  if Node.HasElements then
    Elements.Assign(Node.Elements);
end;

procedure TJvxNode.AssignValue(Source: TJvxNode);
begin
  raise EAbstractError.Create(SAbstractError);
end;

procedure TJvxNode.CheckAssigned;
begin
  if not Assigned then
     raise JvxAccessError(SUnassignedValue);
end;

function TJvxNode.Compare(Source: TPersistent): Boolean;
var
  Node: TJvxNode absolute Source;
begin
  Result := False;
  if Source is TJvxNode then
  begin
    if Name <> Node.Name then Exit;
    if NodeName <> Node.NodeName then Exit;
    if Assigned <> Node.Assigned then Exit;
    if Assigned and not CompareValue(Node) then Exit;
    if not CompareAttributes(Node) then Exit;
    if not CompareElements(Node) then Exit;
    Result := True;
  end;
end;

function TJvxNode.CompareAttributes(Node: TJvxNode): Boolean;
begin
  Result := False;
  if HasAttributes <> Node.HasAttributes then Exit;
  if HasAttributes and not Attributes.Compare(Node.Attributes) then Exit;
  Result := True;
end;

function TJvxNode.CompareElements(Node: TJvxNode): Boolean;
begin
  Result := False;
  if HasElements <> Node.HasElements then Exit;
  if HasElements and not Elements.Compare(Node.Elements) then Exit;
  Result := True;
end;

function TJvxNode.CompareValue(Source: TJvxNode): Boolean;
begin
  raise EAbstractError.Create(SAbstractError);
end;

constructor TJvxNode.Create(Owner: TComponent);
begin
  inherited;
  FOnNotifyEvents := TJvxNodeNotifyEvents.Create(@@FOnNotify);
  CreateAttributes;
  CreateElements;
end;

procedure TJvxNode.CreateAttributes;
var
  Data: TJvxCreateNodeData;
begin
  Data.GetNodes := GetAttributes;
  DefineAttributes(JvxCreateNode, @Data);
end;

procedure TJvxNode.CreateElements;
var
  Data: TJvxCreateNodeData;
begin
  Data.GetNodes := GetElements;
  DefineElements(JvxCreateNode, @Data);
end;

procedure TJvxNode.DataFieldChanged(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
  begin
    if (FDataField <> nil) and FDataField.Assigned then
      LoadData(FDataField)
    else
      Assigned := False;
  end;
end;

procedure TJvxNode.DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
  Data: Pointer);
begin
end;

procedure TJvxNode.DefineElements(DefineElement: TJvxDefineNodeProc;
  Data: Pointer);
begin
end;

procedure TJvxNode.DefineProperties(Filer: TFiler);

  function HasValue: Boolean;
  var
    Ancestor: TJvxNode;
  begin
    Result := Assigned;
    Ancestor := Filer.Ancestor as TJvxNode;
    if (Ancestor <> nil) and Ancestor.Assigned then
      Result := Result and not CompareValue(Ancestor)
  end;

begin
  inherited;
  Filer.DefineProperty('Value', ReadValue, WriteValue, HasValue);
end;

destructor TJvxNode.Destroy;
begin
  Parent := nil;
  DataField := nil;
  FOnNotifyEvents.Free;
  FOnNotifyEvents := nil;
  inherited;
end;

procedure TJvxNode.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
end;

function TJvxNode.GetAsBoolean: Boolean;
begin
  CheckAssigned;
  Result := InternalGetAsBoolean;
end;

function TJvxNode.GetAsDateTime: TDateTime;
begin
  CheckAssigned;
  Result := InternalGetAsDateTime;
end;

function TJvxNode.GetAsFloat: Extended;
begin
  CheckAssigned;
  Result := InternalGetAsFloat;
end;

function TJvxNode.GetAsInteger: Integer;
begin
  CheckAssigned;
  Result := InternalGetAsInteger;
end;

function TJvxNode.GetAssigned: Boolean;
begin
  Result := False;
end;

function TJvxNode.GetAsString: string;
begin
  CheckAssigned;
  Result := InternalGetAsString;
end;

function TJvxNode.GetAsVariant: Variant;
begin
  CheckAssigned;
  Result := InternalGetAsVariant;
end;

function TJvxNode.GetAttributes: TJvxNodes;
begin
  Result := EmptyNodesHelper.EmptyNodes;
end;

function TJvxNode.GetChildOwner: TComponent;
begin
  Result := Self;
end;

procedure TJvxNode.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
  inherited;
  if HasAttributes then
    Proc(Attributes);
  if (nsAggregatedElements in FNodeState) and HasElements then
    Proc(Elements);
end;

function TJvxNode.GetElements: TJvxNodes;
begin
  Result := EmptyNodesHelper.EmptyNodes;
end;

function TJvxNode.GetRoot: TJvxNode;
begin
  Result := Self;
  while Result.Parent <> nil do
    Result := Result.Parent;
end;

function TJvxNode.HasAttributes: Boolean;
begin
  Result := False;
end;

function TJvxNode.HasElements: Boolean;
begin
  Result := False;
end;

function TJvxNode.HasParent: Boolean;
begin
  Result := Parent <> nil;
end;

function TJvxNode.InternalGetAsBoolean: Boolean;
begin
  raise JvxAccessError(TypeInfo(Boolean));
end;

function TJvxNode.InternalGetAsDateTime: TDateTime;
begin
  raise JvxAccessError(TypeInfo(TDateTime));
end;

function TJvxNode.InternalGetAsFloat: Extended;
begin
  raise JvxAccessError(TypeInfo(Extended));
end;

function TJvxNode.InternalGetAsInteger: Integer;
begin
  raise JvxAccessError(TypeInfo(Integer));
end;

function TJvxNode.InternalGetAsString: string;
begin
  raise JvxAccessError(TypeInfo(string));
end;

function TJvxNode.InternalGetAsVariant: Variant;
begin
  raise JvxAccessError(TypeInfo(Variant));
end;

procedure TJvxNode.InternalLoadData(DataField: TJvxDataField);
begin
  case DataField.TypeInfo.Kind of
    tkInteger:
      AsInteger := DataField.AsOrdinal;

    tkChar, tkWChar:
      AsString := Chr(DataField.AsOrdinal);

    tkEnumeration:
      AsString := GetEnumName(DataField.TypeInfo, DataField.AsOrdinal);

    tkFloat:
      if DataField.TypeInfo = System.TypeInfo(TDateTime) then
        AsDateTime := DataField.AsFloat
      else
        AsFloat := DataField.AsFloat;

    tkString, tkLString, tkWString:
      AsString := DataField.AsString;

    tkVariant:
      AsVariant := DataField.AsVariant;
  else
    raise EInvalidOperation.CreateFmt(SUnsupportedFieldType,
      [GetEnumName(TypeInfo(TTypeKind), Ord(DataField.TypeInfo.Kind))]);
  end;
end;

procedure TJvxNode.InternalSetAsBoolean(Value: Boolean);
begin
  raise JvxAccessError(TypeInfo(Boolean));
end;

procedure TJvxNode.InternalSetAsDateTime(Value: TDateTime);
begin
  raise JvxAccessError(TypeInfo(TDateTime));
end;

procedure TJvxNode.InternalSetAsFloat(Value: Extended);
begin
  raise JvxAccessError(TypeInfo(Extended));
end;

procedure TJvxNode.InternalSetAsInteger(Value: Integer);
begin
  raise JvxAccessError(TypeInfo(Integer));
end;

procedure TJvxNode.InternalSetAssigned(Value: Boolean);
begin
  raise JvxAccessError(SNodeValueCannotBeAssigned);
end;

procedure TJvxNode.InternalSetAsString(const Value: string);
begin
  raise JvxAccessError(TypeInfo(string));
end;

procedure TJvxNode.InternalSetAsVariant(const Value: Variant);
begin
  raise JvxAccessError(TypeInfo(Variant));
end;

function TJvxNode.IsAssignedStored: Boolean;
begin
  Result := not Assigned;
end;

function TJvxNode.IsChildOf(Node: TJvxNode): Boolean;
begin
  Result := Node.IsParentOf(Self);
end;

function TJvxNode.IsParentOf(Node: TJvxNode): Boolean;
begin
  while Node <> nil do
  begin
    Node := Node.Parent;
    if Node = Self then
    begin
      Result := True;
      Exit;
    end;
  end;
  Result := False;
end;

procedure TJvxNode.LoadData(DataField: TJvxDataField);
var
  DataLoaded: Boolean;
begin
  if System.Assigned(FOnLoadData) then
  begin
    // todo -cdesign -oosych: decide whether node should be cleared before/after calling FOnLoadData
    DataLoaded := False;
    FOnLoadData(Self, DataField, DataLoaded);
    if DataLoaded then Exit;
  end;
  InternalLoadData(DataField);
end;

procedure TJvxNode.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if Component = FDataField then
      SetDataField(nil)
    else
    begin
      DefineAttributes(JvxClearNodeReference, Component);
      DefineElements(JvxClearNodeReference, Component);
    end;
  end;
end;

procedure TJvxNode.Notify(Operation: TJvxNodeOperation);
var
  Node: TJvxNode;
begin
  if Parent = nil then
    Node := Self
  else
    Node := Parent;
  if not (csDestroying in Node.ComponentState) then
  begin
    Node.DocumentNotification(Self, Operation);
    OnNotifyEvents.Invoke(Self, Operation);
  end;
end;

procedure TJvxNode.ReadValue(Reader: TReader);
begin
  raise EAbstractError.Create(SAbstractError);
end;

procedure TJvxNode.SetAsBoolean(Value: Boolean);

  function ValueIsDifferent: Boolean;
  begin
    try
      Result := (not Assigned) or (AsBoolean <> Value);
    except
      on EConvertError do
        Result := True;
    end;
  end;

begin
  if ValueIsDifferent then
  begin
    InternalSetAsBoolean(Value);
    InternalSetAssigned(True);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAsDateTime(Value: TDateTime);

  function ValueIsDifferent: Boolean;
  begin
    try
      Result := (not Assigned) or (AsDateTime <> Value);
    except
      on EConvertError do
        Result := True;
    end;
  end;

begin
  if ValueIsDifferent then
  begin
    InternalSetAsDateTime(Value);
    InternalSetAssigned(True);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAsFloat(Value: Extended);

  function ValueIsDifferent: Boolean;
  begin
    try
      Result := (not Assigned) or (AsFloat <> Value);
    except
      on EConvertError do
        Result := True;
    end;
  end;

begin
  if ValueIsDifferent then
  begin
    InternalSetAsFloat(Value);
    InternalSetAssigned(True);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAsInteger(Value: Integer);

  function ValueIsDifferent: Boolean;
  begin
    try
      Result := (not Assigned) or (AsInteger <> Value);
    except
      on EConvertError do
        Result := True;
    end;
  end;

begin
  if ValueIsDifferent then
  begin
    InternalSetAsInteger(Value);
    InternalSetAssigned(True);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAssigned(Value: Boolean);
begin
  if Assigned <> Value then
  begin
    if Value = True then
      raise JvxAccessError(SAssignedCanOnlyBeSetToFalse);
    InternalSetAssigned(Value);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAsString(const Value: string);

  function ValueIsAssigned: Boolean;
  begin
    Result := Value <> EmptyStr;
  end;

  function ValueIsDifferent: Boolean;
  begin
    Result := (Assigned <> ValueIsAssigned) or (Assigned and (AsString <> Value));
  end;

begin
  if ValueIsDifferent then
  begin
    if ValueIsAssigned then
      InternalSetAsString(Value);
    InternalSetAssigned(ValueIsAssigned);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetAsVariant(const Value: Variant);

  function ValueIsAssigned: Boolean;
  begin
    Result := not VarIsNull(Value);
  end;

  function ValueIsDifferent: Boolean;
  begin
    Result := (Assigned <> ValueIsAssigned) or
      (Assigned and (VarType(AsVariant) <> VarType(Value))) or
      (Assigned and (AsVariant <> Value));
  end;

begin
  if VarIsEmpty(Value) then
    raise JvxValueError(SUnassigned);

  if ValueIsDifferent then
  begin
    if ValueIsAssigned then
      case VarType(Value) and varTypeMask of
        varSmallint, varByte, varInteger, varError:
          InternalSetAsInteger(Value);
        varSingle, varDouble, varCurrency:
          InternalSetAsFloat(Value);
        varDate:
          InternalSetAsDateTime(Value);
        varOleStr, varStrArg, varString:
          InternalSetAsString(Value);
        varBoolean:
          InternalSetAsBoolean(Value);
      else
        InternalSetAsVariant(Value);
      end;
    InternalSetAssigned(ValueIsAssigned);
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetDataField(Value: TJvxDataField);
begin
  if FDataField <> Value then
  begin
    if FDataField <> nil then
    begin
      FDataField.RemoveFreeNotification(Self);
      FDataField.OnChangeEvents.Remove(DataFieldChanged);
    end;
    FDataField := Value;
    if FDataField <> nil then
    begin
      FDataField.FreeNotification(Self);
      FDataField.OnChangeEvents.Add(DataFieldChanged);
    end;
    DataFieldChanged(FDataField);
  end;
end;

procedure TJvxNode.SetNodeName(const Value: string);
begin
  if FNodeName <> Value then
  begin
    FNodeName := Value;
    Notify(noChange);
  end;
end;

procedure TJvxNode.SetOnLoadData(Value: TJvxLoadDataEvent);
begin
  if not JvxCompareMem(FOnLoadData, Value, SizeOf(TMethod)) then
  begin
    FOnLoadData := Value;
    DataFieldChanged(FDataField);
  end;
end;

procedure TJvxNode.SetParent(Value: TJvxNode);
begin
  FParent := Value;
end;

procedure TJvxNode.SetParentComponent(Value: TComponent);
begin
  if Value is TJvxNodes then
    TJvxNodes(Value).Add(Self);
end;

procedure TJvxNode.ValidateAttribute(Attribute: TJvxNode);
var
  Data: TJvxValidateNodeData;
begin
  Data.Node := Attribute;
  Data.NodeIsValid := False;
  DefineAttributes(JvxValidateNode, @Data);
  if not Data.NodeIsValid then
    raise EInvalidOperation.CreateFmt(SInvalidAttribute, [Attribute.ClassName]);
end;

procedure TJvxNode.ValidateElement(Element: TJvxNode);
var
  Data: TJvxValidateNodeData;
begin
  Data.Node := Element;
  Data.NodeIsValid := False;
  DefineElements(JvxValidateNode, @Data);
  if not Data.NodeIsValid then
    raise EInvalidOperation.CreateFmt(SInvalidElement, [Element.ClassName]);
end;

procedure TJvxNode.ValidateParent(Parent: TJvxNode);
begin
  if (Parent = Self) or Parent.IsChildOf(Self) then
    raise EInvalidOperation.Create(SInvalidParent);
end;

procedure TJvxNode.WriteState(Writer: TWriter);
var
  Helper: TWriteNodeStateHelper;
begin
  if (Writer.Ancestor = nil) and (nsPredefinedNodes in FNodeState) then
  begin
    Helper := TWriteNodeStateHelper.Create(Self, Writer);
    try
      inherited;
    finally
      Helper.Free;
    end;
  end
  else inherited;
end;

procedure TJvxNode.WriteValue(Writer: TWriter);
begin
  raise EAbstractError.Create(SAbstractError);
end;

{ TJvxNodes }

procedure TJvxNodes.Accept(Visitor: TJvxVisitor);
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    Items[I].Accept(Visitor);
end;

function TJvxNodes.Add(Node: TJvxNode): Integer;
begin
  Result := Insert(Count, Node);
end;

procedure TJvxNodes.Assign(Source: TPersistent);
var
  I: Integer;
  Node: TJvxNode;
  Nodes: TJvxNodes absolute Source;
begin
  if Source is TJvxNodes then
  begin
    Clear;
    for I := 0 to Nodes.Count - 1 do
    begin
      Node := JvxClone(Nodes[I], GetChildOwner) as TJvxNode;
      try
        Add(Node);
      except
        Node.Free;
        raise;
      end;
    end;
  end
  else inherited;
end;

procedure TJvxNodes.Clear;
var
  I: Integer;
  Node: TJvxNode;
begin
  for I := Count - 1 downto 0 do
  begin
    Node := Delete(I);
    if Node <> Template then
      Node.Free;
  end;
end;

function TJvxNodes.Compare(Source: TPersistent): Boolean;
var
  I: Integer;
  Nodes: TJvxNodes absolute Source;
begin
  Result := False;
  if Source is TJvxNodes then
  begin
    if Count <> Nodes.Count then Exit;
    for I := 0 to Count - 1 do
      if not Items[I].Compare(Nodes.Items[I]) then Exit;
    Result := True;
  end;
end;

function TJvxNodes.Delete(Index: Integer): TJvxNode;
begin
  Result := Items[Index];
  Result.Notify(noRemove);
  DataSource.Delete(Index);
  Result.FParent := nil;
end;

destructor TJvxNodes.Destroy;
begin
  Clear;
  inherited;
end;

function TJvxNodes.FindNode(NodeName: string): TJvxNode;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Result := Items[I];
    if Result.NodeName = NodeName then
      Exit;
  end;
  Result := nil;
end;

function TJvxNodes.GetChildOwner: TComponent;
begin
  Result := FParent;
end;

procedure TJvxNodes.GetChildren(Proc: TGetChildProc;
  Root: TComponent);
var
  I: Integer;
begin
  inherited;
  for I := 0 to Count - 1 do
    Proc(Items[I]);
end;

function TJvxNodes.GetCount: Integer;
begin
  Result := DataSource.Count;
end;

function TJvxNodes.GetItem(Index: Integer): TJvxNode;
begin
  Result := DataSource.GetNode(Index, Template);
end;

function TJvxNodes.GetDataSource: TJvxDataSource;
begin
  Result := FOnGetDataSource;
end;

function TJvxNodes.GetParentComponent: TComponent;
begin
  Result := Parent;
end;

function TJvxNodes.GetTemplate: TJvxNode;
begin
  if Assigned(FOnGetTemplate) then
    Result := FOnGetTemplate
  else
    Result := nil;
end;

function TJvxNodes.HasParent: Boolean;
begin
  Result := Parent <> nil;
end;

function TJvxNodes.IndexOf(Node: TJvxNode): Integer;
begin
  Result := DataSource.IndexOf(Node);
end;

function TJvxNodes.Insert(Index: Integer; Node: TJvxNode): Integer;
begin
  Result := IndexOf(Node);
  if Result < 0 then
  begin
    ValidateNode(Node);
    DataSource.Insert(Index, Node);
    Node.FParent := FParent;
    Node.Notify(noInsert);
    Result := Index;
  end;
end;

procedure TJvxNodes.Move(FromIndex, ToIndex: Integer);
var
  Node: TJvxNode;
begin
  if FromIndex <> ToIndex then
  begin
    if (ToIndex < 0) or (ToIndex >= Count) then
      raise EListError.CreateFmt(Consts.SListIndexError, [ToIndex]);
    Node := Items[FromIndex];
    Delete(FromIndex);
    Insert(ToIndex, Node);
  end;
end;

function TJvxNodes.Remove(Node: TJvxNode): Integer;
begin
  Result := IndexOf(Node);
  if Result >= 0 then
    Delete(Result);
end;

// todo: test TJvxNodes.SetChildOrder
procedure TJvxNodes.SetChildOrder(Child: TComponent; Order: Integer);
begin
  Move(IndexOf(Child as TJvxNode), Order);
end;

procedure TJvxNodes.SetParentComponent(Value: TComponent);
begin
  Parent := Value as TJvxNode;
end;

procedure TJvxNodes.ValidateNode(Node: TJvxNode);
begin
  if Node = nil then
    raise EInvalidOperation.Create(SInvalidNode);
  if Assigned(FOnValidateNode) then
    FOnValidateNode(Node);
  if FParent <> nil then
    Node.ValidateParent(FParent);
  if (Template <> nil) and not (Node is Template.ClassType) then
    raise EInvalidOperation.Create(SNodeDoesntMatchTemplate);
end;

{ TJvxNodeCollection }

function TJvxNodeCollection.Add: TJvxNode;
begin
  Result := Insert(Count);
end;

procedure TJvxNodeCollection.Assign(Source: TPersistent);
begin
  FNodes.Assign(Source);
end;

procedure TJvxNodeCollection.Clear;
begin
  FNodes.Clear;
end;

constructor TJvxNodeCollection.Create(Nodes: TJvxNodes);
begin
  inherited Create;
  FNodes := Nodes;
end;                     

procedure TJvxNodeCollection.Delete(Index: Integer);
begin
  FNodes.Delete(Index);
end;

function TJvxNodeCollection.GetCount: Integer;
begin
  Result := FNodes.Count;
end;

function TJvxNodeCollection.GetItem(Index: Integer): TJvxNode;
begin
  Result := FNodes.Items[Index];
end;

function TJvxNodeCollection.GetTemplate: TJvxNode;
begin
  Result := FNodes.Template;
end;

function TJvxNodeCollection.IndexOf(Node: TJvxNode): Integer;
begin
  Result := FNodes.IndexOf(Node);
end;

function TJvxNodeCollection.Insert(Index: Integer): TJvxNode;
begin
  Result := JvxClone(Template, Template.Owner) as TJvxNode;
  try
    FNodes.Insert(Index, Result);
  except
    Result.Free;
    raise;
  end;
end;

procedure TJvxNodeCollection.Move(FromIndex, ToIndex: Integer);
begin
  FNodes.Move(FromIndex, ToIndex);
end;

function TJvxNodeCollection.Remove(Node: TJvxNode): Integer;
begin
  Result := FNodes.Remove(Node);
end;

{ TJvxAttribute }

procedure TJvxAttribute.Accept(Visitor: TJvxVisitor);
begin
  Visitor.Visit(Self);
end;

function TJvxAttribute.GetParentComponent: TComponent;
begin
  if HasParent then
    Result := Parent.Attributes
  else
    Result := nil;
end;

procedure TJvxAttribute.SetParent(Value: TJvxNode);
begin
  if Parent <> Value then
  begin
    if (Parent <> nil) and Parent.HasAttributes then
      Parent.Attributes.Remove(Self);
    if Value <> nil then
      Value.Attributes.Add(Self);
  end;
end;

{ TJvxElement }

procedure TJvxElement.Accept(Visitor: TJvxVisitor);
begin
  Visitor.Visit(Self);
end;

destructor TJvxElement.Destroy;
begin
  FAttributes.Free;
  inherited;
end;

procedure TJvxElement.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  inherited;
  if FAttributeSource <> nil then
    FAttributeSource.DocumentNotification(Node, Operation);
end;

function TJvxElement.GetAttributes: TJvxNodes;
begin
  if FAttributes = nil then
  begin
    Result := TJvxNodes.Create(Self);
    Result.Name := SAttributes;
  end else
    Result := FAttributes;
end;

function TJvxElement.GetAttributeSource: TJvxDataSource;
begin
  if FAttributeSource = nil then
    FAttributeSource := TJvxListDataSource.Create(Self);
  Result := FAttributeSource;
end;

function TJvxElement.GetParentComponent: TComponent;
begin
  if HasParent then
    Result := Parent.Elements
  else
    Result := nil;
end;

function TJvxElement.HasAttributes: Boolean;
begin
  Result := (FAttributeSource <> nil) and (FAttributeSource.Count > 0);
end;

procedure TJvxElement.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if Component = FAttributes then
      FAttributes := nil
    else if Component = FAttributeSource then
      FAttributeSource := nil;
  end;
end;

procedure TJvxElement.SetAttributes(Value: TJvxNodes);
begin
  if Value <> nil then
  begin
    Value.OnGetDataSource := GetAttributeSource;
    Value.OnValidateNode := ValidateAttribute;
    Value.Parent := Self;
  end;
  FAttributes := Value;
end;

procedure TJvxElement.SetParent(Value: TJvxNode);
begin
  if Parent <> Value then
  begin
    if (Parent <> nil) and Parent.HasElements then
      Parent.Elements.Remove(Self);
    if Value <> nil then
      Value.Elements.Add(Self);
  end;
end;

procedure TJvxElement.ValidateRename(Component: TComponent;
  const CurName, NewName: string);
begin
  inherited;
  if (Component <> nil) and (Component.Owner = Self) and SameText(NewName, SAttributes) then
    SetAttributes(Component as TJvxNodes);
end;

{ TJvxCompositeElement }

destructor TJvxCompositeElement.Destroy;
begin
  SetElementSource(nil);
  SetTemplate(nil);
  FElements.Free;
  FElementSource.Free; //todo: temporary - destroying FElementSource twice here!!!
  inherited;
end;

procedure TJvxCompositeElement.DocumentNotification(
  Node: TJvxNode; Operation: TJvxNodeOperation);
begin
  inherited;
  if FElementSource <> nil then
    FElementSource.DocumentNotification(Node, Operation);
  if FTemplate <> nil then
    FTemplate.DocumentNotification(Node, Operation);
end;

function TJvxCompositeElement.GetElements: TJvxNodes;
begin
  if FElements = nil then
  begin
    Result := TJvxNodes.Create(Self);
    Result.Name := SElements;
  end else
    Result := FElements;
end;

function TJvxCompositeElement.GetElementSource: TJvxDataSource;
begin
  if FElementSource = nil then
  begin
    FElementSource := TJvxListDataSource.Create(Self);
    Include(FNodeState, nsAggregatedElements);
  end;
  Result := FElementSource;
end;

function TJvxCompositeElement.GetTemplate: TJvxNode;
begin
  Result := FTemplate;
end;

function TJvxCompositeElement.HasElements: Boolean;
begin
  Result := (FElementSource <> nil) and (FElementSource.Count > 0);
end;

procedure TJvxCompositeElement.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if Component = FElements then
      FElements := nil
    else if Component = FElementSource then
      FElementSource := nil
    else if Component = FTemplate then
      FTemplate := nil;
  end;
end;

procedure TJvxCompositeElement.SetElements(Value: TJvxNodes);
begin
  if Value <> nil then
  begin
    Value.OnGetDataSource := GetElementSource;
    Value.OnGetTemplate := GetTemplate;
    Value.OnValidateNode := ValidateElement;
    Value.Parent := Self;
  end;
  FElements := Value;
end;

procedure TJvxCompositeElement.SetElementSource(Value: TJvxDataSource);
begin
  if FElementSource <> Value then
  begin
    if FElementSource <> nil then
    begin
      FElementSource.RemoveFreeNotification(Self);
      if FElementSource.Owner = Self then
      begin
        Elements.Clear;
        FElementSource.Free;
      end;
      Exclude(FNodeState, nsAggregatedElements);
    end;
    FElementSource := Value;
    if FElementSource <> nil then
      FTemplate.FreeNotification(Self);
  end;
end;

// todo: test TJvxCompositeElement.SetTemplate(Value: TJvxNode);
procedure TJvxCompositeElement.SetTemplate(Value: TJvxElement);
begin
  if FTemplate <> Value then
  begin
    if HasElements then
      raise EInvalidOperation.Create(SInvalidTemplateChange);
    if FTemplate <> nil then
    begin
      FTemplate.RemoveFreeNotification(Self);
      FTemplate.FParent := nil;
      if FTemplate.Owner = Self then
        FTemplate.Free;
    end;
    FTemplate := Value;
    if FTemplate <> nil then
    begin
      FTemplate.FreeNotification(Self);
      FTemplate.FParent := Self;
    end;
  end;
end;

procedure TJvxCompositeElement.ValidateRename(Component: TComponent;
  const CurName, NewName: string);
begin
  inherited;
  if (Component <> nil) and (Component.Owner = Self) and SameText(NewName, SElements) then
    SetElements(Component as TJvxNodes);
end;

{ TJvxInstruction }

procedure TJvxInstruction.Accept(Visitor: TJvxVisitor);
begin
  Visitor.Visit(Self);
end;

{ TJvxVisitor }

procedure TJvxVisitor.Visit(Attribute: TJvxAttribute);
begin
end;

procedure TJvxVisitor.Visit(Element: TJvxElement);
begin
end;

procedure TJvxVisitor.Visit(Instruction: TJvxInstruction);
begin
end;

{ TJvxDataField }

procedure TJvxDataField.Changed;
begin
  FOnChangeEvents.Invoke(Self);
end;

procedure TJvxDataField.CheckAssigned;
begin
  if not Assigned then
    raise JvxAccessError(SUnassignedValue);
end;

procedure TJvxDataField.CheckDataType(ValidTypes: TTypeKinds;
  AccessType: PTypeInfo);
begin
  if (TypeInfo = nil) or not (TypeInfo.Kind in ValidTypes) then
    raise JvxAccessError(AccessType);
end;

constructor TJvxDataField.Create(Owner: TComponent);
begin
  inherited;
  FOnChangeEvents := TJvxNotifyEvents.Create(@@FOnChange);
end;

destructor TJvxDataField.Destroy;
begin
  if FDataSource <> nil then
    FDataSource.Fields.Remove(Self);
  inherited;
  FOnChangeEvents.Free;
end;

function TJvxDataField.GetAsFloat: Extended;
begin
  CheckAssigned;
  CheckDataType([tkFloat], System.TypeInfo(Extended));
  Result := InternalGetAsFloat;
end;

function TJvxDataField.GetAsOrdinal: TJvxOrdinal;
begin
  CheckAssigned;
  CheckDataType([tkInteger, tkChar, tkEnumeration], System.TypeInfo(LongInt));
  Result := InternalGetAsOrdinal;
end;

function TJvxDataField.GetAssigned: Boolean;
begin
  Result := InternalGetAssigned;
end;

function TJvxDataField.GetAsString: string;
begin
  CheckAssigned;
  CheckDataType([tkString, tkLString], System.TypeInfo(string));
  Result := InternalGetAsString;
end;

function TJvxDataField.GetAsVariant: Variant;
begin
  CheckAssigned;
  CheckDataType([tkVariant], System.TypeInfo(Variant));
  Result := InternalGetAsVariant;
end;

function TJvxDataField.InternalGetAsFloat: Extended;
begin
  raise JvxAccessError(System.TypeInfo(Extended));
end;

function TJvxDataField.InternalGetAsOrdinal: TJvxOrdinal;
begin
  raise JvxAccessError(System.TypeInfo(TJvxOrdinal));
end;

function TJvxDataField.InternalGetAsString: string;
begin
  raise JvxAccessError(System.TypeInfo(string));
end;

function TJvxDataField.InternalGetAsVariant: Variant;
begin
  raise JvxAccessError(System.TypeInfo(Variant));
end;

procedure TJvxDataField.SetFieldName(const Value: string);
begin
  if (FDataSource <> nil) and (not SameText(FFieldName, Value)) then
    FDataSource.Fields.CheckFieldName(Value);
  FFieldName := Value;
end;

procedure TJvxDataField.SetDataSource(Value: TJvxDataSource);
begin
  if FDataSource <> Value then
  begin
    if FDataSource <> nil then
      FDataSource.Fields.Remove(Self);
    FDataSource := Value;
    if FDataSource <> nil then
      FDataSource.Fields.Add(Self);
  end;
end;

{ TJvxDataFields }

function TJvxDataFields.Add(Item: TJvxDataField): Integer;
begin
  Result := Count;
  Insert(Result, Item);
end;

procedure TJvxDataFields.CheckFieldName(FieldName: string);
begin
  if FindField(FieldName) <> nil then
    raise EJvxError.CreateFmt(SDuplicateDataField, [FieldName]);
end;

procedure TJvxDataFields.Clear;
var
  I: Integer;
  Item: TJvxDataField;
begin
  for I := Count - 1 downto 0 do
  begin
    Item := Delete(I);
    Item.Free;
  end;
end;

constructor TJvxDataFields.Create(DataSource: TJvxDataSource);
begin
  inherited Create;
  FDataSource := DataSource;
end;

function TJvxDataFields.Delete(Index: Integer): TJvxDataField;
begin
  Result := Items[Index];
  FItems.Delete(Index);
  Result.FDataSource := nil;
  if FItems.Count = 0 then
  begin
    FItems.Free;
    FItems := nil;
  end;
end;

destructor TJvxDataFields.Destroy;
begin
  Clear;
  if FDataSource <> nil then
    FDataSource.FFields := nil;
  inherited;
end;

function TJvxDataFields.FieldByName(FieldName: string): TJvxDataField;
begin
  Result := FindField(FieldName);
  if Result = nil then
    raise EJvxError.CreateFmt(SDataFieldNotFound, [FieldName]);
end;

function TJvxDataFields.FindField(FieldName: string): TJvxDataField;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Result := Items[I];
    if SameText(Result.FieldName, FieldName) then
      Exit;
  end;
  Result := nil;
end;

function TJvxDataFields.GetCount: Integer;
begin
  if FItems <> nil then
    Result := FItems.Count
  else
    Result := 0;
end;

function TJvxDataFields.GetItem(Index: Integer): TJvxDataField;
begin
  if FItems <> nil then
    Result := TJvxDataField(FItems[Index])
  else
    raise EListError.CreateFmt(Consts.SListIndexError, [Index]);
end;

function TJvxDataFields.IndexOf(Item: TJvxDataField): Integer;
begin
  if FItems <> nil then
    Result := FItems.IndexOf(Item)
  else
    Result := -1;
end;

procedure TJvxDataFields.Insert(Index: Integer; Item: TJvxDataField);
begin
  ValidateItem(Item);
  if FItems = nil then
    FItems := TList.Create;
  FItems.Insert(Index, Item);
  Item.FDataSource := FDataSource;
end;

function TJvxDataFields.Remove(Item: TJvxDataField): Integer;
begin
  Result := IndexOf(Item);
  if Result >= 0 then
    Delete(Result);
end;

procedure TJvxDataFields.ValidateItem(Item: TJvxDataField);
begin
  if not (Item is TJvxDataField) then
    raise EJvxError.Create(SInvalidDataField);
  if Item.FieldName = EmptyStr then
    raise EJvxError.Create(SEmptyFieldName);
  if IndexOf(Item) >= 0 then
    raise EJvxError.CreateFmt(SDuplicateDataField, [Item.FieldName]);
  CheckFieldName(Item.FieldName);
  if DataSource <> nil then
    DataSource.ValidateField(Item);
end;

{ TJvxDataSource }

procedure TJvxDataSource.CheckActive;
begin
  if not Active then
    raise EJvxError.Create(SDataSourceInactive);
end;

procedure TJvxDataSource.Close;
begin
  if FActive then
  begin
    if FDefaultFields then
      DestroyDefaultFields;
    InternalClose;
    FActive := False;
  end;
end;

function TJvxDataSource.Count: Integer;
begin
  if Active then
    Result := InternalCount
  else
    Result := 0;
end;

procedure TJvxDataSource.CreateDefaultFields;
begin
end;

procedure TJvxDataSource.Delete(Index: Integer);
begin
  CheckActive;
  InternalDelete(Index)
end;

destructor TJvxDataSource.Destroy;
begin
  Close;
  FFields.Free;
  inherited;
end;

procedure TJvxDataSource.DestroyDefaultFields;
var
  Field: TJvxDataField;
  I: Integer;
begin
  if FFields <> nil then
    for I := FFields.Count - 1 downto 0 do
    begin
      Field := FFields[I];
      if Field.Owner = Self then
        Field.Free;
    end;
end;

procedure TJvxDataSource.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
end;

function TJvxDataSource.GetFields: TJvxDataFields;
begin
  if FFields = nil then
    FFields := TJvxDataFields.Create(Self);
  Result := FFields;
end;

function TJvxDataSource.GetNode(Index: Integer;
  Prototype: TJvxNode): TJvxNode;
begin
  CheckActive;
  Result := InternalGetNode(Index, Prototype)
end;

function TJvxDataSource.IndexOf(Node: TJvxNode): Integer;
begin
  if Active then
    Result := InternalIndexOf(Node)
  else
    Result := -1;
end;

procedure TJvxDataSource.Insert(Index: Integer; Node: TJvxNode);
begin
  CheckActive;
  InternalInsert(Index, Node)
end;

procedure TJvxDataSource.InternalClose;
begin
end;

function TJvxDataSource.InternalCount: Integer;
begin
  Result := 0;
end;

procedure TJvxDataSource.InternalDelete(Index: Integer);
begin
  raise EListError.CreateFmt(Consts.SListIndexError, [Index]);
end;

function TJvxDataSource.InternalGetNode(Index: Integer;
  Prototype: TJvxNode): TJvxNode;
begin
  raise EListError.CreateFmt(Consts.SListIndexError, [Index]);
end;

function TJvxDataSource.InternalIndexOf(Node: TJvxNode): Integer;
begin
  Result := -1;
end;

procedure TJvxDataSource.InternalInsert(Index: Integer; Node: TJvxNode);
begin
  raise EInvalidOperation.Create(SNodeCannotHaveChildNodes);
end;

procedure TJvxDataSource.InternalOpen;
begin
  raise EInvalidOperation.Create(SDataSourceCannotBeOpened);
end;

procedure TJvxDataSource.Open;
begin
  if not FActive then
  begin
    FDefaultFields := (FFields = nil) or (FFields.Count = 0);
    InternalOpen;
    if FDefaultFields then
      CreateDefaultFields;
    FActive := True;
  end;
end;

procedure TJvxDataSource.SetActive(Value: Boolean);
begin
  if FActive <> Value then
  begin
    if Value then
      Open
    else
      Close;
  end;
end;

procedure TJvxDataSource.ValidateField(Field: TJvxDataField);
begin
  raise EJvxError.Create(SInvalidDataField);
end;

{ TJvxListDataSource }

constructor TJvxListDataSource.Create(Onwer: TComponent);
begin
  inherited;
  Open;
end;

destructor TJvxListDataSource.Destroy;
begin
  FItems.Free;
  FItems := nil;
  inherited;
end;

procedure TJvxListDataSource.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
var
  I: Integer;
begin
  inherited;
  if FItems <> nil then
    for I := 0 to FItems.Count - 1 do
      TJvxNode(FItems[I]).DocumentNotification(Node, Operation);
end;

procedure TJvxListDataSource.InternalClose;
begin
  if not (csDestroying in ComponentState) then
    raise EJvxError.Create(SDataSourceCannotBeClosed);
end;

function TJvxListDataSource.InternalCount: Integer;
begin
  if FItems <> nil then
    Result := FItems.Count
  else
    Result := inherited InternalCount;
end;

procedure TJvxListDataSource.InternalDelete(Index: Integer);
begin
  if FItems <> nil then
  begin
    FItems.Delete(Index);
    if FItems.Count = 0 then
    begin
      FItems.Free;
      FItems := nil;
    end;
  end
  else inherited;
end;

function TJvxListDataSource.InternalGetNode(Index: Integer;
  Prototype: TJvxNode): TJvxNode;
begin
  if FItems <> nil then
    Result := TJvxNode(FItems[Index])
  else
    Result := inherited InternalGetNode(Index, Prototype);
end;

function TJvxListDataSource.InternalIndexOf(Node: TJvxNode): Integer;
begin
  if FItems <> nil then
    Result := FItems.IndexOf(Node)
  else
    Result := inherited InternalIndexOf(Node);
end;

procedure TJvxListDataSource.InternalInsert(Index: Integer; Node: TJvxNode);
begin
  if FItems = nil then
    FItems := TComponentList.Create(False);
  FItems.Insert(Index, Node);
end;

procedure TJvxListDataSource.InternalOpen;
begin
end;

{ JvxCreateNode }

procedure JvxCreateNode(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer);
var
  Node: TJvxNode absolute Reference;
  ProcData: ^TJvxCreateNodeData absolute Data;
begin
  if Node = nil then
  begin
    Node := NodeClass.Create(ProcData.GetNodes.Owner);
    try
      Node.SetName(Name);
      ProcData.GetNodes.Add(Node);
    except
      Node.Free;
      Node := nil;
      raise;
    end;
    Node.SetAncestor(True);
    Node.Parent.SetInline(True);
    Include(Node.Parent.FNodeState, nsPredefinedNodes);
  end;
end;

{ JvxClearNodeReference }

procedure JvxClearNodeReference(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer);
begin
  if TJvxNode(Data) = TJvxNode(Reference) then
    TJvxNode(Reference) := nil;
end;

{ TValidateNodeHelper }

procedure JvxValidateNode(var Reference; const Name: string;
  NodeClass: TJvxNodeClass; Data: Pointer);
var
  Node: TJvxNode absolute Reference;
  ProcData: ^TJvxValidateNodeData absolute Data;
begin
  if ProcData.NodeIsValid then Exit;
  if ProcData.Node.ClassType <> NodeClass then Exit;
  if not SameText(ProcData.Node.Name, Name) then Exit;
  if Node = nil then
    Node := ProcData.Node;
  if ProcData.Node <> Node then Exit;
  ProcData.NodeIsValid := True;
end;

{ TEmptyNodesHelper }

constructor TEmptyNodesHelper.Create;
begin
  FEmptyDataSource := TJvxDataSource.Create(nil);
  FEmptyNodes := TJvxNodes.Create(nil);
  FEmptyNodes.OnGetDataSource := GetDataSource;
end;

destructor TEmptyNodesHelper.Destroy;
begin
  FEmptyNodes.Free;
  FEmptyDataSource.Free;
  inherited;
end;

function TEmptyNodesHelper.GetDataSource: TJvxDataSource;
begin
  Result := FEmptyDataSource;
end;

{ TWriteNodeStateHelper }

constructor TWriteNodeStateHelper.Create(Node: TJvxNode;
  Writer: TWriter);
begin
  inherited Create;

  FNode := Node;

  FAncestor := TJvxNodeClass(Node.ClassType).Create(nil);
  FAncestor.GetAttributes;
  FAncestor.GetElements;

  FWriter := Writer;
  FOnFindAncestor := FWriter.OnFindAncestor;
  FWriter.OnFindAncestor := FindAncestor;
end;

destructor TWriteNodeStateHelper.Destroy;
begin
  FWriter.OnFindAncestor := FOnFindAncestor;
  FAncestor.Free;
  inherited;
end;

procedure TWriteNodeStateHelper.FindAncestor(Writer: TWriter;
  Component: TComponent; const Name: string; var Ancestor,
  RootAncestor: TComponent);
begin
  if (Ancestor = nil) and (Component.Owner = FNode) then
    Ancestor := FAncestor.FindComponent(Name);

  if Assigned(FOnFindAncestor) then
    FOnFindAncestor(Writer, Component, Name, Ancestor, RootAncestor);
end;

initialization
  EmptyNodesHelper := TEmptyNodesHelper.Create;
  RegisterClass(TJvxNodes);

finalization
  EmptyNodesHelper.Free;

end.
