{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxDebugMem.pas.                                       }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxDebugMem;

interface

uses
  SysUtils;

function JvxValidMethod(Method: TMethod): Boolean;
function JvxValidObject(Obj: TObject): Boolean; overload;
function JvxValidObject(Obj: TObject; Cls: TClass): Boolean; overload;
function JvxValidPointer(Ptr: Pointer): Boolean; overload;
function JvxValidPointer(Ptr: Pointer; Size: Integer): Boolean; overload;

implementation

uses
  JvxMemoryManager;

function JvxValidMethod(Method: TMethod): Boolean;
begin
  try
    Result := False;
    if not JvxValidObject(TObject(Method.Data)) then Exit;
    if not Assigned(Method.Code) then Exit;
    Result := True;
  except
    on EAccessViolation do
      Result := False;
  end;
end;

function JvxValidObject(Obj: TObject): Boolean; overload;
begin
  Result := JvxValidObject(Obj, TObject);
end;

function JvxValidObject(Obj: TObject; Cls: TClass): Boolean; overload;
begin
  try
    Result := False;
    if not JvxValidPointer(Obj, Obj.InstanceSize) then Exit;
    if not Obj.InheritsFrom(Cls) then Exit;
    Result := True;
  except
    on EAccessViolation do
      Result := False;
  end;
end;

function JvxValidPointer(Ptr: Pointer): Boolean; overload;
begin
  Result := JvxVerifyMemoryBlock(Ptr);
end;

function JvxValidPointer(Ptr: Pointer; Size: Integer): Boolean; overload;
var
  BlockSize: Integer;
begin
  Result := False;
  if not JvxValidPointer(Ptr) then Exit;
  if not JvxGetMemoryBlockSize(Ptr, BlockSize) then Exit;
  if BlockSize < JvxAlignedMemoryBlockSize(Size) then Exit;
  Result := True;
end;

end.
