
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxHTMLClasses.pas.                                    }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxHTMLClasses;

{$include Jvx.inc}

interface

uses
  Classes, TypInfo, JvxClasses, JvxStdNodes, JvxUtils;

type

  { TJvxHTMLURIAttribute }

  TJvxHTMLURIAttribute = class(TJvxProtectedStringAttribute);

  { TJvxHTMLScriptAttribute }

  TJvxHTMLScriptAttribute = class(TJvxProtectedStringAttribute);

  { TJvxHTMLIdReferenceAttribute }

  TJvxHTMLIdReferenceAttribute = class(TJvxProtectedStringAttribute);

  TJvxHTMLAlignAttribute = class;

  { TJvxHTMLCustomElement }

  TJvxHTMLCustomElement = class(TJvxCustomCompositeTextElement)
  private
    FHTMLClass: TJvxProtectedStringAttribute;
    FID: TJvxProtectedStringAttribute;
    FOnClick: TJvxHTMLScriptAttribute;
    FOnDblClick: TJvxHTMLScriptAttribute;
    FOnKeyDown: TJvxHTMLScriptAttribute;
    FOnKeyPress: TJvxHTMLScriptAttribute;
    FOnKeyUp: TJvxHTMLScriptAttribute;
    FOnMouseDown: TJvxHTMLScriptAttribute;
    FOnMouseMove: TJvxHTMLScriptAttribute;
    FOnMouseOut: TJvxHTMLScriptAttribute;
    FOnMouseOver: TJvxHTMLScriptAttribute;
    FOnMouseUp: TJvxHTMLScriptAttribute;
    FStyle: TJvxProtectedStringAttribute;
    FTitle: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property HTMLClass: TJvxProtectedStringAttribute read FHTMLClass;
    property ID: TJvxProtectedStringAttribute read FID;
    property OnClick: TJvxHTMLScriptAttribute read FOnClick;
    property OnDblClick: TJvxHTMLScriptAttribute read FOnDblClick;
    property OnKeyDown: TJvxHTMLScriptAttribute read FOnKeyDown;
    property OnKeyPress: TJvxHTMLScriptAttribute read FOnKeyPress;
    property OnKeyUp: TJvxHTMLScriptAttribute read FOnKeyUp;
    property OnMouseDown: TJvxHTMLScriptAttribute read FOnMouseDown;
    property OnMouseMove: TJvxHTMLScriptAttribute read FOnMouseMove;
    property OnMouseOut: TJvxHTMLScriptAttribute read FOnMouseOut;
    property OnMouseOver: TJvxHTMLScriptAttribute read FOnMouseOver;
    property OnMouseUp: TJvxHTMLScriptAttribute read FOnMouseUp;
    property Style: TJvxProtectedStringAttribute read FStyle;
    property Title: TJvxProtectedStringAttribute read FTitle;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { todo: test TJvxHTMLCustomInlineElement }

  TJvxHTMLCustomInlineElement = class(TJvxHTMLCustomElement);

  { todo: test TJvxHTMLCustomBlockElement }

  TJvxHTMLCustomBlockElement = class(TJvxHTMLCustomElement);

  { todo: test TJvxHTMLSpan }

  TJvxHTMLCustomSpan = class(TJvxHTMLCustomInlineElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLSpan = class(TJvxHTMLCustomSpan)
  published
    property Assigned;
    property DataField;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Text;
    property Title;
  end;

  { todo: test TJvxHTMLDiv }

  TJvxHTMLCustomDiv = class(TJvxHTMLCustomBlockElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLDiv = class(TJvxHTMLCustomDiv)
  published
    property Assigned;
    property DataField;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Text;
    property Title;
  end;

  { todo: test TJvxHTMLBooleanAttribute }

  TJvxHTMLCustomBooleanAttribute = class(TJvxAttribute)
  private
    FValue: Boolean;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;

    property Value: Boolean read GetAsBoolean write SetAsBoolean stored False;
  end;

  TJvxHTMLBooleanAttribute = class(TJvxHTMLCustomBooleanAttribute)
  published
    property Assigned;
    property OnNotify;
    property Value;
  end;

  { TJvxHTMLFormMethodAttribute }

  TJvxHTMLFormMethod = (fmGet, fmPost);
  TJvxHTMLCustomFormMethodAttribute = class(TJvxEnumeratedAttribute)
  private
    FValue: TJvxHTMLFormMethod;
    function GetValue: TJvxHTMLFormMethod;
    procedure SetValue(Value: TJvxHTMLFormMethod);
  protected
    function InternalGetAsInteger: Integer; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Ord: TJvxOrdinal): string; override;
    function StrToOrd(const Str: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;

    property Value: TJvxHTMLFormMethod read GetValue write SetValue stored False;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLFormMethodAttribute = class(TJvxHTMLCustomFormMethodAttribute)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  { TJvxHTMLCustomForm }

  TJvxHTMLCustomForm = class(TJvxHTMLCustomBlockElement)
  private
    FAction: TJvxHTMLURIAttribute;
    FFormName: TJvxProtectedStringAttribute;
    FMethod: TJvxHTMLFormMethodAttribute;
    FOnReset: TJvxHTMLScriptAttribute;
    FOnSubmit: TJvxHTMLScriptAttribute;
    FTarget: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property Action: TJvxHTMLURIAttribute read FAction;
    property FormName: TJvxProtectedStringAttribute read FFormName;
    property Method: TJvxHTMLFormMethodAttribute read FMethod;
    property OnReset: TJvxHTMLScriptAttribute read FOnReset;
    property OnSubmit: TJvxHTMLScriptAttribute read FOnSubmit;
    property Target: TJvxProtectedStringAttribute read FTarget;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLForm }

  TJvxHTMLForm = class(TJvxHTMLCustomForm)
  published
    property Action;
    property FormName;
    property HTMLClass;
    property ID;
    property Method;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnReset;
    property OnSubmit;
    property Style;
    property Target;
    property Title;
  end;

  { TJvxHTMLCustomFocusableElement }

  TJvxHTMLCustomFocusableElement = class(TJvxHTMLCustomInlineElement)
  private
    FAccessKey: TJvxProtectedStringAttribute;
    FOnBlur: TJvxHTMLScriptAttribute;
    FOnFocus: TJvxHTMLScriptAttribute;
    FTabIndex: TJvxProtectedIntegerAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property AccessKey: TJvxProtectedStringAttribute read FAccessKey;
    property OnBlur: TJvxHTMLScriptAttribute read FOnBlur;
    property OnFocus: TJvxHTMLScriptAttribute read FOnFocus;
    property TabIndex: TJvxProtectedIntegerAttribute read FTabIndex;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLCustomFormElement }

  TJvxHTMLCustomFormElement = class(TJvxHTMLCustomFocusableElement);

  { TJvxHTMLCustomFormControl }

  TJvxHTMLCustomFormControl = class(TJvxHTMLCustomFormElement)
  private
    FControlName: TJvxProtectedStringAttribute;
    FDisabled: TJvxHTMLBooleanAttribute;
    FHTMLDataField: TJvxProtectedStringAttribute;
    FHTMLDataSource: TJvxProtectedStringAttribute;
    FOnChange: TJvxHTMLScriptAttribute;
    FOnSelect: TJvxHTMLScriptAttribute;
    FReadOnly: TJvxHTMLBooleanAttribute;
    FSize: TJvxProtectedIntegerAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property ControlName: TJvxProtectedStringAttribute read FControlName;
    property Disabled: TJvxHTMLBooleanAttribute read FDisabled;
    property HTMLDataField: TJvxProtectedStringAttribute read FHTMLDataField;
    property HTMLDataSource: TJvxProtectedStringAttribute read FHTMLDataSource;
    property OnChange: TJvxHTMLScriptAttribute read FOnChange;
    property OnSelect: TJvxHTMLScriptAttribute read FOnSelect;
    property ReadOnly: TJvxHTMLBooleanAttribute read FReadOnly;
    property Size: TJvxProtectedIntegerAttribute read FSize;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLInputTypeAttribute }

  TJvxHTMLInputType = (itText, itPassword, itCheckbox, itRadio, itSubmit,
    itReset, itFile, itHidden, itImage, itButton);
  TJvxHTMLCustomInputTypeAttribute = class(TJvxEnumeratedAttribute)
  private
    FValue: TJvxHTMLInputType;
    function GetValue: TJvxHTMLInputType;
    procedure SetValue(Value: TJvxHTMLInputType);
  protected
    function InternalGetAsInteger: Integer; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Ord: TJvxOrdinal): string; override;
    function StrToOrd(const Str: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;

    property Value: TJvxHTMLInputType read GetValue write SetValue stored False;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLInputTypeAttribute = class(TJvxHTMLCustomInputTypeAttribute)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  { TJvxHTMLCustomInput }

  TJvxHTMLCustomInput = class(TJvxHTMLCustomFormControl)
  private
    FChecked: TJvxHTMLBooleanAttribute;
    FInputType: TJvxHTMLInputTypeAttribute;
    FMaxLength: TJvxProtectedIntegerAttribute;
    FSrc: TJvxHTMLURIAttribute;
    FValue: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Checked: TJvxHTMLBooleanAttribute read FChecked;
    property InputType: TJvxHTMLInputTypeAttribute read FInputType;
    property MaxLength: TJvxProtectedIntegerAttribute read FMaxLength;
    property Src: TJvxHTMLURIAttribute read FSrc;
    property Value: TJvxProtectedStringAttribute read FValue;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLInput }

  TJvxHTMLInput = class(TJvxHTMLCustomInput)
  published
    property AccessKey;
    property Checked;
    property ControlName;
    property DataField;
    property Disabled;
    property HTMLClass;
    property HTMLDataField;
    property HTMLDataSource;
    property Id;
    property InputType;
    property MaxLength;
    property ReadOnly;
    property OnBlur;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnFocus;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property OnSelect;
    property Size;
    property Src;
    property Style;
    property TabIndex;
    property Text;
    property Title;
    property Value;
  end;

  { TJvxHTMLCustomTextArea }

  TJvxHTMLCustomTextArea = class(TJvxHTMLCustomFormControl)
  private
    FCols: TJvxProtectedIntegerAttribute;
    FRows: TJvxProtectedIntegerAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Cols: TJvxProtectedIntegerAttribute read FCols;
    property Rows: TJvxProtectedIntegerAttribute read FRows;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTextArea }

  TJvxHTMLTextArea = class(TJvxHTMLCustomTextArea)
  published
    property Assigned;
    property AccessKey;
    property Cols;
    property ControlName;
    property Disabled;
    property HTMLClass;
    property HTMLDataField;
    property HTMLDataSource;
    property ID;
    property DataField;
    property OnBlur;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnFocus;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property OnSelect;
    property ReadOnly;
    property Rows;
    property Style;
    property TabIndex;
    property Text;
    property Title;
  end;

  { TJvxHTMLCustomOption }

  TJvxHTMLCustomOption = class(TJvxHTMLCustomElement)
  private
    FDisabled: TJvxHTMLBooleanAttribute;
    FOptionLabel: TJvxProtectedStringAttribute;
    FSelected: TJvxHTMLBooleanAttribute;
    FValue: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Disabled: TJvxHTMLBooleanAttribute read FDisabled;
    property OptionLabel: TJvxProtectedStringAttribute read FOptionLabel;
    property Selected: TJvxHTMLBooleanAttribute read FSelected;
    property Value: TJvxProtectedStringAttribute read FValue;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLOption }

  TJvxHTMLOption = class(TJvxHTMLCustomOption)
  published
    property DataField;
    property Disabled;
    property HTMLClass;
    property Id;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OptionLabel;
    property Selected;
    property Style;
    property Text;
    property Title;
    property Value;
  end;

  { TJvxHTMLOptions }

  TJvxHTMLOptions = class(TJvxNodeCollection)
  private
    function GetItem(Index: Integer): TJvxHTMLOption;
  public
    function Add: TJvxHTMLOption;
    function Insert(Index: Integer): TJvxHTMLOption;
    property Items[Index: Integer]: TJvxHTMLOption read GetItem; default;
  end;

  { TJvxHTMLCustomSelect }

  TJvxHTMLCustomSelect = class(TJvxHTMLCustomFormControl)
  private
    FMultiple: TJvxHTMLBooleanAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property Multiple: TJvxHTMLBooleanAttribute read FMultiple;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLSelect }

  TJvxHTMLSelect = class(TJvxHTMLCustomSelect)
  private
    FOptions: TJvxHTMLOptions;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    property AccessKey;
    property ControlName;
    property Disabled;
    property HTMLClass;
    property HTMLDataField;
    property HTMLDataSource;
    property Id;
    property Multiple;
    property OnBlur;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnFocus;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Options: TJvxHTMLOptions read FOptions;
    property Size;
    property Style;
    property TabIndex;
    property Template write SetTemplate;
    property Title;
  end;

  { TJvxHTMLLabel }

  TJvxHTMLCustomLabel = class(TJvxHTMLCustomFormElement)
  private
    FFocusControl: TJvxHTMLIdReferenceAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property FocusControl: TJvxHTMLIdReferenceAttribute read FFocusControl;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLLabel = class(TJvxHTMLCustomLabel)
  published
    property AccessKey;
    property Assigned;
    property DataField;
    property DataSource;
    property FocusControl;
    property HTMLClass;
    property ID;
    property OnBlur;
    property OnClick;
    property OnDblClick;
    property OnFocus;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Template write SetTemplate;
    property Text;
    property Title;
  end;

  { todo: test TJvxHTMLImage }

  TJvxHTMLCustomImage = class(TJvxHTMLCustomFormControl)
  private
    FIsMap: TJvxHTMLBooleanAttribute;
    FUseMap: TJvxHTMLURIAttribute;
    FLongDesc: TJvxHTMLURIAttribute;
    FSource: TJvxHTMLURIAttribute;
    FAlt: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Alt: TJvxProtectedStringAttribute read FAlt;
    property IsMap: TJvxHTMLBooleanAttribute read FIsMap;
    property LongDesc: TJvxHTMLURIAttribute read FLongDesc;
    property Source: TJvxHTMLURIAttribute read FSource;
    property UseMap: TJvxHTMLURIAttribute read FUseMap;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLImage = class(TJvxHTMLCustomImage)
  published
    property AccessKey;
    property Alt;
    property ControlName;
    property Disabled;
    property HTMLClass;
    property HTMLDataField;
    property HTMLDataSource;
    property ID;
    property IsMap;
    property LongDesc;
    property OnBlur;
    property OnClick;
    property OnDblClick;
    property OnFocus;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Source;
    property Style;
    property TabIndex;
    property Title;
    property UseMap;
  end;

  { TJvxHTMLLegend }

  TJvxHTMLCustomLegend = class(TJvxHTMLCustomElement)
  private
    FAlign: TJvxHTMLAlignAttribute;
    FAccessKey: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property AccessKey: TJvxProtectedStringAttribute read FAccessKey;
    property Align: TJvxHTMLAlignAttribute read FAlign;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLLegend = class(TJvxHTMLCustomLegend)
  published
    property AccessKey;
    property Align;
    property Assigned;
    property DataField;
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Template write SetTemplate;
    property Text;
    property Title;
  end;

  { TJvxHTMLFieldSet }

  TJvxHTMLCustomFieldSet = class(TJvxHTMLCustomBlockElement)
  private
    FLegend: TJvxHTMLLegend;
  protected
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;
    property Legend: TJvxHTMLLegend read FLegend;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLFieldSet = class(TJvxHTMLCustomFieldSet)
  published
    property Assigned;
    property DataField;
    property DataSource;
    property HTMLClass;
    property ID;
    property Legend;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Template write SetTemplate;
    property Text;
    property Title;
  end;

  { TJvxHTMLTitle }

  TJvxHTMLCustomTitle = class(TJvxCustomStringElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLTitle = class(TJvxHTMLCustomTitle)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  { TJvxHTMLHead }

  TJvxHTMLCustomHead = class(TJvxCompositeElement)
  private
    FProfile: TJvxHTMLURIAttribute;
    FTitle: TJvxHTMLTitle;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Profile: TJvxHTMLURIAttribute read FProfile;
    property Title: TJvxHTMLTitle read FTitle;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLHead = class(TJvxHTMLCustomHead)
  published
    property OnNotify;
    property Profile;
    property Title;
  end;

  { TJvxHTMLCustomBodyElement }

  TJvxHTMLCustomBodyElement = class(TJvxHTMLCustomElement)
  private
    FOnLoad: TJvxHTMLScriptAttribute;
    FOnUnload: TJvxHTMLScriptAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property OnLoad: TJvxHTMLScriptAttribute read FOnLoad;
    property OnUnload: TJvxHTMLScriptAttribute read FOnUnload;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLBody }

  TJvxHTMLCustomBody = class(TJvxHTMLCustomBodyElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLBody = class(TJvxHTMLCustomBody)
  published
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoad;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property OnUnload;
    property Style;
    property Template write SetTemplate;
    property Title;
  end;

  { TJvxHTMLCustomRootElement }

  TJvxHTMLCustomRootElement = class(TJvxCompositeElement)
  private
    FHead: TJvxHTMLHead;
  protected
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); override;
    property Head: TJvxHTMLHead read FHead;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLElement }

  TJvxCustomHTMLElement = class(TJvxHTMLCustomRootElement)
  private
    FBody: TJvxHTMLBody;
  protected
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); override;
    property Body: TJvxHTMLBody read FBody;
  end;

  TJvxHTMLElement = class(TJvxCustomHTMLElement)
  published
    property Body;
    property Head;
    property OnNotify;
  end;

  { todo: test TJvxHTMLCustomFrameScrollingAttribute }

  TJvxHTMLFrameScrolling = (fsAuto, fsYes, fsNo);
  TJvxHTMLCustomFrameScrollingAttribute = class(TJvxEnumeratedAttribute)
  private
    FValue: TJvxHTMLFrameScrolling;
    function GetValue: TJvxHTMLFrameScrolling;
    procedure SetValue(Value: TJvxHTMLFrameScrolling);
  protected
    function InternalGetAsInteger: Integer; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Ord: TJvxOrdinal): string; override;
    function StrToOrd(const Str: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;

    property Value: TJvxHTMLFrameScrolling read GetValue write SetValue stored False;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLFrameScrollingAttribute = class(TJvxHTMLCustomFrameScrollingAttribute)
  published
    property Assigned;
    property OnNotify;
    property Value;
  end;

  { todo: test TJvxHTMLFrameBorderAttribute }

  TJvxHTMLFrameBorderAttribute = class(TJvxCustomBooleanAttribute)
  protected
    function OrdToStr(Ord: TJvxOrdinal): string; override;
    function StrToOrd(const Str: string): TJvxOrdinal; override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { todo: test TJvxHTMLAlignAttribute }

  TJvxHTMLAlign = (haLeft, haTop, haRight, haBottom, haBaseLine, haMiddle,
    haCenter, haJustify, haChar);
  TJvxHTMLAligns = set of TJvxHTMLAlign;

  TJvxHTMLCustomAlignAttribute = class(TJvxEnumeratedAttribute)
  private
    FValue: TJvxHTMLAlign;
    FValidValues: TJvxHTMLAligns;
    function GetValue: TJvxHTMLAlign;
    procedure SetValidValues(AValue: TJvxHTMLAligns);
    procedure SetValue(Value: TJvxHTMLAlign);
  protected
    function InternalGetAsInteger: Integer; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Ord: TJvxOrdinal): string; override;
    function StrToOrd(const Str: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;

    property ValidValues: TJvxHTMLAligns read FValidValues write SetValidValues;
    property Value: TJvxHTMLAlign read GetValue write SetValue stored False;
  public
    constructor Create(Owner: TComponent); override;
  end;

  TJvxHTMLAlignAttribute = class(TJvxHTMLCustomAlignAttribute)
  public
    constructor Create(Owner: TComponent); override;
  published
    property Assigned;
    property OnNotify;
    property Value;
  end;

  { todo: test TJvxHTMLCustomFrame }

  TJvxHTMLCustomFrame = class(TJvxHTMLCustomElement)
  private
    FFrameBorder: TJvxHTMLFrameBorderAttribute;
    FFrameName: TJvxProtectedStringAttribute;
    FLongDesc: TJvxHTMLURIAttribute;
    FMarginHeight: TJvxProtectedIntegerAttribute;
    FMarginWidth: TJvxProtectedIntegerAttribute;
    FNoResize: TJvxHTMLBooleanAttribute;
    FSrolling: TJvxHTMLFrameScrollingAttribute;
    FSrc: TJvxHTMLURIAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property FrameBorder: TJvxHTMLFrameBorderAttribute read FFrameBorder;
    property FrameName: TJvxProtectedStringAttribute read FFrameName;
    property LongDesc: TJvxHTMLURIAttribute read FLongDesc;
    property MarginHeight: TJvxProtectedIntegerAttribute read FMarginHeight;
    property MarginWidth: TJvxProtectedIntegerAttribute read FMarginWidth;
    property NoResize: TJvxHTMLBooleanAttribute read FNoResize;
    property Srolling: TJvxHTMLFrameScrollingAttribute read FSrolling;
    property Src: TJvxHTMLURIAttribute read FSrc;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { todo: test TJvxHTMLCustomInlineFrame }

  TJvxHTMLCustomInlineFrame = class(TJvxHTMLCustomFrame)
  private
    FAlign: TJvxHTMLAlignAttribute;
    FHeight: TJvxProtectedStringAttribute;
    FWidth: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Align: TJvxHTMLAlignAttribute read FAlign;
    property Height: TJvxProtectedStringAttribute read FHeight;
    property Width: TJvxProtectedStringAttribute read FWidth;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLInlineFrame }

  TJvxHTMLInlineFrame = class(TJvxHTMLCustomInlineFrame)
  published
    property Align;
    property FrameName;
    property FrameBorder;
    property Height;
    property HTMLClass;
    property Id;
    property LongDesc;
    property MarginHeight;
    property MarginWidth;
    property NoResize;
    property Srolling;
    property Src;
    property Style;
    property Title;
    property Width;
  end;

  { todo: test TJvxHTMLCustomScript }

  TJvxHTMLCustomScript = class(TJvxHTMLCustomElement)
  private
    FSrc: TJvxHTMLURIAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    property Src: TJvxHTMLURIAttribute read FSrc;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLScript }

  TJvxHTMLScript = class(TJvxHTMLCustomScript)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Src;
    property Text;
  end;

  { TJvxHTMLCustomTableCaption }

  TJvxHTMLCustomTableCaption = class(TJvxHTMLCustomInlineElement)
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLTableCaption }

  TJvxHTMLTableCaption = class(TJvxHTMLCustomTableCaption)
  published
    property Assigned;
    property DataField;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property Style;
    property Text;
    property Title;
  end;

  { TJvxHTMLCustomTableElement }

  TJvxHTMLCustomTableElement = class(TJvxHTMLCustomBlockElement)
  private
    FAlign: TJvxHTMLAlignAttribute;
    FAlignChar: TJvxProtectedStringAttribute;
    FAlignCharOffset: TJvxProtectedStringAttribute;
    FHeight: TJvxProtectedStringAttribute;
    FVerticalAlign: TJvxHTMLAlignAttribute;
    FWidth: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Align: TJvxHTMLAlignAttribute read FAlign;
    property AlignChar: TJvxProtectedStringAttribute read FAlignChar;
    property AlignCharOffset: TJvxProtectedStringAttribute read FAlignCharOffset;
    property Height: TJvxProtectedStringAttribute read FHeight;
    property VerticalAlign: TJvxHTMLAlignAttribute read FVerticalAlign;
    property Width: TJvxProtectedStringAttribute read FWidth;
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLCustomTableColumnElement }

  TJvxHTMLCustomTableColumnElement = class(TJvxHTMLCustomTableElement)
  private
    FSpan: TJvxProtectedIntegerAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property Span: TJvxProtectedIntegerAttribute read FSpan;
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLCustomTableColumn }

  TJvxHTMLCustomTableColumn = class(TJvxHTMLCustomTableColumnElement)
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLTableColumn }

  TJvxHTMLTableColumn = class(TJvxHTMLCustomTableColumn)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Span;
    property Style;
    property Title;
    property VerticalAlign;
    property Width;
  end;

  { TJvxHTMLCustomTableColumnGroup }

  TJvxHTMLCustomTableColumnGroup = class(TJvxHTMLCustomTableColumnElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableColumnGroup }

  TJvxHTMLTableColumnGroup = class(TJvxHTMLCustomTableColumnGroup)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Span;
    property Style;
    property Template write SetTemplate;
    property Title;
    property VerticalAlign;
    property Width;
  end;

  { TJvxHTMLCustomTableRowGroupElement }

  TJvxHTMLCustomTableRowGroupElement = class(TJvxHTMLCustomTableElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  end;

  { TJvxHTMLCustomTableHeader }

  TJvxHTMLCustomTableHeader = class(TJvxHTMLCustomTableRowGroupElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableHeader }

  TJvxHTMLTableHeader = class(TJvxHTMLCustomTableHeader)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Style;
    property Template write SetTemplate;
    property Title;
    property VerticalAlign;
  end;

  { TJvxHTMLCustomTableBody }

  TJvxHTMLCustomTableBody = class(TJvxHTMLCustomTableRowGroupElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableBody }

  TJvxHTMLTableBody = class(TJvxHTMLCustomTableBody)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Style;
    property Template write SetTemplate;
    property Title;
    property VerticalAlign;
  end;

  { TJvxHTMLCustomTableFooter }

  TJvxHTMLCustomTableFooter = class(TJvxHTMLCustomTableRowGroupElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableFooter }

  TJvxHTMLTableFooter = class(TJvxHTMLCustomTableFooter)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property DataSource;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Style;
    property Template write SetTemplate;
    property Title;
    property VerticalAlign;
  end;

  { TJvxHTMLCustomTableRow }

  TJvxHTMLCustomTableRow = class(TJvxHTMLCustomTableElement)
  protected
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableRow }

  TJvxHTMLTableRow = class(TJvxHTMLCustomTableRow)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property DataSource;
    property Height;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Style;
    property Template write SetTemplate;
    property Title;
    property VerticalAlign;
    property Width;
  end;

  { TJvxHTMLCustomTableCellElement }

  TJvxHTMLCustomTableCellElement = class(TJvxHTMLCustomTableElement)
  private
    FColSpan: TJvxProtectedIntegerAttribute;
    FNoWrap: TJvxHTMLBooleanAttribute;
    FRowSpan: TJvxProtectedIntegerAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property ColSpan: TJvxProtectedIntegerAttribute read FColSpan;
    property NoWrap: TJvxHTMLBooleanAttribute read FNoWrap;
    property RowSpan: TJvxProtectedIntegerAttribute read FRowSpan;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLCustomTableHeaderCell }

  TJvxHTMLCustomTableHeaderCell = class(TJvxHTMLCustomTableCellElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableHeaderCell }

  TJvxHTMLTableHeaderCell = class(TJvxHTMLCustomTableHeaderCell)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property Assigned;
    property ColSpan;
    property DataField;
    property Height;
    property HTMLClass;
    property ID;
    property NoWrap;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property RowSpan;
    property Style;
    property Text;
    property Title;
    property VerticalAlign;
    property Width;
  end;

  { TJvxHTMLCustomTableDataCell }

  TJvxHTMLCustomTableDataCell = class(TJvxHTMLCustomTableCellElement)
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLTableDataCell }

  TJvxHTMLTableDataCell = class(TJvxHTMLCustomTableDataCell)
  published
    property Align;
    property AlignChar;
    property AlignCharOffset;
    property Assigned;
    property ColSpan;
    property DataField;
    property Height;
    property HTMLClass;
    property ID;
    property NoWrap;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadData;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property OnNotify;
    property RowSpan;
    property Style;
    property Text;
    property Title;
    property VerticalAlign;
    property Width;
  end;

  { TJvxHTMLCustomTable }

  // todo -cnew feature -oosych: add properties: Header, Body, Footer
  TJvxHTMLCustomTable = class(TJvxHTMLCustomTableElement)
  private
    FBorder: TJvxProtectedIntegerAttribute;
    FCellPadding: TJvxProtectedStringAttribute;
    FCellSpacing: TJvxProtectedStringAttribute;
    FSummary: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property Border: TJvxProtectedIntegerAttribute read FBorder;
    property CellPadding: TJvxProtectedStringAttribute read FCellPadding;
    property CellSpacing: TJvxProtectedStringAttribute read FCellSpacing;
    property Summary: TJvxProtectedStringAttribute read FSummary;
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLTable }

  TJvxHTMLTable = class(TJvxHTMLCustomTable)
  published
    property Border;
    property CellPadding;
    property CellSpacing;
    property Height;
    property HTMLClass;
    property ID;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseOut;
    property OnMouseOver;
    property OnMouseUp;
    property Style;
    property Summary;
    property Title;
    property Width;
  end;

  { TJvxHTMLCustomXMLDataIsland }

  TJvxHTMLCustomXMLDataIsland = class(TJvxHTMLCustomInlineElement)
  private
    FSrc: TJvxHTMLURIAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure ValidateElement(Element: TJvxNode); override;

    property Src: TJvxHTMLURIAttribute read FSrc;
  public
    constructor Create(Onwer: TComponent); override;
  end;

  { TJvxHTMLXMLDataIsland }

  TJvxHTMLXMLDataIsland = class(TJvxHTMLCustomXMLDataIsland)
  published
    property Id;
    property Src;
  end;

  { TJvxHTMLCustomImportInstruction }

  TJvxHTMLCustomImportInstruction = class(TJvxInstruction)
  private
    FElementImplementation: TJvxHTMLURIAttribute;
    FNameSpace: TJvxProtectedStringAttribute;
  protected
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;

    property ElementImplementation: TJvxHTMLURIAttribute read FElementImplementation;
    property NameSpace: TJvxProtectedStringAttribute read FNameSpace;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TJvxHTMLImportInstruction }

  TJvxHTMLImportInstruction = class(TJvxHTMLCustomImportInstruction)
  published
    property ElementImplementation;
    property NameSpace;
    property OnNotify;
  end;

implementation

uses
  SysUtils;

const
  AlignStrings: array[TJvxHTMLAlign] of string =
  (
    'left', 'top', 'right', 'bottom', 'baseline',
    'middle', 'center', 'justify', 'char'
  );
  FormMethodStrings: array[TJvxHTMLFormMethod] of string =
  (
    'get', 'post'
  );
  FrameScrollingStrings: array[TJvxHTMLFrameScrolling] of string =
  (
    'auto', 'yes', 'no'
  );
  InputTypeStrings: array[TJvxHTMLInputType] of string =
  (
    'text', 'password', 'checkbox', 'radio', 'submit',
    'reset', 'file', 'hidden', 'image', 'button'
  );

resourcestring
  SInvalidAlign = '''%s'' is not a valid align';
  SInvalidAlignSet = 'Set of allowed align values cannot be empty';
  SInvalidBoolean = '''%s'' is not a valid boolean';
  SInvalidFormMethod = '''%s'' is not a valid form method';
  SInvalidFrameScrolling = '''%s'' is not a valid frame scrolling';
  SInvalidInputType = '''%s'' is not a valid input type';


{ TJvxHTMLCustomElement }

constructor TJvxHTMLCustomElement.Create(Owner: TComponent);
begin
  inherited;
  TJvxExposedNode(HTMLClass).NodeName := 'class';
  TJvxExposedNode(ID).NodeName := 'id';
  TJvxExposedNode(OnClick).NodeName := 'onclick';
  TJvxExposedNode(OnDblClick).NodeName := 'ondblclick';
  TJvxExposedNode(OnKeyDown).NodeName := 'onkeydown';
  TJvxExposedNode(OnKeyPress).NodeName := 'onkeypress';
  TJvxExposedNode(OnKeyUp).NodeName := 'onkeyup';
  TJvxExposedNode(OnMouseDown).NodeName := 'onmousedown';
  TJvxExposedNode(OnMouseMove).NodeName := 'onmousemove';
  TJvxExposedNode(OnMouseOut).NodeName := 'onmouseout';
  TJvxExposedNode(OnMouseOver).NodeName := 'onmouseover';
  TJvxExposedNode(OnMouseUp).NodeName := 'onmouseup';
  TJvxExposedNode(Style).NodeName := 'style';
  TJvxExposedNode(Title).NodeName := 'title';
end;

procedure TJvxHTMLCustomElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FHTMLClass, 'FHTMLClass', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FID, 'FID', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FOnClick, 'FOnClick', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnDblClick, 'FOnDblClick', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnKeyDown, 'FOnKeyDown', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnKeyPress, 'FOnKeyPress', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnKeyUp, 'FOnKeyUp', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnMouseDown, 'FOnMouseDown', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnMouseMove, 'FOnMouseMove', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnMouseOut, 'FOnMouseOut', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnMouseOver, 'FOnMouseOver', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnMouseUp, 'FOnMouseUp', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FStyle, 'FStyle', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FTitle, 'FTitle', TJvxProtectedStringAttribute, Data);
end;

{ TJvxHTMLCustomSpan }

constructor TJvxHTMLCustomSpan.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('span');
end;

procedure TJvxHTMLCustomSpan.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomInlineElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomDiv }

constructor TJvxHTMLCustomDiv.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('div');
end;

procedure TJvxHTMLCustomDiv.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomBlockElement then Exit;
  if Element is TJvxHTMLCustomInlineElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomBooleanAttribute }

procedure TJvxHTMLCustomBooleanAttribute.AssignValue(Source: TJvxNode);
begin
  AsBoolean := Source.AsBoolean;
end;

function TJvxHTMLCustomBooleanAttribute.GetAssigned: Boolean;
begin
  Result := FValue;
end;

function TJvxHTMLCustomBooleanAttribute.InternalGetAsBoolean: Boolean;
begin
  Result := FValue;
end;

function TJvxHTMLCustomBooleanAttribute.InternalGetAsString: string;
begin
  Result := NodeName;
end;

function TJvxHTMLCustomBooleanAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxHTMLCustomBooleanAttribute.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := Value;
end;

procedure TJvxHTMLCustomBooleanAttribute.InternalSetAssigned(Value: Boolean);
begin
  // todo: find the solution instead of this workaround for TJvxNodeValue.SetAsXXX limitation
  if (FValue = True) and (Value = False) then
    FValue := Value;
end;

procedure TJvxHTMLCustomBooleanAttribute.InternalSetAsString(
  const Value: string);
begin
  if Value = NodeName then
    FValue := True
  else if Value = EmptyStr then
    FValue := False
  else
    raise EConvertError.CreateFmt(SInvalidBoolean, [Value]);
end;

{ TJvxHTMLCustomFormMethodAttribute }

constructor TJvxHTMLCustomFormMethodAttribute.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('method');
end;

function TJvxHTMLCustomFormMethodAttribute.GetValue: TJvxHTMLFormMethod;
begin
  Result := TJvxHTMLFormMethod(AsInteger);
end;

function TJvxHTMLCustomFormMethodAttribute.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

procedure TJvxHTMLCustomFormMethodAttribute.InternalSetAsInteger(
  Value: Integer);
begin
  inherited;
  FValue := TJvxHTMLFormMethod(Value);
end;

function TJvxHTMLCustomFormMethodAttribute.OrdToStr(
  Ord: TJvxOrdinal): string;
begin
  Result := FormMethodStrings[TJvxHTMLFormMethod(Ord)];
end;

procedure TJvxHTMLCustomFormMethodAttribute.SetValue(
  Value: TJvxHTMLFormMethod);
begin
  AsInteger := Ord(Value);
end;

function TJvxHTMLCustomFormMethodAttribute.StrToOrd(
  const Str: string): TJvxOrdinal;
begin
  for Result := Ord(Low(FormMethodStrings)) to Ord(High(FormMethodStrings)) do
    if SameText(FormMethodStrings[TJvxHTMLFormMethod(Result)], Str) then
      Exit;
  raise EConvertError.CreateFmt(SInvalidFormMethod, [Str]);
end;

class function TJvxHTMLCustomFormMethodAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(TJvxHTMLFormMethod);
end;

{ TJvxHTMLCustomForm }

constructor TJvxHTMLCustomForm.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('form');
  TJvxExposedNode(Action).NodeName := 'action';
  TJvxExposedNode(FormName).NodeName := 'name';
  TJvxExposedNode(OnReset).NodeName := 'onreset';
  TJvxExposedNode(OnSubmit).NodeName := 'onsubmit';
  TJvxExposedNode(Target).NodeName := 'target';
end;

procedure TJvxHTMLCustomForm.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAction, 'FAction', TJvxHTMLURIAttribute, Data);
  DefineAttribute(FFormName, 'FFormName', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FMethod, 'FMethod', TJvxHTMLFormMethodAttribute, Data);
  DefineAttribute(FOnReset, 'FOnReset', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnSubmit, 'FOnSubmit', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FTarget, 'FTarget', TJvxProtectedStringAttribute, Data);
end;

procedure TJvxHTMLCustomForm.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomFocusableElement }

constructor TJvxHTMLCustomFocusableElement.Create(Owner: TComponent);
begin
  inherited;
  TJvxExposedNode(AccessKey).NodeName := 'accesskey';
  TJvxExposedNode(OnBlur).NodeName := 'onblur';
  TJvxExposedNode(OnFocus).NodeName := 'onfocus';
  TJvxExposedNode(TabIndex).NodeName := 'tabindex';
end;

procedure TJvxHTMLCustomFocusableElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAccessKey, 'FAccessKey', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FOnBlur, 'FOnBlur', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnFocus, 'FOnFocus', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FTabIndex, 'FTabIndex', TJvxProtectedIntegerAttribute, Data);
end;

{ TJvxHTMLCustomFormControl }

constructor TJvxHTMLCustomFormControl.Create(Owner: TComponent);
begin
  inherited;
  TJvxExposedNode(ControlName).NodeName := 'name';
  TJvxExposedNode(Disabled).NodeName := 'disabled';
  TJvxExposedNode(HTMLDataField).NodeName := 'datafld';
  TJvxExposedNode(HTMLDataSource).NodeName := 'datasrc';
  TJvxExposedNode(OnChange).NodeName := 'onchange';
  TJvxExposedNode(OnSelect).NodeName := 'onselect';
  TJvxExposedNode(ReadOnly).NodeName := 'readonly';
  TJvxExposedNode(Size).NodeName := 'size';
end;

procedure TJvxHTMLCustomFormControl.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FControlName, 'FControlName', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FDisabled, 'FDisabled', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FHTMLDataField, 'FHTMLDataField', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FHTMLDataSource, 'FHTMLDataSource', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FOnChange, 'FOnChange', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnSelect, 'FOnSelect', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FReadOnly, 'FReadOnly', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FSize, 'FSize', TJvxProtectedIntegerAttribute, Data);
end;

{ TJvxHTMLCustomInputTypeAttribute }

constructor TJvxHTMLCustomInputTypeAttribute.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('type');
end;

function TJvxHTMLCustomInputTypeAttribute.GetValue: TJvxHTMLInputType;
begin
  Result := TJvxHTMLInputType(AsInteger);
end;

function TJvxHTMLCustomInputTypeAttribute.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

procedure TJvxHTMLCustomInputTypeAttribute.InternalSetAsInteger(
  Value: Integer);
begin
  inherited;
  FValue := TJvxHTMLInputType(Value);
end;

function TJvxHTMLCustomInputTypeAttribute.OrdToStr(
  Ord: TJvxOrdinal): string;
begin
  Result := InputTypeStrings[TJvxHTMLInputType(Ord)];
end;

procedure TJvxHTMLCustomInputTypeAttribute.SetValue(
  Value: TJvxHTMLInputType);
begin
  AsInteger := Ord(Value);
end;

function TJvxHTMLCustomInputTypeAttribute.StrToOrd(
  const Str: string): TJvxOrdinal;
begin
  for Result := Ord(Low(InputTypeStrings)) to Ord(High(InputTypeStrings)) do
    if SameText(InputTypeStrings[TJvxHTMLInputType(Result)], Str) then
      Exit;
  raise EConvertError.CreateFmt(SInvalidInputType, [Str]);
end;

class function TJvxHTMLCustomInputTypeAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(TJvxHTMLInputType);
end;

{ TJvxHTMLCustomInput }

constructor TJvxHTMLCustomInput.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('input');
  TJvxExposedNode(Checked).NodeName := 'checked';
  TJvxExposedNode(MaxLength).NodeName := 'maxlength';
  TJvxExposedNode(Src).NodeName := 'src';
  TJvxExposedNode(Value).NodeName := 'value';
end;

procedure TJvxHTMLCustomInput.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FChecked, 'FChecked', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FInputType, 'FInputType', TJvxHTMLInputTypeAttribute, Data);
  DefineAttribute(FMaxLength, 'FMaxLength', TJvxProtectedIntegerAttribute, Data);
  DefineAttribute(FSrc, 'FSrc', TJvxHTMLURIAttribute, Data);
  DefineAttribute(FValue, 'FValue', TJvxProtectedStringAttribute, Data);
end;

{ TJvxHTMLCustomTextArea }

constructor TJvxHTMLCustomTextArea.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('textarea');
  TJvxExposedNode(Cols).NodeName := 'cols';
  TJvxExposedNode(Rows).NodeName := 'rows';
end;

procedure TJvxHTMLCustomTextArea.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FCols, 'FCols', TJvxProtectedIntegerAttribute, Data);
  DefineAttribute(FRows, 'FRows', TJvxProtectedIntegerAttribute, Data);
end;

{ TJvxHTMLCustomOption }

constructor TJvxHTMLCustomOption.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('option');
  TJvxExposedNode(Disabled).NodeName := 'disabled';
  TJvxExposedNode(OptionLabel).NodeName := 'label';
  TJvxExposedNode(Selected).NodeName := 'selected';
  TJvxExposedNode(Value).NodeName := 'value';
end;

procedure TJvxHTMLCustomOption.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FDisabled, 'FDisabled', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FOptionLabel, 'FOptionLabel', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FSelected, 'FSelected', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FValue, 'FValue', TJvxProtectedStringAttribute, Data);
end;

{ TJvxHTMLOptions }

function TJvxHTMLOptions.Add: TJvxHTMLOption;
begin
  Result := inherited Add as TJvxHTMLOption;
end;

function TJvxHTMLOptions.GetItem(Index: Integer): TJvxHTMLOption;
begin
  Result := inherited Items[Index] as TJvxHTMLOption;
end;

function TJvxHTMLOptions.Insert(Index: Integer): TJvxHTMLOption;
begin
  Result := inherited Insert(Index) as TJvxHTMLOption;
end;

{ TJvxHTMLCustomSelect }

constructor TJvxHTMLCustomSelect.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('select');
  TJvxExposedNode(Multiple).NodeName := 'multiple';
  SetTemplate(TJvxHTMLOption.Create(Self));
end;

procedure TJvxHTMLCustomSelect.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FMultiple, 'FMultiple', TJvxHTMLBooleanAttribute, Data);
end;

procedure TJvxHTMLCustomSelect.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
var
  I: Integer;
  Option: TJvxHTMLCustomOption;
begin
  inherited;
  if (Operation = noChange) and (Node.Parent is TJvxHTMLCustomOption) and
     ((not Multiple.Assigned) or (Multiple.AsBoolean = False)) then
  begin
    Option := TJvxHTMLCustomOption(Node.Parent);
    if (Option.Parent = Self) and (Node = Option.Selected) and
       (Option.Selected.Assigned = True) and
       (Option.Selected.AsBoolean = True) then
    begin
      for I := 0 to Elements.Count - 1 do
        (Elements[I] as TJvxHTMLCustomOption).Selected.Assigned :=
          Elements[I] = Option;
    end;
  end;
end;

procedure TJvxHTMLCustomSelect.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomOption then Exit;
  inherited;
end;

{ TJvxHTMLSelect }

constructor TJvxHTMLSelect.Create(Owner: TComponent);
begin
  inherited;
  FOptions := TJvxHTMLOptions.Create(Elements);
end;

destructor TJvxHTMLSelect.Destroy;
begin
  FOptions.Free;
  inherited;
end;

{ TJvxHTMLCustomLabel }

constructor TJvxHTMLCustomLabel.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('label');
  TJvxExposedNode(FocusControl).NodeName := 'for';
end;

procedure TJvxHTMLCustomLabel.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FFocusControl, 'FFocusControl', TJvxHTMLIdReferenceAttribute, Data);
end;

procedure TJvxHTMLCustomLabel.ValidateElement(Element: TJvxNode);
begin
  if (Element is TJvxHTMLCustomInlineElement) and not (Element is TJvxHTMLCustomLabel) then Exit;
  inherited;
end;

{ TJvxHTMLCustomImage }

constructor TJvxHTMLCustomImage.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('img');
  TJvxExposedNode(Alt).NodeName := 'alt';
  TJvxExposedNode(IsMap).NodeName := 'ismap';
  TJvxExposedNode(LongDesc).NodeName := 'longdesc';
  TJvxExposedNode(Source).NodeName := 'src';
  TJvxExposedNode(UseMap).NodeName := 'usemap';
end;

procedure TJvxHTMLCustomImage.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAlt, 'FAlt', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FIsMap, 'FIsMap', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FLongDesc, 'FLongDesc', TJvxHTMLURIAttribute, Data);
  DefineAttribute(FSource, 'FSource', TJvxHTMLURIAttribute, Data);
  DefineAttribute(FUseMap, 'FUseMap', TJvxHTMLURIAttribute, Data);
end;

{ TJvxHTMLCustomLegend }

constructor TJvxHTMLCustomLegend.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('legend');
  TJvxExposedNode(AccessKey).NodeName := 'accesskey';
  Align.ValidValues := [haTop, haBottom, haLeft, haRight];
end;

procedure TJvxHTMLCustomLegend.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAccessKey, 'FAccessKey', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FAlign, 'FAlign', TJvxHTMLAlignAttribute, Data);
end;

procedure TJvxHTMLCustomLegend.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomInlineElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomFieldSet }

constructor TJvxHTMLCustomFieldSet.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('fieldset');
end;

procedure TJvxHTMLCustomFieldSet.DefineElements(
  DefineElement: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineElement(FLegend, 'FLegend', TJvxHTMLLegend, Data);
end;

procedure TJvxHTMLCustomFieldSet.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomInlineElement then Exit;
  if Element is TJvxHTMLCustomBlockElement then Exit;
  if Element is TJvxHTMLCustomScript then Exit;
  if Element is TJvxHTMLCustomForm then Exit;
  inherited;
end;

{ TJvxHTMLCustomTitle }

constructor TJvxHTMLCustomTitle.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('title');
end;

{ TJvxHTMLCustomHead }

constructor TJvxHTMLCustomHead.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('head');
  TJvxExposedNode(Profile).NodeName := 'profile';
end;

procedure TJvxHTMLCustomHead.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FProfile, 'FProfile', TJvxHTMLURIAttribute, Data);
end;

procedure TJvxHTMLCustomHead.DefineElements(
  DefineElement: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineElement(FTitle, 'FTitle', TJvxHTMLTitle, Data);
end;

{ TJvxHTMLCustomBodyElement }

constructor TJvxHTMLCustomBodyElement.Create(Owner: TComponent);
begin
  inherited;
  TJvxExposedNode(OnLoad).NodeName := 'onload';
  TJvxExposedNode(OnUnload).NodeName := 'onunload';
end;

procedure TJvxHTMLCustomBodyElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FOnLoad, 'FOnLoad', TJvxHTMLScriptAttribute, Data);
  DefineAttribute(FOnUnload, 'FOnUnload', TJvxHTMLScriptAttribute, Data);
end;

{ TJvxHTMLCustomBody }

constructor TJvxHTMLCustomBody.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('body');
end;

procedure TJvxHTMLCustomBody.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomBlockElement then Exit;
  if Element is TJvxHTMLScript then Exit;
  inherited;
end;

{ TJvxHTMLCustomRootElement }

constructor TJvxHTMLCustomRootElement.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('html');
end;

procedure TJvxHTMLCustomRootElement.DefineElements(
  DefineElement: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineElement(FHead, 'FHead', TJvxHTMLHead, Data);
end;

{ TJvxCustomHTMLElement }

procedure TJvxCustomHTMLElement.DefineElements(
  DefineElement: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineElement(FBody, 'FBody', TJvxHTMLBody, Data);
end;

{ TJvxHTMLCustomFrameScrollingAttribute }

constructor TJvxHTMLCustomFrameScrollingAttribute.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('scrolling');
end;

function TJvxHTMLCustomFrameScrollingAttribute.GetValue: TJvxHTMLFrameScrolling;
begin
  Result := TJvxHTMLFrameScrolling(AsInteger);
end;

function TJvxHTMLCustomFrameScrollingAttribute.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

procedure TJvxHTMLCustomFrameScrollingAttribute.InternalSetAsInteger(
  Value: Integer);
begin
  inherited;
  FValue := TJvxHTMLFrameScrolling(Value);
end;

function TJvxHTMLCustomFrameScrollingAttribute.OrdToStr(Ord: TJvxOrdinal): string;
begin
  Result := FrameScrollingStrings[TJvxHTMLFrameScrolling(Ord)];
end;

procedure TJvxHTMLCustomFrameScrollingAttribute.SetValue(Value: TJvxHTMLFrameScrolling);
begin
  AsInteger := Ord(Value);
end;

function TJvxHTMLCustomFrameScrollingAttribute.StrToOrd(const Str: string): TJvxOrdinal;
begin
  for Result := Ord(Low(FrameScrollingStrings)) to Ord(High(FrameScrollingStrings)) do
    if SameText(FrameScrollingStrings[TJvxHTMLFrameScrolling(Result)], Str) then
      Exit;
  raise EConvertError.CreateFmt(SInvalidFrameScrolling, [Str]);
end;

class function TJvxHTMLCustomFrameScrollingAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(TJvxHTMLFrameScrolling);
end;

{ TJvxHTMLFrameBorderAttribute }

constructor TJvxHTMLFrameBorderAttribute.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('frameborder');
end;

function TJvxHTMLFrameBorderAttribute.OrdToStr(Ord: TJvxOrdinal): string;
begin
  Result := IntToStr(Ord);
end;

function TJvxHTMLFrameBorderAttribute.StrToOrd(const Str: string): TJvxOrdinal;
begin
  Result := StrToInt(Str);
end;

{ TJvxHTMLCustomAlignAttribute }

constructor TJvxHTMLCustomAlignAttribute.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('align');
end;

function TJvxHTMLCustomAlignAttribute.GetValue: TJvxHTMLAlign;
begin
  Result := TJvxHTMLAlign(AsInteger);
end;

function TJvxHTMLCustomAlignAttribute.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

procedure TJvxHTMLCustomAlignAttribute.InternalSetAsInteger(
  Value: Integer);
begin
  inherited;
  FValue := TJvxHTMLAlign(Value);
end;

function TJvxHTMLCustomAlignAttribute.OrdToStr(Ord: TJvxOrdinal): string;
begin
  Result := AlignStrings[TJvxHTMLAlign(Ord)];
end;

procedure TJvxHTMLCustomAlignAttribute.SetValidValues(AValue: TJvxHTMLAligns);
var
  I: TJvxHTMLAlign;
begin
  if FValidValues <> AValue then
  begin
    if AValue = [] then
      raise EInvalidOperation.Create(SInvalidAlignSet);

    FValidValues := AValue;

    if Assigned and not (Value in FValidValues) then
      for I := Low(TJvxHTMLAlign) to High(TJvxHTMLAlign) do
        if I in FValidValues then
        begin
          Value := I;
          Exit;
        end;
  end;
end;

procedure TJvxHTMLCustomAlignAttribute.SetValue(Value: TJvxHTMLAlign);
begin
  if not (Value in FValidValues) then
    raise JvxValueError(GetEnumName(System.TypeInfo(TJvxHTMLAlign), Ord(Value)));
  AsInteger := Ord(Value);
end;

function TJvxHTMLCustomAlignAttribute.StrToOrd(const Str: string): TJvxOrdinal;
begin
  for Result := Ord(Low(AlignStrings)) to Ord(High(AlignStrings)) do
    if SameText(AlignStrings[TJvxHTMLAlign(Result)], Str) then
      Exit;
  raise EConvertError.CreateFmt(SInvalidAlign, [Str]);
end;

class function TJvxHTMLCustomAlignAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(TJvxHTMLAlign);
end;

{ TJvxHTMLAlignAttribute }

constructor TJvxHTMLAlignAttribute.Create(Owner: TComponent);
begin
  inherited;
  ValidValues := [haTop, haMiddle, haBottom, haLeft, haRight];
end;

{ TJvxHTMLCustomFrame }

constructor TJvxHTMLCustomFrame.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('frame');
  TJvxExposedNode(FrameName).NodeName := 'name';
  TJvxExposedNode(LongDesc).NodeName := 'longdesc';
  TJvxExposedNode(MarginHeight).NodeName := 'marginheight';
  TJvxExposedNode(MarginWidth).NodeName := 'marginwidth';
  TJvxExposedNode(NoResize).NodeName := 'noresize';
  TJvxExposedNode(Src).NodeName := 'src';
end;

procedure TJvxHTMLCustomFrame.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FFrameBorder, 'FFrameBorder', TJvxHTMLFrameBorderAttribute, Data);
  DefineAttribute(FFrameName, 'FFrameName', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FLongDesc, 'FLongDesc', TJvxHTMLURIAttribute, Data);
  DefineAttribute(FMarginHeight, 'FMarginHeight', TJvxProtectedIntegerAttribute, Data);
  DefineAttribute(FMarginWidth, 'FMarginWidth', TJvxProtectedIntegerAttribute, Data);
  DefineAttribute(FNoResize, 'FNoResize', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FSrolling, 'FSrolling', TJvxHTMLFrameScrollingAttribute, Data);
  DefineAttribute(FSrc, 'FSrc', TJvxHTMLURIAttribute, Data);
end;

{ TJvxHTMLCustomInlineFrame }

constructor TJvxHTMLCustomInlineFrame.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('iframe');
  TJvxExposedNode(Height).NodeName := 'height';
  TJvxExposedNode(Width).NodeName := 'width';
end;

procedure TJvxHTMLCustomInlineFrame.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAlign, 'FAlign', TJvxHTMLAlignAttribute, Data);
  DefineAttribute(FHeight, 'FHeight', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FWidth, 'FWidth', TJvxProtectedStringAttribute, Data);
end;

{ TJvxHTMLCustomScript }

constructor TJvxHTMLCustomScript.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('script');
  TJvxExposedNode(Src).NodeName := 'src';
end;

procedure TJvxHTMLCustomScript.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FSrc, 'FSrc', TJvxHTMLURIAttribute, Data);
end;

{ TJvxHTMLCustomTableCaption }

constructor TJvxHTMLCustomTableCaption.Create(Onwer: TComponent);
begin
  inherited;
  SetNodeName('caption');
end;

{ TJvxHTMLCustomTableElement }

constructor TJvxHTMLCustomTableElement.Create(Onwer: TComponent);
begin
  inherited;
  TJvxExposedNode(AlignChar).NodeName := 'char';
  TJvxExposedNode(AlignCharOffset).NodeName := 'charoff';
  TJvxExposedNode(Height).NodeName := 'height';
  TJvxExposedNode(VerticalAlign).NodeName := 'valign';
  TJvxExposedNode(Width).NodeName := 'width';

  Align.ValidValues := [haLeft, haCenter, haRight, haJustify, haChar];
  VerticalAlign.ValidValues := [haTop, haMiddle, haBottom, haBaseLine];
end;

procedure TJvxHTMLCustomTableElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAlign, 'FAlign', TJvxHTMLAlignAttribute, Data);
  DefineAttribute(FAlignChar, 'FAlignChar', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FAlignCharOffset, 'FAlignCharOffset', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FHeight, 'FHeight', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FVerticalAlign, 'FVerticalAlign', TJvxHTMLAlignAttribute, Data);
  DefineAttribute(FWidth, 'FWidth', TJvxProtectedStringAttribute, Data);
end;

{ TJvxHTMLCustomTableColumnElement }

constructor TJvxHTMLCustomTableColumnElement.Create(Onwer: TComponent);
begin
  inherited;
  TJvxExposedNode(Span).NodeName := 'span';
end;

procedure TJvxHTMLCustomTableColumnElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FSpan, 'FSpan', TJvxProtectedIntegerAttribute, Data);
end;

{ TJvxHTMLCustomTableColumn }

constructor TJvxHTMLCustomTableColumn.Create(Onwer: TComponent);
begin
  inherited;
  SetNodeName('col');
end;

{ TJvxHTMLCustomTableColumnGroup }

constructor TJvxHTMLCustomTableColumnGroup.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('colgroup');
end;

procedure TJvxHTMLCustomTableColumnGroup.ValidateElement(
  Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomTableColumn then Exit;
  inherited;
end;

{ TJvxHTMLCustomTableRowGroupElement }

procedure TJvxHTMLCustomTableRowGroupElement.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomTableRow then Exit;
  inherited;
end;

{ TJvxHTMLCustomTableHeader }

constructor TJvxHTMLCustomTableHeader.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('thead');
end;

{ TJvxHTMLCustomTableBody }

constructor TJvxHTMLCustomTableBody.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('tbody');
end;

{ TJvxHTMLCustomTableFooter }

constructor TJvxHTMLCustomTableFooter.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('tfoot');
end;

{ TJvxHTMLCustomTableRow }

constructor TJvxHTMLCustomTableRow.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('tr');
end;

procedure TJvxHTMLCustomTableRow.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomTableCellElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomTableCellElement }

constructor TJvxHTMLCustomTableCellElement.Create(Owner: TComponent);
begin
  inherited;
  TJvxExposedNode(ColSpan).NodeName := 'colspan';
  TJvxExposedNode(NoWrap).NodeName := 'nowrap';
  TJvxExposedNode(RowSpan).NodeName := 'rowspan';
end;

procedure TJvxHTMLCustomTableCellElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FColSpan, 'FColSpan', TJvxProtectedIntegerAttribute, Data);
  DefineAttribute(FNoWrap, 'FNoWrap', TJvxHTMLBooleanAttribute, Data);
  DefineAttribute(FRowSpan, 'FRowSpan', TJvxProtectedIntegerAttribute, Data);
end;

procedure TJvxHTMLCustomTableCellElement.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomTableHeaderCell }

constructor TJvxHTMLCustomTableHeaderCell.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('th');
end;

{ TJvxHTMLCustomTableDataCell }

constructor TJvxHTMLCustomTableDataCell.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('td');
end;

{ TJvxHTMLCustomTable }

constructor TJvxHTMLCustomTable.Create(Onwer: TComponent);
begin
  inherited;
  SetNodeName('table');
  TJvxExposedNode(Border).NodeName := 'border';
  TJvxExposedNode(CellPadding).NodeName := 'cellpadding';
  TJvxExposedNode(CellSpacing).NodeName := 'cellspacing';
  TJvxExposedNode(Summary).NodeName := 'summary';
end;

procedure TJvxHTMLCustomTable.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FBorder, 'FBorder', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FCellPadding, 'FCellPadding', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FCellSpacing, 'FCellSpacing', TJvxProtectedStringAttribute, Data);
  DefineAttribute(FSummary, 'FSummary', TJvxProtectedStringAttribute, Data);
end;

procedure TJvxHTMLCustomTable.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxHTMLCustomTableCaption then Exit;
  if Element is TJvxHTMLCustomTableColumnGroup then Exit;
  if Element is TJvxHTMLCustomTableColumn then Exit;
  if Element is TJvxHTMLCustomTableRowGroupElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomXMLDataIsland }

constructor TJvxHTMLCustomXMLDataIsland.Create(Onwer: TComponent);
begin
  inherited;
  SetNodeName('xml');
  TJvxExposedNode(Src).NodeName := 'src';
end;

procedure TJvxHTMLCustomXMLDataIsland.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FSrc, 'FSrc', TJvxHTMLURIAttribute, Data);
end;

procedure TJvxHTMLCustomXMLDataIsland.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxElement then Exit;
  inherited;
end;

{ TJvxHTMLCustomImportInstruction }

constructor TJvxHTMLCustomImportInstruction.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('import');
  TJvxExposedNode(ElementImplementation).NodeName := 'implementation';
  TJvxExposedNode(NameSpace).NodeName := 'namespace';
end;

procedure TJvxHTMLCustomImportInstruction.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FElementImplementation, 'FElementImplementation',
    TJvxHTMLURIAttribute, Data);
  DefineAttribute(FNameSpace, 'FNameSpace', TJvxProtectedStringAttribute, Data);
end;

end.
