
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxStdNodes.pas.                                       }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxStdNodes;

{$include Jvx.inc}

interface

uses
  SysUtils, Classes, TypInfo, JvxClasses, JvxUtils;

type

  { todo: test TJvxOrdinalAttribute }

  TJvxOrdinalAttribute = class(TJvxAttribute)
  private
    FAssigned: Boolean;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsString: string; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;

    function OrdToStr(Value: TJvxOrdinal): string; virtual; abstract;
    function StrToOrd(const Value: string): TJvxOrdinal; virtual; abstract;
    class function TypeInfo: PTypeInfo; virtual; abstract;
  end;

  { todo: test TJvxOrdinalElement }

  TJvxOrdinalElement = class(TJvxElement)
  private
    FAssigned: Boolean;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsString: string; override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;

    function OrdToStr(Value: TJvxOrdinal): string; virtual; abstract;
    function StrToOrd(const Value: string): TJvxOrdinal; virtual; abstract;
    class function TypeInfo: PTypeInfo; virtual; abstract;
  end;

  { todo: test TJvxEnumeratedAttribute }

  TJvxEnumeratedAttribute = class(TJvxOrdinalAttribute)
  protected
    procedure InternalLoadData(DataField: TJvxDataField); override;
    procedure ReadValue(Reader: TReader); override;
    function OrdToStr(Value: TJvxOrdinal): string; override;
    function StrToOrd(const Value: string): TJvxOrdinal; override;
    procedure WriteValue(Writer: TWriter); override;
  end;

  { todo: test TJvxEnumeratedElement }

  TJvxEnumeratedElement = class(TJvxOrdinalElement)
  protected
    procedure InternalLoadData(DataField: TJvxDataField); override;
    procedure ReadValue(Reader: TReader); override;
    function OrdToStr(Value: TJvxOrdinal): string; override;
    function StrToOrd(const Value: string): TJvxOrdinal; override;
    procedure WriteValue(Writer: TWriter); override;
  end;

  { todo: test TJvxBooleanAttribute }

  TJvxCustomBooleanAttribute = class(TJvxEnumeratedAttribute)
  private
    FValue: Boolean;
  protected
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    class function TypeInfo: PTypeInfo; override;

    property Value: Boolean read GetAsBoolean write SetAsBoolean stored False;
  end;

  TJvxProtectedBooleanAttribute = class(TJvxCustomBooleanAttribute)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxBooleanAttribute = class(TJvxProtectedBooleanAttribute)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxBooleanElement }

  TJvxCustomBooleanElement = class(TJvxEnumeratedElement)
  private
    FValue: Boolean;
  protected
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    class function TypeInfo: PTypeInfo; override;

    property Value: Boolean read GetAsBoolean write SetAsBoolean stored False;
  end;

  TJvxProtectedBooleanElement = class(TJvxCustomBooleanElement)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxBooleanElement = class(TJvxProtectedBooleanElement)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxDateTimeAttribute }

  TJvxCustomDateTimeAttribute = class(TJvxAttribute)
  private
    FAssigned: Boolean;
    FValue: TDateTime;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function DateTimeToStr(Value: TDateTime): string; virtual;
    function GetAssigned: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    function StrToDateTime(const Value: string): TDateTime; virtual;
    procedure WriteValue(Writer: TWriter); override;

    property Value: TDateTime read GetAsDateTime write SetAsDateTime stored False;
  end;

  TJvxProtectedDateTimeAttribute = class(TJvxCustomDateTimeAttribute)
  private
    FFormat: string;
    procedure SetFormat(const Value: string);
  protected
    function DateTimeToStr(DateTime: TDateTime): string; override;
  published
    property Assigned;
    property DataField;
    property Format: string read FFormat write SetFormat;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxDateTimeAttribute = class(TJvxProtectedDateTimeAttribute)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxDateTimeElement }

  TJvxCustomDateTimeElement = class(TJvxElement)
  private
    FAssigned: Boolean;
    FValue: TDateTime;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function DateTimeToStr(Value: TDateTime): string; virtual;
    function GetAssigned: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    function StrToDateTime(const Value: string): TDateTime; virtual;
    procedure WriteValue(Writer: TWriter); override;

    property Value: TDateTime read GetAsDateTime write SetAsDateTime stored False;
  end;

  TJvxProtectedDateTimeElement = class(TJvxCustomDateTimeElement)
  private
    FFormat: string;
    procedure SetFormat(const Value: string);
  protected
    function DateTimeToStr(DateTime: TDateTime): string; override;
  published
    property Assigned;
    property DataField;
    property Format: string read FFormat write SetFormat;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxDateTimeElement = class(TJvxProtectedDateTimeElement)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxFloatAttribute }

  TJvxCustomFloatAttribute = class(TJvxAttribute)
  private
    FAssigned: Boolean;
    FValue: Extended;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function FloatToStr(Value: Extended): string; virtual;
    function GetAssigned: Boolean; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    function StrToFloat(const Value: string): Extended; virtual;
    procedure WriteValue(Writer: TWriter); override;

    property Value: Extended read GetAsFloat write SetAsFloat stored False;
  end;

  TJvxProtectedFloatAttribute = class(TJvxCustomFloatAttribute)
  private
    FFormat: string;
    procedure SetFormat(const Value: string);
  protected
    function FloatToStr(Float: Extended): string; override;
  published
    property Assigned;
    property DataField;
    property Format: string read FFormat write SetFormat;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxFloatAttribute = class(TJvxProtectedFloatAttribute)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxFloatElement }

  TJvxCustomFloatElement = class(TJvxElement)
  private
    FAssigned: Boolean;
    FValue: Extended;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function FloatToStr(Value: Extended): string; virtual;
    function GetAssigned: Boolean; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    function StrToFloat(const Value: string): Extended; virtual;
    procedure WriteValue(Writer: TWriter); override;

    property Value: Extended read GetAsFloat write SetAsFloat stored False;
  end;

  TJvxProtectedFloatElement = class(TJvxCustomFloatElement)
  private
    FFormat: string;
    procedure SetFormat(const Value: string);
  protected
    function FloatToStr(Float: Extended): string; override;
  published
    property Assigned;
    property DataField;
    property Format: string read FFormat write SetFormat;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxFloatElement = class(TJvxProtectedFloatElement)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxIntegerAttribute }

  TJvxCustomIntegerAttribute = class(TJvxOrdinalAttribute)
  private
    FValue: Integer;
  protected
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Value: TJvxOrdinal): string; override;
    function StrToOrd(const Value: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;

    property Value: Integer read GetAsInteger write SetAsInteger stored False;
  end;

  TJvxProtectedIntegerAttribute = class(TJvxCustomIntegerAttribute)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxIntegerAttribute = class(TJvxProtectedIntegerAttribute)
  published
    property NodeName write SetNodeName;
  end;

  { todo: test TJvxIntegerElement }

  TJvxCustomIntegerElement = class(TJvxOrdinalElement)
  private
    FValue: Integer;
  protected
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    function OrdToStr(Value: TJvxOrdinal): string; override;
    function StrToOrd(const Value: string): TJvxOrdinal; override;
    class function TypeInfo: PTypeInfo; override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;

    property Value: Integer read GetAsInteger write SetAsInteger stored False;
  end;

  TJvxProtectedIntegerElement = class(TJvxCustomIntegerElement)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxIntegerElement = class(TJvxProtectedIntegerElement)
  published
    property NodeName write SetNodeName;
  end;

  { TJvxStringAttribute }

  TJvxCustomStringAttribute = class(TJvxAttribute)
  private
    FValue: string;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;

    property Value: string read GetAsString write SetAsString stored False;
  end;

  TJvxProtectedStringAttribute = class(TJvxCustomStringAttribute)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxStringAttribute = class(TJvxProtectedStringAttribute)
  published
    property NodeName write SetNodeName;
  end;

  { TJvxStringElement }

  TJvxCustomStringElement = class(TJvxElement)
  private
    FValue: string;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;

    property Value: string read GetAsString write SetAsString stored False;
  end;

  TJvxProtectedStringElement = class(TJvxCustomStringElement)
  published
    property Assigned;
    property DataField;
    property OnLoadData;
    property OnNotify;
    property Value;
  end;

  TJvxStringElement = class(TJvxProtectedStringElement)
  published
    property NodeName write SetNodeName;
  end;

  { TJvxCompositeTextElement }

  TJvxCustomCompositeTextElement = class(TJvxCompositeElement)
  private
    FText: TStrings;
    function GetText: TStrings;
    procedure TextChanged(Sender: TObject);
  protected
    procedure AssignValue(Source: TJvxNode); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;

    property Text: TStrings read GetText stored False;
  public
    destructor Destroy; override;
  end;

  TJvxProtectedCompositeTextElement = class(TJvxCustomCompositeTextElement)
  published
    property Assigned;
    property DataField;
    property DataSource;
    property OnNotify;
    property Text;
  end;

  TJvxCompositeTextElement = class(TJvxProtectedCompositeTextElement)
  protected
    procedure ValidateAttribute(Attribute: TJvxNode); override;
    procedure ValidateElement(Element: TJvxNode); override;
  published
    property NodeName write SetNodeName;
    property Template write SetTemplate;
  end;

implementation

{ TJvxOrdinalAttribute }

procedure TJvxOrdinalAttribute.AssignValue(Source: TJvxNode);
begin
  AsInteger := Source.AsInteger;
end;

function TJvxOrdinalAttribute.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsInteger = Source.AsInteger;
end;

function TJvxOrdinalAttribute.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxOrdinalAttribute.InternalGetAsString: string;
begin
  Result := OrdToStr(InternalGetAsInteger);
end;

procedure TJvxOrdinalAttribute.InternalSetAsInteger(Value: Integer);
begin
{$ifopt R+}
  JvxCheckRange(TypeInfo, Value);
{$endif}
end;

procedure TJvxOrdinalAttribute.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxOrdinalAttribute.InternalSetAsString(const Value: string);
begin
  InternalSetAsInteger(StrToOrd(Value));
end;

{ TJvxOrdinalElement }

procedure TJvxOrdinalElement.AssignValue(Source: TJvxNode);
begin
  AsInteger := Source.AsInteger;
end;

function TJvxOrdinalElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsInteger = Source.AsInteger;
end;

function TJvxOrdinalElement.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxOrdinalElement.InternalGetAsString: string;
begin
  Result := OrdToStr(InternalGetAsInteger);
end;

procedure TJvxOrdinalElement.InternalSetAsInteger(Value: Integer);
begin
{$ifopt R+}
  JvxCheckRange(TypeInfo, Value);
{$endif}
end;

procedure TJvxOrdinalElement.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxOrdinalElement.InternalSetAsString(const Value: string);
begin
  InternalSetAsInteger(StrToOrd(Value));
end;

{ TJvxEnumeratedAttribute }

procedure TJvxEnumeratedAttribute.InternalLoadData(DataField: TJvxDataField);
begin
  if (DataField.TypeInfo.Kind = tkEnumeration) and (DataField.TypeInfo = TypeInfo) then
    AsInteger := DataField.AsOrdinal
  else
    inherited;
end;

function TJvxEnumeratedAttribute.OrdToStr(Value: TJvxOrdinal): string;
begin
  Result := GetEnumName(TypeInfo, Value);
end;

procedure TJvxEnumeratedAttribute.ReadValue(Reader: TReader);
begin
  AsInteger := GetEnumValue(TypeInfo, Reader.ReadIdent);
end;

function TJvxEnumeratedAttribute.StrToOrd(const Value: string): TJvxOrdinal;
begin
  Result := GetEnumValue(TypeInfo, Value);
end;

procedure TJvxEnumeratedAttribute.WriteValue(Writer: TWriter);
begin
  Writer.WriteIdent(GetEnumName(TypeInfo, AsInteger));
end;

{ TJvxEnumeratedElement }

procedure TJvxEnumeratedElement.InternalLoadData(DataField: TJvxDataField);
begin
  if (DataField.TypeInfo.Kind = tkEnumeration) and (DataField.TypeInfo = TypeInfo) then
    AsInteger := DataField.AsOrdinal
  else
    inherited;
end;

function TJvxEnumeratedElement.OrdToStr(Value: TJvxOrdinal): string;
begin
  Result := GetEnumName(TypeInfo, Value);
end;

procedure TJvxEnumeratedElement.ReadValue(Reader: TReader);
begin
  AsInteger := GetEnumValue(TypeInfo, Reader.ReadIdent);
end;

function TJvxEnumeratedElement.StrToOrd(const Value: string): TJvxOrdinal;
begin
  Result := GetEnumValue(TypeInfo, Value);
end;

procedure TJvxEnumeratedElement.WriteValue(Writer: TWriter);
begin
  Writer.WriteIdent(GetEnumName(TypeInfo, AsInteger));
end;

{ TJvxCustomBooleanAttribute }

function TJvxCustomBooleanAttribute.InternalGetAsBoolean: Boolean;
begin
  Result := FValue;
end;

function TJvxCustomBooleanAttribute.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

function TJvxCustomBooleanAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomBooleanAttribute.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := Value;
end;

procedure TJvxCustomBooleanAttribute.InternalSetAsInteger(Value: Integer);
begin
  inherited;
  FValue := Boolean(Value);
end;

class function TJvxCustomBooleanAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(Boolean);
end;

{ TJvxCustomBooleanElement }

function TJvxCustomBooleanElement.InternalGetAsBoolean: Boolean;
begin
  Result := FValue;
end;

function TJvxCustomBooleanElement.InternalGetAsInteger: Integer;
begin
  Result := Ord(FValue);
end;

function TJvxCustomBooleanElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomBooleanElement.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := Value;
end;

procedure TJvxCustomBooleanElement.InternalSetAsInteger(Value: Integer);
begin
  inherited;
  FValue := Boolean(Value);
end;

class function TJvxCustomBooleanElement.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(Boolean);
end;

{ TJvxCustomDateTimeAttribute }

procedure TJvxCustomDateTimeAttribute.AssignValue(Source: TJvxNode);
begin
  AsDateTime := Source.AsDateTime;
end;

function TJvxCustomDateTimeAttribute.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsDateTime = Source.AsDateTime;
end;

function TJvxCustomDateTimeAttribute.DateTimeToStr(
  Value: TDateTime): string;
begin
  Result := SysUtils.DateTimeToStr(Value);
end;

function TJvxCustomDateTimeAttribute.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxCustomDateTimeAttribute.InternalGetAsDateTime: TDateTime;
begin
  Result := FValue;
end;

function TJvxCustomDateTimeAttribute.InternalGetAsString: string;
begin
  Result := DateTimeToStr(FValue);
end;

function TJvxCustomDateTimeAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomDateTimeAttribute.InternalSetAsDateTime(Value: TDateTime);
begin
  FValue := Value;
end;

procedure TJvxCustomDateTimeAttribute.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxCustomDateTimeAttribute.InternalSetAsString(const Value: string);
begin
  FValue := StrToDateTime(Value);
end;

procedure TJvxCustomDateTimeAttribute.ReadValue(Reader: TReader);
begin
  Value := Reader.ReadDate;
end;

function TJvxCustomDateTimeAttribute.StrToDateTime(
  const Value: string): TDateTime;
begin
  Result := SysUtils.StrToDateTime(Value);
end;

procedure TJvxCustomDateTimeAttribute.WriteValue(Writer: TWriter);
begin
  Writer.WriteDate(Value);
end;

{ TJvxProtectedDateTimeAttribute }

function TJvxProtectedDateTimeAttribute.DateTimeToStr(
  DateTime: TDateTime): string;
begin
  if Format <> EmptyStr then
    Result := FormatDateTime(Format, DateTime)
  else
    Result := inherited DateTimeToStr(DateTime);
end;

procedure TJvxProtectedDateTimeAttribute.SetFormat(const Value: string);
begin
  if FFormat <> Value then
  begin
    FFormat := Value;
    Notify(noChange);
  end;
end;

{ TJvxCustomDateTimeElement }

procedure TJvxCustomDateTimeElement.AssignValue(Source: TJvxNode);
begin
  AsDateTime := Source.AsDateTime;
end;

function TJvxCustomDateTimeElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsDateTime = Source.AsDateTime;
end;

function TJvxCustomDateTimeElement.DateTimeToStr(
  Value: TDateTime): string;
begin
  Result := SysUtils.DateTimeToStr(Value);
end;

function TJvxCustomDateTimeElement.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxCustomDateTimeElement.InternalGetAsDateTime: TDateTime;
begin
  Result := FValue;
end;

function TJvxCustomDateTimeElement.InternalGetAsString: string;
begin
  Result := DateTimeToStr(FValue);
end;

function TJvxCustomDateTimeElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomDateTimeElement.InternalSetAsDateTime(Value: TDateTime);
begin
  FValue := Value;
end;

procedure TJvxCustomDateTimeElement.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxCustomDateTimeElement.InternalSetAsString(const Value: string);
begin
  FValue := StrToDateTime(Value);
end;

procedure TJvxCustomDateTimeElement.ReadValue(Reader: TReader);
begin
  Value := Reader.ReadDate;
end;

function TJvxCustomDateTimeElement.StrToDateTime(
  const Value: string): TDateTime;
begin
  Result := SysUtils.StrToDateTime(Value);
end;

procedure TJvxCustomDateTimeElement.WriteValue(Writer: TWriter);
begin
  Writer.WriteDate(Value);
end;

{ TJvxProtectedDateTimeElement }

function TJvxProtectedDateTimeElement.DateTimeToStr(
  DateTime: TDateTime): string;
begin
  if Format <> EmptyStr then
    Result := FormatDateTime(Format, DateTime)
  else
    Result := inherited DateTimeToStr(DateTime);
end;

procedure TJvxProtectedDateTimeElement.SetFormat(const Value: string);
begin
  if FFormat <> Value then
  begin
    FFormat := Value;
    Notify(noChange);
  end;
end;

{ TJvxCustomFloatAttribute }

procedure TJvxCustomFloatAttribute.AssignValue(Source: TJvxNode);
begin
  AsFloat := Source.AsFloat;
end;

function TJvxCustomFloatAttribute.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsFloat = Source.AsFloat;
end;

function TJvxCustomFloatAttribute.FloatToStr(Value: Extended): string;
begin
  Result := SysUtils.FloatToStr(Value);
end;

function TJvxCustomFloatAttribute.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxCustomFloatAttribute.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TJvxCustomFloatAttribute.InternalGetAsInteger: Integer;
begin
  Result := JvxFloatToInt(FValue);
end;

function TJvxCustomFloatAttribute.InternalGetAsString: string;
begin
  Result := FloatToStr(FValue);
end;

function TJvxCustomFloatAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomFloatAttribute.InternalSetAsFloat(Value: Extended);
begin
  FValue := Value;
end;

procedure TJvxCustomFloatAttribute.InternalSetAsInteger(Value: Integer);
begin
  FValue := Value;
end;

procedure TJvxCustomFloatAttribute.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxCustomFloatAttribute.InternalSetAsString(
  const Value: string);
begin
  FValue := StrToFloat(Value);
end;

procedure TJvxCustomFloatAttribute.ReadValue(Reader: TReader);
begin
  Value := Reader.ReadFloat;
end;

function TJvxCustomFloatAttribute.StrToFloat(const Value: string): Extended;
begin
  Result := SysUtils.StrToFloat(Value);
end;

procedure TJvxCustomFloatAttribute.WriteValue(Writer: TWriter);
begin
  Writer.WriteFloat(Value);
end;

{ TJvxProtectedFloatAttribute }

function TJvxProtectedFloatAttribute.FloatToStr(Float: Extended): string;
begin
  if Format <> EmptyStr then
    Result := FormatFloat(Format, Float)
  else
    Result := inherited FloatToStr(Float);
end;

procedure TJvxProtectedFloatAttribute.SetFormat(const Value: string);
begin
  if FFormat <> Value then
  begin
    FFormat := Value;
    Notify(noChange);
  end;
end;

{ TJvxCustomFloatElement }

procedure TJvxCustomFloatElement.AssignValue(Source: TJvxNode);
begin
  AsFloat := Source.AsFloat;
end;

function TJvxCustomFloatElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsFloat = Source.AsFloat;
end;

function TJvxCustomFloatElement.FloatToStr(Value: Extended): string;
begin
  Result := SysUtils.FloatToStr(Value);
end;

function TJvxCustomFloatElement.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TJvxCustomFloatElement.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TJvxCustomFloatElement.InternalGetAsInteger: Integer;
begin
  Result := JvxFloatToInt(FValue);
end;

function TJvxCustomFloatElement.InternalGetAsString: string;
begin
  Result := FloatToStr(FValue);
end;

function TJvxCustomFloatElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomFloatElement.InternalSetAsFloat(Value: Extended);
begin
  FValue := Value;
end;

procedure TJvxCustomFloatElement.InternalSetAsInteger(Value: Integer);
begin
  FValue := Value;
end;

procedure TJvxCustomFloatElement.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TJvxCustomFloatElement.InternalSetAsString(
  const Value: string);
begin
  FValue := StrToFloat(Value);
end;

procedure TJvxCustomFloatElement.ReadValue(Reader: TReader);
begin
  Value := Reader.ReadFloat;
end;

function TJvxCustomFloatElement.StrToFloat(const Value: string): Extended;
begin
  Result := SysUtils.StrToFloat(Value);
end;

procedure TJvxCustomFloatElement.WriteValue(Writer: TWriter);
begin
  Writer.WriteFloat(Value);
end;

{ TJvxProtectedFloatElement }

function TJvxProtectedFloatElement.FloatToStr(Float: Extended): string;
begin
  if Format <> EmptyStr then
    Result := FormatFloat(Format, Float)
  else
    Result := inherited FloatToStr(Float);
end;

procedure TJvxProtectedFloatElement.SetFormat(const Value: string);
begin
  if FFormat <> Value then
  begin
    FFormat := Value;
    Notify(noChange);
  end;
end;

{ TJvxCustomIntegerAttribute }

function TJvxCustomIntegerAttribute.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TJvxCustomIntegerAttribute.InternalGetAsInteger: Integer;
begin
  Result := FValue;
end;

function TJvxCustomIntegerAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomIntegerAttribute.InternalSetAsFloat(Value: Extended);
begin
  InternalSetAsInteger(JvxFloatToInt(Value));
end;

procedure TJvxCustomIntegerAttribute.InternalSetAsInteger(Value: Integer);
begin
  inherited;
  FValue := Value;
end;

function TJvxCustomIntegerAttribute.OrdToStr(Value: TJvxOrdinal): string;
begin
  Result := IntToStr(Value);
end;

procedure TJvxCustomIntegerAttribute.ReadValue(Reader: TReader);
begin
  AsInteger := Reader.ReadInteger;
end;

function TJvxCustomIntegerAttribute.StrToOrd(const Value: string): TJvxOrdinal;
begin
  Result := StrToInt(Value);
end;

class function TJvxCustomIntegerAttribute.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(Integer);
end;

procedure TJvxCustomIntegerAttribute.WriteValue(Writer: TWriter);
begin
  Writer.WriteInteger(AsInteger);
end;

{ TJvxCustomIntegerElement }

function TJvxCustomIntegerElement.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TJvxCustomIntegerElement.InternalGetAsInteger: Integer;
begin
  Result := FValue;
end;

function TJvxCustomIntegerElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomIntegerElement.InternalSetAsFloat(Value: Extended);
begin
  InternalSetAsInteger(JvxFloatToInt(Value));
end;

procedure TJvxCustomIntegerElement.InternalSetAsInteger(Value: Integer);
begin
  inherited;
  FValue := Value;
end;

function TJvxCustomIntegerElement.OrdToStr(Value: TJvxOrdinal): string;
begin
  Result := IntToStr(Value);
end;

procedure TJvxCustomIntegerElement.ReadValue(Reader: TReader);
begin
  AsInteger := Reader.ReadInteger;
end;

function TJvxCustomIntegerElement.StrToOrd(const Value: string): TJvxOrdinal;
begin
  Result := StrToInt(Value);
end;

class function TJvxCustomIntegerElement.TypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(Integer);
end;

procedure TJvxCustomIntegerElement.WriteValue(Writer: TWriter);
begin
  Writer.WriteInteger(AsInteger);
end;

{ TJvxCustomStringAttribute }

procedure TJvxCustomStringAttribute.AssignValue(Source: TJvxNode);
begin
  AsString := Source.AsString;
end;

function TJvxCustomStringAttribute.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsString = Source.AsString;
end;

function TJvxCustomStringAttribute.GetAssigned: Boolean;
begin
  Result := FValue <> EmptyStr;
end;

function TJvxCustomStringAttribute.InternalGetAsBoolean: Boolean;
begin
  Result := JvxStrToBool(FValue);
end;

function TJvxCustomStringAttribute.InternalGetAsDateTime: TDateTime;
begin
  Result := StrToDateTime(FValue);
end;

function TJvxCustomStringAttribute.InternalGetAsFloat: Extended;
begin
  Result := StrToFloat(FValue);
end;

function TJvxCustomStringAttribute.InternalGetAsInteger: Integer;
begin
  Result := StrToInt(FValue);
end;

function TJvxCustomStringAttribute.InternalGetAsString: string;
begin
  Result := FValue;
end;

function TJvxCustomStringAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomStringAttribute.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := JvxBoolToStr(Value);
end;

procedure TJvxCustomStringAttribute.InternalSetAsDateTime(
  Value: TDateTime);
begin
  FValue := DateTimeToStr(Value);
end;

procedure TJvxCustomStringAttribute.InternalSetAsFloat(Value: Extended);
begin
  FValue := FloatToStr(Value);
end;

procedure TJvxCustomStringAttribute.InternalSetAsInteger(Value: Integer);
begin
  FValue := IntToStr(Value);
end;

procedure TJvxCustomStringAttribute.InternalSetAssigned(Value: Boolean);
begin
  if not Value then
    FValue := EmptyStr;
end;

procedure TJvxCustomStringAttribute.InternalSetAsString(const Value: string);
begin
  FValue := Value;
end;

procedure TJvxCustomStringAttribute.ReadValue(Reader: TReader);
begin
  AsString := Reader.ReadString;
end;

procedure TJvxCustomStringAttribute.WriteValue(Writer: TWriter);
begin
  Writer.WriteString(AsString);
end;

{ TJvxCustomStringElement }

procedure TJvxCustomStringElement.AssignValue(Source: TJvxNode);
begin
  AsString := Source.AsString;
end;

function TJvxCustomStringElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsString = Source.AsString;
end;

function TJvxCustomStringElement.GetAssigned: Boolean;
begin
  Result := FValue <> EmptyStr;
end;

function TJvxCustomStringElement.InternalGetAsBoolean: Boolean;
begin
  Result := JvxStrToBool(FValue);
end;

function TJvxCustomStringElement.InternalGetAsDateTime: TDateTime;
begin
  Result := StrToDateTime(FValue);
end;

function TJvxCustomStringElement.InternalGetAsFloat: Extended;
begin
  Result := StrToFloat(FValue);
end;

function TJvxCustomStringElement.InternalGetAsInteger: Integer;
begin
  Result := StrToInt(FValue);
end;

function TJvxCustomStringElement.InternalGetAsString: string;
begin
  Result := FValue;
end;

function TJvxCustomStringElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TJvxCustomStringElement.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := JvxBoolToStr(Value);
end;

procedure TJvxCustomStringElement.InternalSetAsDateTime(
  Value: TDateTime);
begin
  FValue := DateTimeToStr(Value);
end;

procedure TJvxCustomStringElement.InternalSetAsFloat(Value: Extended);
begin
  FValue := FloatToStr(Value);
end;

procedure TJvxCustomStringElement.InternalSetAsInteger(Value: Integer);
begin
  FValue := IntToStr(Value);
end;

procedure TJvxCustomStringElement.InternalSetAssigned(Value: Boolean);
begin
  if not Value then
    FValue := EmptyStr;
end;

procedure TJvxCustomStringElement.InternalSetAsString(const Value: string);
begin
  FValue := Value;
end;

procedure TJvxCustomStringElement.ReadValue(Reader: TReader);
begin
  AsString := Reader.ReadString;
end;

procedure TJvxCustomStringElement.WriteValue(Writer: TWriter);
begin
  Writer.WriteString(AsString);
end;

{ TJvxCustomCompositeTextElement }

procedure TJvxCustomCompositeTextElement.AssignValue(Source: TJvxNode);
begin
  AsString := Source.AsString;
end;

destructor TJvxCustomCompositeTextElement.Destroy;
begin
  FText.Free;
  FText := nil;
  inherited;
end;

function TJvxCustomCompositeTextElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsString = Source.AsString;
end;

function TJvxCustomCompositeTextElement.GetAssigned: Boolean;
begin
  Result := False;
  if FText = nil then Exit;
  if InternalGetAsString = EmptyStr then Exit;
  Result := True;
end;

function TJvxCustomCompositeTextElement.GetText: TStrings;
begin
  if FText = nil then
  begin
    FText := TStringList.Create;
    TStringList(FText).OnChange := TextChanged;
  end;
  Result := FText;
end;

function TJvxCustomCompositeTextElement.InternalGetAsBoolean: Boolean;
begin
  Result := JvxStrToBool(InternalGetAsString);
end;

function TJvxCustomCompositeTextElement.InternalGetAsDateTime: TDateTime;
begin
  Result := StrToDateTime(InternalGetAsString);
end;

function TJvxCustomCompositeTextElement.InternalGetAsFloat: Extended;
begin
  Result := StrToFloat(InternalGetAsString);
end;

function TJvxCustomCompositeTextElement.InternalGetAsInteger: Integer;
begin
  Result := StrToInt(InternalGetAsString);
end;

function TJvxCustomCompositeTextElement.InternalGetAsString: string;
begin
  Result := FText.Text;
  if Length(Result) <> 0 then
    SetLength(Result, Length(Result) - 2);
end;

function TJvxCustomCompositeTextElement.InternalGetAsVariant: Variant;
begin
  Result := InternalGetAsString;
end;

procedure TJvxCustomCompositeTextElement.InternalSetAsBoolean(Value: Boolean);
begin
  InternalSetAsString(JvxBoolToStr(Value));
end;

procedure TJvxCustomCompositeTextElement.InternalSetAsDateTime(
  Value: TDateTime);
begin
  InternalSetAsString(DateTimeToStr(Value));
end;

procedure TJvxCustomCompositeTextElement.InternalSetAsFloat(
  Value: Extended);
begin
  InternalSetAsString(FloatToStr(Value));
end;

procedure TJvxCustomCompositeTextElement.InternalSetAsInteger(
  Value: Integer);
begin
  InternalSetAsString(IntToStr(Value));
end;

procedure TJvxCustomCompositeTextElement.InternalSetAssigned(Value: Boolean);
begin
  if not Value then
  begin
    FText.Free;
    FText := nil;
  end;
end;

procedure TJvxCustomCompositeTextElement.InternalSetAsString(
  const Value: string);
begin
  Text.Text := Value
end;

procedure TJvxCustomCompositeTextElement.ReadValue(Reader: TReader);
begin
  Reader.ReadListBegin;
  Text.BeginUpdate;
  try
    Text.Clear;
    while not Reader.EndOfList do
      Text.Add(Reader.ReadString);
  finally
    Text.EndUpdate;
  end;
  Reader.ReadListEnd;
end;

procedure TJvxCustomCompositeTextElement.TextChanged(Sender: TObject);
begin
  if FText.Count = 0 then
  begin
    FText.Free;
    FText := nil;
  end;
  Notify(noChange);
end;

procedure TJvxCustomCompositeTextElement.WriteValue(Writer: TWriter);
var
  I: Integer;
begin
  Writer.WriteListBegin;
  for I := 0 to Text.Count - 1 do
    Writer.WriteString(Text[I]);
  Writer.WriteListEnd;
end;

{ TJvxCompositeTextElement }

procedure TJvxCompositeTextElement.ValidateAttribute(Attribute: TJvxNode);
begin
  if Attribute is TJvxAttribute then Exit;
  inherited;
end;

procedure TJvxCompositeTextElement.ValidateElement(Element: TJvxNode);
begin
  if Element is TJvxElement then Exit;
  inherited;
end;

end.
