
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxTestRunner.pas.                                     }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxTestRunner;

{$include Jvx.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ComCtrls, StdCtrls, AppEvnts, JvxTestClasses;

type
  TJvxTestRunnerForm = class(TForm)
    FTestNameLabel: TLabel;
    FTestNameComboBox: TComboBox;
    FStartButton: TButton;
    FProgressBar: TProgressBar;
    FImage: TImage;
    FStopOnErrorCheckBox: TCheckBox;
    FStopOnFailureCheckBox: TCheckBox;
    FStatusBar: TStatusBar;
    FProgressLabel: TLabel;
    FResultsLabel: TLabel;
    FResultsListView: TListView;
    FApplicationEvents: TApplicationEvents;
    Bevel: TBevel;
    FTotalProgressLabel: TLabel;
    FTotalProgressBar: TProgressBar;
    FStopButton: TButton;
    FBreakOnFailureCheckBox: TCheckBox;
    procedure ApplicationHint(Sender: TObject);
    procedure ApplicationIdle(Sender: TObject; var Done: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure ResultsListViewData(Sender: TObject; Item: TListItem);
    procedure FormDestroy(Sender: TObject);
    procedure FStatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure StopButtonClick(Sender: TObject);
    procedure WriteDebugOutput(Result: TJvxTestResult);
  private
    FAutoStart: Boolean;
    FTestResults: TJvxTestResults;
    FTestRunning: Boolean;
    procedure DrawStatistics(Canvas: TCanvas; Text: string;
      FailureCount: Integer; const Rect: TRect);
    procedure StartTest;
    procedure StopTest;
    procedure TestError(Error: TJvxTestFailure);
    procedure TestFailure(Failure: TJvxTestFailure);
    procedure TestStarted(Test: TJvxTest);
    procedure TestStopped(Test: TJvxTest);
    procedure UpdateForm;
    procedure UpdateResults;
  public
    property AutoStart: Boolean read FAutoStart write FAutoStart;
  end;

procedure JvxRunTests(AutoStart: Boolean = True); overload;

implementation

{$R *.DFM}

resourcestring
  SCloseButtonCaption = '&Close';
  SStopButtonCaption = '&Stop';
  SErrorsStatusFormat = 'Errors: %d';
  SFailuresStatusFormat = 'Failures: %d';
  SNoTestsFound = '<No tests found>';
  SAllTests = '<All tests>';

procedure JvxRunTests(AutoStart: Boolean = True);
var
  Form: TJvxTestRunnerForm;
begin
  Application.CreateForm(TJvxTestRunnerForm, Form);
  try
    Form.AutoStart := AutoStart;
    Form.ShowModal;
  finally
    Form.Free;
  end;
end;

{ TJvxTestRunnerForm }

procedure TJvxTestRunnerForm.ApplicationHint(Sender: TObject);
begin
  with FStatusBar do
    Panels[Panels.Count - 1].Text := GetLongHint(Application.Hint);
end;

procedure TJvxTestRunnerForm.ApplicationIdle(Sender: TObject;
  var Done: Boolean);
begin
  UpdateForm;
  if AutoStart then
  begin
    AutoStart := False;
    StartTest;
  end;
end;

procedure TJvxTestRunnerForm.FormCreate(Sender: TObject);

  procedure InitializeTestNames;
  begin
    JvxGetTestClassNames(FTestNameComboBox.Items);
    if FTestNameComboBox.Items.Count = 0 then
      FTestNameComboBox.Items.Insert(0, SNoTestsFound)
    else
      FTestNameComboBox.Items.Insert(0, SAllTests);
    FTestNameComboBox.ItemIndex := 0;
  end;

  procedure InitializeTestResults;
  begin
    FTestResults := TJvxTestResults.Create;
    FTestResults.OnTestError   := TestError;
    FTestResults.OnTestFailure := TestFailure;
    FTestResults.OnTestStarted := TestStarted;
    FTestResults.OnTestStopped := TestStopped;
  end;

begin
  InitializeTestNames;
  InitializeTestResults;
end;

procedure TJvxTestRunnerForm.FormDestroy(Sender: TObject);
begin
  FTestResults.Free;
end;

procedure TJvxTestRunnerForm.ResultsListViewData(Sender: TObject;
  Item: TListItem);
begin
  Item.Caption := FTestResults.Results[Item.Index].TestName;
  Item.SubItems.Add(FTestResults.Results[Item.Index].Message);
end;

procedure TJvxTestRunnerForm.StartButtonClick(Sender: TObject);
begin
  StartTest;
end;

procedure TJvxTestRunnerForm.StopButtonClick(Sender: TObject);
begin
  if FTestRunning then
    StopTest
  else
    ModalResult := mrCancel;
end;

procedure TJvxTestRunnerForm.StartTest;

  procedure ExecuteTest(TestName: string);
  var
    Test: TJvxTest;
  begin
    if FTestResults.Aborted then
      Exit;
      
    Test := JvxGetTestClass(TestName).Create(Self);
    try
      FProgressBar.Position := 0;
      FProgressBar.Max := Test.TestCount * 2;
      FTotalProgressBar.StepIt;
      Test.Run(FTestResults);
      FTotalProgressBar.StepIt;
    finally
      Test.Free;
    end;
  end;

var
  I: Integer;
begin
  FTestResults.Clear;
  FTestResults.BreakOnFailure := FBreakOnFailureCheckBox.Checked;

  FTotalProgressBar.Position := 0;

  FTestRunning := True;
  FStopButton.Caption := SStopButtonCaption;
  Cursor := crHourGlass;
  try
    UpdateForm;
    UpdateResults;
    FResultsListView.SetFocus;

    if FTestNameComboBox.ItemIndex = 0 then
    begin
      FTotalProgressBar.Max := (FTestNameComboBox.Items.Count - 1) * 2;
      for I := 1 to FTestNameComboBox.Items.Count - 1 do
        ExecuteTest(FTestNameComboBox.Items[I]);
    end
    else
    begin
      FTotalProgressBar.Max := 2;
      ExecuteTest(FTestNameComboBox.Text);
    end;

  finally
    Cursor := crDefault;
    FStopButton.Caption := SCloseButtonCaption;
    FTestRunning := False;
    UpdateForm;
    UpdateResults;
  end;
end;

procedure TJvxTestRunnerForm.StopTest;
begin
  FTestResults.Abort;
  FTestRunning := False;
  UpdateForm;
  UpdateResults;
end;

procedure TJvxTestRunnerForm.TestError(Error: TJvxTestFailure);
begin
  WriteDebugOutput(Error);
  if FStopOnErrorCheckBox.Checked then
    StopTest;
  UpdateForm;
  UpdateResults;
  Application.ProcessMessages;
end;

procedure TJvxTestRunnerForm.TestFailure(Failure: TJvxTestFailure);
begin
  WriteDebugOutput(Failure);
  if FStopOnFailureCheckBox.Checked then
    StopTest;
  UpdateForm;
  UpdateResults;
  Application.ProcessMessages;
end;

procedure TJvxTestRunnerForm.TestStarted(Test: TJvxTest);
var
  TestDescription: string;
begin
  TestDescription := Test.ClassName;
  if Test.Name <> EmptyStr then
    TestDescription := TestDescription + ' - ' + Test.Name;
  Application.Hint := TestDescription;
  FProgressBar.StepIt;
  UpdateForm;
  UpdateResults;
  Application.ProcessMessages;
end;

procedure TJvxTestRunnerForm.TestStopped(Test: TJvxTest);
begin
  Application.Hint := EmptyStr;
  FProgressBar.StepIt;
  UpdateForm;
  UpdateResults;
  Application.ProcessMessages;
end;

procedure TJvxTestRunnerForm.UpdateForm;

  procedure UpdateConfigControls;
  var
    ConfigControlsEnabled: Boolean;
  begin
    ConfigControlsEnabled := (not FTestRunning) and
      (FTestNameComboBox.Items.Count > 1);

    FTestNameLabel.Enabled          := ConfigControlsEnabled;
    FTestNameComboBox.Enabled       := ConfigControlsEnabled;
    FStopOnErrorCheckBox.Enabled    := ConfigControlsEnabled;
    FBreakOnFailureCheckBox.Enabled := ConfigControlsEnabled;
    FStopOnFailureCheckBox.Enabled  := ConfigControlsEnabled;
    FStartButton.Enabled            := ConfigControlsEnabled;
  end;

  procedure UpdateResultControls;
  var
    ResultControlsEnabled: Boolean;
  begin
    ResultControlsEnabled := (FTestResults.ResultCount <> 0) or (FTestRunning);

    FProgressLabel.Enabled      := ResultControlsEnabled;
    FProgressBar.Enabled        := ResultControlsEnabled;
    FTotalProgressLabel.Enabled := ResultControlsEnabled;
    FTotalProgressBar.Enabled   := ResultControlsEnabled;
    FResultsLabel.Enabled       := ResultControlsEnabled;
    FResultsListView.Enabled    := ResultControlsEnabled;
  end;

  procedure UpdateStatusBar;
  begin
    FStatusBar.Panels[0].Text := Format(SErrorsStatusFormat, [FTestResults.ErrorCount]);
    FStatusBar.Panels[1].Text := Format(SFailuresStatusFormat, [FTestResults.FailureCount]);
  end;

begin
  UpdateConfigControls;
  UpdateResultControls;
  UpdateStatusBar;
end;

procedure TJvxTestRunnerForm.UpdateResults;
begin
  with FResultsListView do
  begin
    Items.Count := FTestResults.ResultCount;
    if (Selected <> nil) and (Selected.Index = (Items.Count - 2)) then
    begin
      Selected := Items[Items.Count - 1];
      ItemFocused := Selected;
      Selected.MakeVisible(False);
    end;
    if (ItemFocused = nil) and (Items.Count > 0) then
      ItemFocused := Items[0];
    Invalidate;
  end;
end;

procedure TJvxTestRunnerForm.FStatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  if Panel.Index = 0 then
    DrawStatistics(StatusBar.Canvas, Panel.Text, FTestResults.ErrorCount, Rect)
  else
    DrawStatistics(StatusBar.Canvas, Panel.Text, FTestResults.FailureCount, Rect);
end;

procedure TJvxTestRunnerForm.DrawStatistics(Canvas: TCanvas; Text: string;
  FailureCount: Integer; const Rect: TRect);
begin
  with Canvas do
  begin
    if FailureCount > 0 then
    begin
      Font.Color := clRed;
      Font.Style := Font.Style + [fsBold];
    end;
    TextRect(Rect, Rect.left + 1, Rect.top + 1, Text);
  end;
end;

procedure TJvxTestRunnerForm.WriteDebugOutput(Result: TJvxTestResult);
var
  OutputMessage: string;
begin
  OutputMessage := Result.TestName + ': ';
  if Result is TJvxTestFailure then
    OutputMessage := OutputMessage + 'Exception ' +
      TJvxTestFailure(Result).ErrorClass.ClassName + ' - ';
  OutputMessage := OutputMessage + Result.Message;
  OutputDebugString(PChar(OutputMessage));
end;

end.

