
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxClassesTest.pas.                                    }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxClassesTest;

{$include Jvx.inc}

interface

uses
  Classes, TypInfo, JvxClasses, JvxTestClasses, JvxStdNodes, JvxUtils;

type

  { TTestNotifyEvents }

  TTestNotifyEvents = class(TJvxNodeNotifyEvents);

  { TTestNodes }

  TTestNodes = class(TJvxNodes)
  private
    FDataSource: TJvxDataSource;
    function GetDataSource: TJvxDataSource;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TTestNodeList }

  TTestNodeList = class(TJvxNodes)
  private
    FListDataSource: TJvxListDataSource;
    function GetDataSource: TJvxDataSource;
  public
    constructor Create(Owner: TComponent); override;
  end;

  { TTestNode }

  TTestNode = class(TJvxNode)
  private
    FAssigned: Boolean;
    FChangedNode: TJvxNode;
    FInsertedNode: TJvxNode;
    FLoadedNodeField: TJvxDataField;
    FNotificationOperation: TJvxNodeOperation;
    FRemovedNode: TJvxNode;
  protected
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    function GetAssigned: Boolean; override;
    procedure InternalLoadData(DataField: TJvxDataField); override;
  public
    procedure Reset;
    property ChangedNode: TJvxNode read FChangedNode write FChangedNode;
    property InsertedNode: TJvxNode read FInsertedNode write FInsertedNode;
    property NotificationOperation: TJvxNodeOperation read FNotificationOperation write FNotificationOperation;
    property RemovedNode: TJvxNode read FRemovedNode write FRemovedNode;
  published
    property NodeName write SetNodeName;
  end;

  { TTestVisitor }

  TTestVisitor = class(TJvxVisitor)
  private
    FVisitCount: Integer;
    FVisitedAttribute: TJvxAttribute;
    FVisitedElement: TJvxElement;
  public
    procedure Reset;
    procedure Visit(Attribute: TJvxAttribute); override;
    procedure Visit(Element: TJvxElement); override;
    property VisitCount: Integer read FVisitCount;
    property VisitedAttribute: TJvxAttribute read FVisitedAttribute;
    property VisitedElement: TJvxElement read FVisitedElement;
  end;

  { TTestAttribute }

  TTestAttribute = class(TJvxAttribute)
  private
    FAssigned: Boolean;
    FChangedNode: TJvxNode;
    FInsertedNode: TJvxNode;
    FNotificationOperation: TJvxNodeOperation;
    FRemovedNode: TJvxNode;
    FValue: Variant;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure InternalSetAsVariant(const Value: Variant); override;
    procedure ReadValue(Reader: TReader); override;
    procedure WriteValue(Writer: TWriter); override;
  public
    constructor Create(Onwer: TComponent); override;
    procedure Reset;

    property ChangedNode: TJvxNode read FChangedNode;
    property InsertedNode: TJvxNode read FInsertedNode;
    property NotificationOperation: TJvxNodeOperation read FNotificationOperation;
    property RemovedNode: TJvxNode read FRemovedNode;
  published
    property Assigned;
    property NodeName write SetNodeName;
    property Value: Variant read GetAsVariant write SetAsVariant stored False;
  end;

  { TTestElement }

  TTestElement = class(TJvxElement)
  private
    FAssigned: Boolean;
    FValue: Variant;
    FAttribute: TTestAttribute;
    FChangedNode: TJvxNode;
    FInsertedNode: TJvxNode;
    FNotificationOperation: TJvxNodeOperation;
    FRemovedNode: TJvxNode;
  protected
    procedure AssignValue(Source: TJvxNode); override;
    procedure DefineAttributes(DefineAttribute: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    function CompareValue(Source: TJvxNode): Boolean; override;
    function GetAssigned: Boolean; override;
    function InternalGetAsBoolean: Boolean; override;
    function InternalGetAsDateTime: TDateTime; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsInteger: Integer; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
    procedure InternalSetAsBoolean(Value: Boolean); override;
    procedure InternalSetAsDateTime(Value: TDateTime); override;
    procedure InternalSetAsFloat(Value: Extended); override;
    procedure InternalSetAsInteger(Value: Integer); override;
    procedure InternalSetAssigned(Value: Boolean); override;
    procedure InternalSetAsString(const Value: string); override;
    procedure InternalSetAsVariant(const Value: Variant); override;
    procedure ReadValue(Reader: TReader); override;
    procedure ValidateAttribute(Attribute: TJvxNode); override;
    procedure WriteValue(Writer: TWriter); override;
  public
    constructor Create(Owner: TComponent); override;
    procedure Reset;

    property ChangedNode: TJvxNode read FChangedNode;
    property InsertedNode: TJvxNode read FInsertedNode;
    property NotificationOperation: TJvxNodeOperation read FNotificationOperation;
    property RemovedNode: TJvxNode read FRemovedNode;
  published
    property Attribute: TTestAttribute read FAttribute;
    property Assigned;
    property NodeName write SetNodeName;
    property Value: string read GetAsString write SetAsString stored False;
  end;

  { TTestComposite }

  TTestComposite = class(TJvxCompositeTextElement)
  private
    FChangedNode: TJvxNode;
    FElement: TTestElement;
    FInsertedNode: TJvxNode;
    FNotificationOperation: TJvxNodeOperation;
    FRemovedNode: TJvxNode;
  protected
    procedure DefineElements(DefineElement: TJvxDefineNodeProc;
      Data: Pointer); override;
    procedure DocumentNotification(Node: TJvxNode;
      Operation: TJvxNodeOperation); override;
    procedure ValidateElement(Element: TJvxNode); override;
  public
    constructor Create(Owner: TComponent); override;
    property ChangedNode: TJvxNode read FChangedNode;
    property Element: TTestElement read FElement;
    property InsertedNode: TJvxNode read FInsertedNode;
    property NotificationOperation: TJvxNodeOperation read FNotificationOperation;
    property RemovedNode: TJvxNode read FRemovedNode;
  published
    property Assigned;
    property NodeName write SetNodeName;
    property Template write SetTemplate;
  end;

  { TTestNodeField }

  TTestNodeField = class(TJvxDataField)
  private
    FValue: Variant;
    procedure SetValue(Value: Variant);
  protected
    function GetTypeInfo: PTypeInfo; override;
    function InternalGetAsFloat: Extended; override;
    function InternalGetAsOrdinal: TJvxOrdinal; override;
    function InternalGetAssigned: Boolean; override;
    function InternalGetAsString: string; override;
    function InternalGetAsVariant: Variant; override;
  public
    property Value: Variant read FValue write SetValue;
  end;

  { TTestNodeFields }

  TTestNodeFields = class(TJvxDataFields);

  { TTestDataSource }

  TTestDataSource = class(TJvxDataSource)
  private
    FField1: TTestNodeField;
    FField2: TTestNodeField;
    FField3: TTestNodeField;
    FValidatedField: TJvxDataField;
  protected
    procedure CreateDefaultFields; override;
    procedure InternalOpen; override;
    procedure Notification(Component: TComponent; Operation: TOperation); override;
    procedure ValidateField(Field: TJvxDataField); override;
  end;

  { TDocumentNotifyEventsTest }

  TDocumentNotifyEventsTest = class(TJvxTestSuite)
  private
    FEvent: TJvxNodeNotifyEvent;
    FEvents: TTestNotifyEvents;
    FFirstEventInvoked: Boolean;
    FFirstEventOperation: TJvxNodeOperation;
    FSecondEventInvoked: Boolean;
    FSecondEventOperation: TJvxNodeOperation;
    procedure FirstEvent(Sender: TJvxNode; Operation: TJvxNodeOperation);
    procedure SecondEvent(Sender: TJvxNode; Operation: TJvxNodeOperation);
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAdd;
    procedure TestEvent;
    procedure TestInvoke;
    procedure TestRemove;
  end;

  { TDocumentNodeTest }

  TDocumentNodeTest = class(TJvxTestSuite)
  private
    FAttribute: TTestAttribute;
    FNodeField: TTestNodeField;
    FElement: TTestElement;
    FNode: TTestNode;
    FNode2: TTestNode;
    FNotificationNode: TJvxNode;
    FNotificationOperation: TJvxNodeOperation;
    procedure OnDocumentNotification(Sender: TJvxNode;
      Operation: TJvxNodeOperation);
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestAccessError;
    procedure TestAsBoolean;
    procedure TestAsDateTime;
    procedure TestAsFloat;
    procedure TestAsInteger;
    procedure TestAssign;
    procedure TestAssigned;
    procedure TestAsString;
    procedure TestAsVariant;
    procedure TestAttributes;
    procedure TestCompare;
    procedure TestNodeField;
    procedure TestDocumentNotification;
    procedure TestElements;
    procedure TestIsChildOf;
    procedure TestIsParentOf;
    procedure TestNodeName;
    procedure TestNotify;
    procedure TestOnNotify;
    procedure TestOnNotifyEvents;
    procedure TestParent;
    procedure TestRoot;
    procedure TestValidateAttribute;
    procedure TestValidateElement;
    procedure TestValidateParent;
    procedure TestValueErrorInteger;
    procedure TestValueErrorString;
  end;

  { TDocumentNodesTest }

  TDocumentNodesTest = class(TJvxTestSuite)
  private
    FNode: TTestNode;
    FNodes: TTestNodes;
    FParent: TTestNode;
    FValidatedNode: TJvxNode;
    procedure OnValidateNode(Node: TJvxNode);
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestAdd;
    procedure TestAssign;
    procedure TestClear;
    procedure TestCount;
    procedure TestDelete;
    procedure TestCompare;
    procedure TestIndexOf;
    procedure TestInsert;
    procedure TestItems;
    procedure TestMove;
    procedure TestRemove;
    procedure TestValidateNode;
  end;

  { TDocumentNodeListTest }

  TDocumentNodeListTest = class(TJvxTestSuite)
  private
    FNode: TTestNode;
    FNode2: TTestNode;
    FNode3: TTestNode;
    FNodeList: TTestNodeList;
    FNotifier: TJvxNotifyComponent;
    FParent: TTestNode;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestActive;
    procedure TestAdd;
    procedure TestAssign;
    procedure TestClear;
    procedure TestCount;
    procedure TestDelete;
    procedure TestCompare;
    procedure TestFindNode;
    procedure TestIndexOf;
    procedure TestInsert;
    procedure TestItems;
    procedure TestMove;
    procedure TestRemove;
  end;

  { TDocumentAttributeTest }

  TDocumentAttributeTest = class(TJvxTestSuite)
  private
    FElement: TTestElement;
    FElement2: TTestElement;
    FAttribute: TTestAttribute;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestAsBoolean;
    procedure TestAsDateTime;
    procedure TestAsFloat;
    procedure TestAsInteger;
    procedure TestAsString;
    procedure TestAsVariant;
    procedure TestSetParent;
    procedure TestDestroy;
  end;

  { TDocumentElementTest }

  TDocumentElementTest = class(TJvxTestSuite)
  private
    FAttribute: TTestAttribute;
    FComposite: TTestComposite;
    FComposite2: TTestComposite;
    FElement: TTestElement;
    FNotifier: TJvxNotifyComponent;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestAssign;
    procedure TestAttributes;
    procedure TestCreateDefinedAttributes;
    procedure TestDestroy;
    procedure TestDestroyAttributes;
    procedure TestDocumentNotification;
    procedure TestCompare;
    procedure TestParent;
  end;

  { TCompositeDocumentElementTest }

  TCompositeDocumentElementTest = class(TJvxTestSuite)
  private
    FAttribute: TTestAttribute;
    FComposite: TTestComposite;
    FElement: TTestElement;
    FNotifier: TJvxNotifyComponent;
    FParentComposite: TTestComposite;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    procedure TestAccept;
    procedure TestAssign;
    procedure TestCreateDefinedElements;
    procedure TestDestroy;
    procedure TestDocumentNotification;
    procedure TestElements;
    procedure TestCompare;
    procedure TestHasAttributes;
    procedure TestHasElements;
    procedure TestNotification;
    procedure TestStreaming;
    procedure TestTemplate;
  end;

  { TNodeFieldTest }

  TNodeFieldTest = class(TJvxTestSuite)
  private
    FAttribute: TTestAttribute;
    FNodeField: TJvxDataField;
    FDataSource: TTestDataSource;
    FTestNodeField: TTestNodeField;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAccessError;
    procedure TestAssigned;
    procedure TestChanged;
    procedure TestGetAs;
    procedure TestSetDataSource;
  end;

  { TNodeFieldsTest }

  TNodeFieldsTest = class(TJvxTestSuite)
  private
    FNodeField: TTestNodeField;
    FDataSource: TTestDataSource;
    FNotifier: TJvxNotifyComponent;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
    procedure Notification(Component: TComponent; Operation: TOperation); override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAdd;
    procedure TestClear;
    procedure TestCount;
    procedure TestDelete;
    procedure TestDestroy;
    procedure TestFieldByName;
    procedure TestFieldDestroy;
    procedure TestFindField;
    procedure TestGetItem;
    procedure TestIndexOf;
    procedure TestInsert;
    procedure TestRemove;
    procedure TestValidation;
  end;

  { TDataSourceTest }

  TDataSourceTest = class(TJvxTestSuite)
  private
    FNode: TTestNode;
    FNodeField: TTestNodeField;
    FDataSource: TTestDataSource;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestActive;
    procedure TestCount;
    procedure TestDefaultFields;
    procedure TestDelete;
    procedure TestGetNode;
    procedure TestIndexOf;
    procedure TestInsert;
  end;

  { TTypedNodeTest }

  TTypedNodeTest = class(TJvxTestSuite)
  private
    FLoadedNode: TObject;
    FNodeField: TTestNodeField;
    FNode: TJvxNode;
    FNodeClass: TJvxNodeClass;
    FNotificationOperation: TJvxNodeOperation;
    FNotificationSender: TJvxNode;
    procedure DataLoaded(Sender: TJvxNode; DataField: TJvxDataField; var Done: Boolean);
    procedure DocumentNotification(Sender: TJvxNode; Operation: TJvxNodeOperation);
    procedure LoadData(Sender: TJvxNode; DataField: TJvxDataField; var Done: Boolean);
  protected
    procedure AssignNodeValue; virtual;
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
    property DataField: TTestNodeField read FNodeField write FNodeField;
    property Node: TJvxNode read FNode write FNode;
  public
    constructor Create(Owner: TComponent); override;
    property NodeClass: TJvxNodeClass read FNodeClass write FNodeClass;
  published
    procedure TestAsBoolean; virtual;
    procedure TestAsDateTime; virtual;
    procedure TestAsFloat; virtual;
    procedure TestAsInteger; virtual;
    procedure TestAssign;
    procedure TestAssigned; virtual;
    procedure TestAsString; virtual;
    procedure TestAsVariant; virtual;
    procedure TestLoadData; virtual;
  end;

  { TBooleanNodeTest }

  TBooleanNodeTest = class(TTypedNodeTest)
  protected
    procedure AssignNodeValue; override;
  published
    procedure TestAsBoolean; override;
    procedure TestAsInteger; override;
    procedure TestAssigned; override;
    procedure TestAsString; override;
    procedure TestAsVariant; override;
    procedure TestLoadData; override;
  end;

  { TDateTimeNodeTest }

  TDateTimeNodeTest = class(TTypedNodeTest)
  protected
    procedure AssignNodeValue; override;
  published
    procedure TestAsDateTime; override;
    procedure TestAssigned; override;
    procedure TestAsString; override;
    procedure TestAsVariant; override;
    procedure TestLoadData; override;
  end;

  { TFloatNodeTest }

  TFloatNodeTest = class(TTypedNodeTest)
  protected
    procedure AssignNodeValue; override;
  published
    procedure TestAsFloat; override;
    procedure TestAsInteger; override;
    procedure TestAssigned; override;
    procedure TestAsString; override;
    procedure TestAsVariant; override;
    procedure TestLoadData; override;
  end;

  { TIntegerNodeTest }

  TIntegerNodeTest = class(TTypedNodeTest)
  protected
    procedure AssignNodeValue; override;
  published
    procedure TestAsFloat; override;
    procedure TestAsInteger; override;
    procedure TestAssigned; override;
    procedure TestAsString; override;
    procedure TestAsVariant; override;
    procedure TestLoadData; override;
  end;

  { TStringNodeTest }

  TStringNodeTest = class(TTypedNodeTest)
  protected
    procedure AssignNodeValue; override;
  published
    procedure TestAsBoolean; override;
    procedure TestAsDateTime; override;
    procedure TestAsFloat; override;
    procedure TestAsInteger; override;
    procedure TestAssigned; override;
    procedure TestAsString; override;
    procedure TestAsVariant; override;
    procedure TestLoadData; override;
  end;

  { TGenericNodesTestSuite }

  TGenericNodesTestSuite = class(TJvxTestSuite)
  public
    constructor Create(Owner: TComponent); override;
  end;

implementation

uses
  SysUtils, Math, JvxDebugMem;

{ TTestNode }

procedure TTestNode.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FNotificationOperation := Operation;
  case Operation of
    noInsert: FInsertedNode := Node;
    noChange: FChangedNode := Node;
    noRemove: FRemovedNode := Node;
  else
    Assert(False);
  end;
end;

function TTestNode.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

procedure TTestNode.InternalLoadData(DataField: TJvxDataField);
begin
  FLoadedNodeField := DataField;
end;

procedure TTestNode.Reset;
begin
  FNotificationOperation := noInsert;
  FChangedNode := nil;
  FInsertedNode := nil;
  FLoadedNodeField := nil;
  FRemovedNode := nil;
end;

{ TTestVisitor }

procedure TTestVisitor.Visit(Attribute: TJvxAttribute);
begin
  inherited;
  Inc(FVisitCount);
  FVisitedAttribute := Attribute;
end;

procedure TTestVisitor.Reset;
begin
  FVisitCount := 0;
  FVisitedAttribute := nil;
  FVisitedElement := nil;
end;

procedure TTestVisitor.Visit(Element: TJvxElement);
begin
  inherited;
  Inc(FVisitCount);
  FVisitedElement := Element;
end;

{ TTestAttribute }

procedure TTestAttribute.AssignValue(Source: TJvxNode);
begin
  AsVariant := Source.AsVariant;
end;

constructor TTestAttribute.Create(Onwer: TComponent);
begin
  inherited;
  SetNodeName('TestAttribute');
end;

procedure TTestAttribute.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  inherited;
  FNotificationOperation := Operation;
  case Operation of
    noInsert: FInsertedNode := Node;
    noChange: FChangedNode := Node;
    noRemove: FRemovedNode := Node;
  else
    Assert(False);
  end;
end;

function TTestAttribute.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsVariant = Source.AsVariant;
end;

function TTestAttribute.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TTestAttribute.InternalGetAsBoolean: Boolean;
begin
  Result := FValue;
end;

function TTestAttribute.InternalGetAsDateTime: TDateTime;
begin
  Result := FValue;
end;

function TTestAttribute.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TTestAttribute.InternalGetAsInteger: Integer;
begin
  Result := FValue;
end;

function TTestAttribute.InternalGetAsString: string;
begin
  Result := FValue;
end;

function TTestAttribute.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TTestAttribute.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := Value;
end;

procedure TTestAttribute.InternalSetAsDateTime(Value: TDateTime);
begin
  FValue := Value;
end;

procedure TTestAttribute.InternalSetAsFloat(Value: Extended);
begin
  FValue := Value;
end;

procedure TTestAttribute.InternalSetAsInteger(Value: Integer);
begin
  FValue := Value;
end;

procedure TTestAttribute.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TTestAttribute.InternalSetAsString(const Value: string);
begin
  FValue := Value;
end;

procedure TTestAttribute.InternalSetAsVariant(const Value: Variant);
begin
  FValue := Value;
end;

procedure TTestAttribute.ReadValue(Reader: TReader);
begin
  FValue := Reader.ReadString;
end;

procedure TTestAttribute.Reset;
begin
  FNotificationOperation := noInsert;
  FChangedNode := nil;
  FInsertedNode := nil;
  FRemovedNode := nil;
end;

procedure TTestAttribute.WriteValue(Writer: TWriter);
begin
  Assert(VarType(FValue) = varString);
  Writer.WriteString(FValue);
end;

{ TTestElement }

procedure TTestElement.AssignValue(Source: TJvxNode);
begin
  AsVariant := Source.AsVariant;
end;

constructor TTestElement.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('TestElement');
  FAttribute.Value := 'Default Attribute Value';
end;

procedure TTestElement.DefineAttributes(
  DefineAttribute: TJvxDefineNodeProc; Data: Pointer);
begin
  inherited;
  DefineAttribute(FAttribute, 'FAttribute', TTestAttribute, Data);
end;

procedure TTestElement.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  inherited;
  FNotificationOperation := Operation;
  case Operation of
    noInsert: FInsertedNode := Node;
    noChange: FChangedNode := Node;
    noRemove: FRemovedNode := Node;
  else
    Assert(False);
  end;
end;

function TTestElement.CompareValue(Source: TJvxNode): Boolean;
begin
  Result := AsVariant = Source.AsVariant;
end;

function TTestElement.GetAssigned: Boolean;
begin
  Result := FAssigned;
end;

function TTestElement.InternalGetAsBoolean: Boolean;
begin
  Result := FValue;
end;

function TTestElement.InternalGetAsDateTime: TDateTime;
begin
  Result := FValue;
end;

function TTestElement.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TTestElement.InternalGetAsInteger: Integer;
begin
  Result := FValue;
end;

function TTestElement.InternalGetAsString: string;
begin
  Result := FValue;
end;

function TTestElement.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TTestElement.InternalSetAsBoolean(Value: Boolean);
begin
  FValue := Value;
end;

procedure TTestElement.InternalSetAsDateTime(Value: TDateTime);
begin
  FValue := Value;
end;

procedure TTestElement.InternalSetAsFloat(Value: Extended);
begin
  FValue := Value;
end;

procedure TTestElement.InternalSetAsInteger(Value: Integer);
begin
  FValue := Value;
end;

procedure TTestElement.InternalSetAssigned(Value: Boolean);
begin
  FAssigned := Value;
end;

procedure TTestElement.InternalSetAsString(const Value: string);
begin
  FValue := Value;
end;

procedure TTestElement.InternalSetAsVariant(const Value: Variant);
begin
  FValue := Value;
end;

procedure TTestElement.ReadValue(Reader: TReader);
begin
  FValue := Reader.ReadString;
end;

procedure TTestElement.Reset;
begin
  FNotificationOperation := noInsert;
  FChangedNode := nil;
  FInsertedNode := nil;
  FRemovedNode := nil;
end;

procedure TTestElement.ValidateAttribute(Attribute: TJvxNode);
begin
  if Attribute is TTestAttribute then Exit;
  inherited;
end;

procedure TTestElement.WriteValue(Writer: TWriter);
begin
  Assert(VarType(FValue) = varString);
  Writer.WriteString(FValue);
end;

{ TTestComposite }

constructor TTestComposite.Create(Owner: TComponent);
begin
  inherited;
  SetNodeName('TestComposite');
  FElement.Value := 'Default Element Value';
end;

procedure TTestComposite.DefineElements(DefineElement: TJvxDefineNodeProc;
  Data: Pointer);
begin
  inherited;
  DefineElement(FElement, 'FElement', TTestElement, Data);
end;

procedure TTestComposite.DocumentNotification(Node: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  inherited;
  FNotificationOperation := Operation;
  case Operation of
    noInsert: FInsertedNode := Node;
    noChange: FChangedNode := Node;
    noRemove: FRemovedNode := Node;
  else
    Assert(False);
  end;
end;

procedure TTestComposite.ValidateElement(Element: TJvxNode);
begin
  if Element is TTestElement then Exit;
  if Element is TTestComposite then Exit;
  inherited ValidateElement(Element);
end;

{ TTestNodeField }

function TTestNodeField.GetTypeInfo: PTypeInfo;
begin
  Result := System.TypeInfo(Variant);
end;

function TTestNodeField.InternalGetAsFloat: Extended;
begin
  Result := FValue;
end;

function TTestNodeField.InternalGetAsOrdinal: TJvxOrdinal;
begin
  if VarType(FValue) = varBoolean then
    Result := Ord(FValue = True)
  else
    Result := FValue;
end;

function TTestNodeField.InternalGetAssigned: Boolean;
begin
  Result := (not VarIsEmpty(FValue)) and (not VarIsNull(FValue));
end;

function TTestNodeField.InternalGetAsString: string;
begin
  Result := FValue;
end;

function TTestNodeField.InternalGetAsVariant: Variant;
begin
  Result := FValue;
end;

procedure TTestNodeField.SetValue(Value: Variant);
begin
  FValue := Value;
  Changed;
end;

{ TDocumentNotifyEventsTest }

constructor TDocumentNotifyEventsTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAdd);
  AddTest(TestEvent);
  AddTest(TestInvoke);
  AddTest(TestRemove);
end;

procedure TDocumentNotifyEventsTest.InternalCreateFixture;
begin
  inherited;
  FEvents := TTestNotifyEvents.Create(@@FEvent);
end;

procedure TDocumentNotifyEventsTest.InternalDestroyFixture;
begin
  FreeAndNil(FEvents);
  FFirstEventInvoked := False;
  FFirstEventOperation := noInsert;
  FSecondEventInvoked := False;
  FSecondEventOperation := noInsert;
  inherited;
end;

procedure TDocumentNotifyEventsTest.FirstEvent(Sender: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FFirstEventInvoked := True;
  FFirstEventOperation := Operation;
end;

procedure TDocumentNotifyEventsTest.SecondEvent(Sender: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FSecondEventInvoked := True;
  FSecondEventOperation := Operation;
end;

procedure TDocumentNotifyEventsTest.TestAdd;
begin
  Assert(not FFirstEventInvoked);
  FEvents.Add(FirstEvent);
  FEvents.Invoke(nil, noInsert);
  Assert(FFirstEventInvoked);
end;

procedure TDocumentNotifyEventsTest.TestEvent;
begin
  FEvent := FirstEvent;
  Assert(not FFirstEventInvoked);
  FEvents.Invoke(nil, noChange);
  Assert(FFirstEventInvoked);
  Assert(FFirstEventOperation = noChange);
end;

procedure TDocumentNotifyEventsTest.TestInvoke;
begin
  FEvents.Add(FirstEvent);
  FEvent := SecondEvent;
  Assert(not FFirstEventInvoked);
  Assert(not FSecondEventInvoked);
  FEvents.Invoke(nil, noChange);
  Assert(FFirstEventInvoked);
  Assert(FFirstEventOperation = noChange);
  Assert(FSecondEventInvoked);
  Assert(FSecondEventOperation = noChange);
end;

procedure TDocumentNotifyEventsTest.TestRemove;
begin
  FEvents.Add(FirstEvent);
  FEvents.Invoke(nil, noInsert);
  Assert(FFirstEventInvoked);

  FFirstEventInvoked := False;
  FEvents.Remove(FirstEvent);
  FEvents.Invoke(nil, noInsert);
  Assert(not FFirstEventInvoked);
end;

{ TDocumentNodeTest }

constructor TDocumentNodeTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAccept);
  AddTest(TestAccessError);
  AddTest(TestAsBoolean);
  AddTest(TestAsDateTime);
  AddTest(TestAsFloat);
  AddTest(TestAsInteger);
  AddTest(TestAssign);
  AddTest(TestAssigned);
  AddTest(TestAsString);
  AddTest(TestAsVariant);
  AddTest(TestAttributes);
  AddTest(TestNodeField);
  AddTest(TestDocumentNotification);
  AddTest(TestElements);
  AddTest(TestCompare);
  AddTest(TestIsChildOf);
  AddTest(TestIsParentOf);
  AddTest(TestNodeName);
  AddTest(TestNotify);
  AddTest(TestOnNotify);
  AddTest(TestOnNotifyEvents);
  AddTest(TestParent);
  AddTest(TestRoot);
  AddTest(TestValidateAttribute);
  AddTest(TestValidateElement);
  AddTest(TestValidateParent);
  AddTest(TestValueErrorInteger);
  AddTest(TestValueErrorString);
end;

procedure TDocumentNodeTest.InternalCreateFixture;
begin
  inherited;
  FAttribute := TTestAttribute.Create(nil);
  FAttribute.NodeName := 'FAttribute';
  FAttribute.Reset;

  FElement := TTestElement.Create(nil);
  FElement.NodeName := 'FElement';
  FElement.Reset;

{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FNode := TTestNode.Create(nil);
  FNode.NodeName := 'FNode';
  FNode.Reset;

  FNode2 := TTestNode.Create(nil);
  FNode2.NodeName := 'FNode2';
  FNode2.Reset;
{$WARNINGS ON}

  FNodeField := TTestNodeField.Create(nil);
end;

procedure TDocumentNodeTest.InternalDestroyFixture;
begin
  FreeAndNil(FAttribute);
  FreeAndNil(FElement);
  FreeAndNil(FNode);
  FreeAndNil(FNode2);
  FreeAndNil(FNodeField);
  FNotificationNode := nil;
  FNotificationOperation := Low(TJvxNodeOperation);
  inherited;
end;

procedure TDocumentNodeTest.OnDocumentNotification(Sender: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FNotificationNode := Sender;
  FNotificationOperation := Operation;
end;

procedure TDocumentNodeTest.TestAccept;
var
  Visitor: TTestVisitor;
begin
  Visitor := TTestVisitor.Create;
  try
    try
      FNode.Accept(Visitor);
      Assert(EAbstractError);
    except
      Assert(EAbstractError);
    end;
  finally
    Visitor.Free;
  end;
end;

procedure TDocumentNodeTest.TestAccessError;
var
  E: EInvalidOperation;
begin
  E := JvxAccessError(TypeInfo(Integer));
  try
    Assert(E <> nil);
    Assert(E is EInvalidOperation);
  finally
    E.Free;
  end;
end;

procedure TDocumentNodeTest.TestAsBoolean;
begin
  // Test that unassigned node cannot be accessed as boolean
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsBoolean;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as boolean
  FNode.FAssigned := True;
  try
    FNode.GetAsBoolean;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsBoolean(True);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsBoolean(True);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAsDateTime;
begin
  // Test that unassigned node cannot be accessed as DateTime
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsDateTime;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as DateTime
  FNode.FAssigned := True;
  try
    FNode.GetAsDateTime;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsDateTime(Now);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsDateTime(Now);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAsFloat;
begin
  // Test that unassigned node cannot be accessed as Float
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as Float
  FNode.FAssigned := True;
  try
    FNode.GetAsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsFloat(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsFloat(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAsInteger;
begin
  // Test that unassigned node cannot be accessed as Integer
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsInteger;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as Integer
  FNode.FAssigned := True;
  try
    FNode.GetAsInteger;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsInteger(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsInteger(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAssign;
begin
  // Test that assigning nil causes default AssignTo to raise an exception
  try
    FNode.Assign(nil);
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;

  // Test assigning node with name and event handlers
  FNode.NodeName := 'Test';
  FNode.OnNotifyEvents.Add(OnDocumentNotification);
  FNode.OnNotify := OnDocumentNotification;
  Assert(not FNode.Compare(FNode2));
  FNode2.Assign(FNode);
  Assert(FNode.Compare(FNode2));
end;

procedure TDocumentNodeTest.TestAssigned;
begin
  // Test that by default all nodes are not assigned
  Assert(FNode.Assigned = False);

  // Test that by default assigning a node causes an exception
  try
    FNode.Assigned := True;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigning a node sends a document change notification
  FAttribute.AsString := 'test';
  FAttribute.Reset;
  Assert(FAttribute.ChangedNode = nil);
  FAttribute.Assigned := False;
  Assert(FAttribute.ChangedNode = FAttribute);
end;

procedure TDocumentNodeTest.TestAsString;
begin
  // Test that unassigned node cannot be accessed as String
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as String
  FNode.FAssigned := True;
  try
    FNode.GetAsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsString('Test');
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsString('Test');
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAsVariant;
begin
  // Test that unassigned node cannot be accessed as Variant
  Assert(FNode.Assigned = False);
  try
    FNode.GetAsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigned node raises exception by default when accessed as Variant
  FNode.FAssigned := True;
  try
    FNode.GetAsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.SetAsVariant('Test');
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    FNode.InternalSetAsVariant('Test');
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestAttributes;
begin
  // Test GetAttributes/SetAttributes
  Assert(FNode.Attributes <> nil);
  Assert(FNode.Attributes is TJvxNodes);

  // Test GetAttributeSet
  Assert(FNode.Attributes.Count = 0);
end;

procedure TDocumentNodeTest.TestCompare;
begin
  // Test that node is never equal to nil
  Assert(not FNode.Compare(nil));

  // Test that node name should be the same for nodes to be equal
  FNode.NodeName := 'test';
  Assert(not FNode.Compare(FNode2));
  FNode2.NodeName := 'test';
  Assert(FNode.Compare(FNode2));

  // Test that component name should be the same for nodes to be equal
  FNode.Name := 'FNode';
  Assert(not FNode.Compare(FNode2));
  FNode2.Name := 'FNode';
  Assert(FNode.Compare(FNode2));

  // Test that OnNotifyEvents are not compared
  FNode.OnNotifyEvents.Add(OnDocumentNotification);
  Assert(FNode.Compare(FNode2));
  FNode2.OnNotifyEvents.Add(OnDocumentNotification);
  Assert(FNode.Compare(FNode2));

  // Test that OnNotify events are not compared
  FNode.OnNotify := OnDocumentNotification;
  Assert(FNode.Compare(FNode2));
  FNode2.OnNotify := OnDocumentNotification;
  Assert(FNode.Compare(FNode2));
end;

procedure TDocumentNodeTest.TestNodeField;
begin
  Assert(FNode.DataField = nil);
  Assert(FNode.FLoadedNodeField = nil);

  // Test that assigning data source calls NodeFieldChanged
  FNodeField.Value := 'Test';
  FNode.DataField := FNodeField;
  Assert(FNode.FLoadedNodeField = FNodeField);

  // Test that assigning data source assigns NodeFieldChanged as an event handler
  FNode.FLoadedNodeField := nil;
  FNodeField.Value := 'New Test';
  Assert(FNode.FLoadedNodeField = FNodeField);

  // Test that assigning data source creates FreeNotification link
  FNodeField.Free;
  Assert(FNode.DataField = nil);
  FNodeField := nil;
end;

procedure TDocumentNodeTest.TestDocumentNotification;
begin
  FNode.ChangedNode := nil;
  FNode.Notify(noChange);
  Assert(FNode.ChangedNode = FNode);
end;

procedure TDocumentNodeTest.TestElements;
begin
  // Test GetElements/SetElements
  Assert(FNode.Elements <> nil);
  Assert(FNode.Elements is TJvxNodes);

  // Test GetElementSet
  Assert(FNode.Elements.Count = 0);
end;

procedure TDocumentNodeTest.TestIsChildOf;
begin
  FAttribute.Parent := FElement;
  Assert(FAttribute.IsChildOf(FElement));
end;

procedure TDocumentNodeTest.TestIsParentOf;
begin
  FAttribute.Parent := FElement;
  Assert(FElement.IsParentOf(FAttribute));
end;

procedure TDocumentNodeTest.TestNodeName;
begin
  FNode.NodeName := 'Test';
  Assert(FNode.NodeName = 'Test');
  Assert(FNode.ChangedNode = FNode);
end;

procedure TDocumentNodeTest.TestNotify;
begin
  Assert(FNode.ChangedNode = nil);
  Assert(FNotificationNode = nil);
  Assert(FNotificationOperation = noInsert);

  FNode.OnNotify := OnDocumentNotification;
  FNode.Notify(noChange);

  Assert(FNode.ChangedNode = FNode);
  Assert(FNotificationNode = FNode);
  Assert(FNotificationOperation = noChange);
end;

procedure TDocumentNodeTest.TestOnNotify;
begin
  Assert(not Assigned(FNode.OnNotify));
  Assert(FNotificationNode = nil);
  Assert(FNotificationOperation = noInsert);

  FNode.OnNotify := OnDocumentNotification;
  Assert(Assigned(FNode.OnNotify));

  FNode.Notify(noChange);

  Assert(FNotificationNode = FNode);
  Assert(FNotificationOperation = noChange);
end;

procedure TDocumentNodeTest.TestOnNotifyEvents;
begin
  Assert(FNode.OnNotifyEvents <> nil);
  Assert(FNode.OnNotifyEvents is TJvxNodeNotifyEvents);
  Assert(FNotificationNode = nil);
  Assert(FNotificationOperation = noInsert);

  FNode.OnNotifyEvents.Add(OnDocumentNotification);
  FNode.Notify(noChange);

  Assert(FNotificationNode = FNode);
  Assert(FNotificationOperation = noChange);
end;

procedure TDocumentNodeTest.TestParent;
begin
  FNode.Parent := nil;
end;

procedure TDocumentNodeTest.TestRoot;
begin
  Assert(FNode.Root = FNode);
end;

procedure TDocumentNodeTest.TestValidateAttribute;
begin
  try
    FNode.ValidateAttribute(FAttribute);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestValidateElement;
begin
  try
    FNode.ValidateElement(FElement);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestValidateParent;
begin
  FNode.ValidateParent(FElement);
  FNode.ValidateParent(nil);

  try
    FNode.ValidateParent(FNode);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentNodeTest.TestValueErrorInteger;
var
  E: EJvxError;
begin
  E := JvxValueError(0);
  try
    Assert(E <> nil);
    Assert(E is EJvxError);
  finally
    E.Free;
  end;
end;

procedure TDocumentNodeTest.TestValueErrorString;
var
  E: EJvxError;
begin
  E := JvxValueError('0');
  try
    Assert(E <> nil);
    Assert(E is EJvxError);
  finally
    E.Free;
  end;
end;

{ TDocumentNodesTest }

constructor TDocumentNodesTest.Create(AOwner: TComponent);
begin
  inherited;
  AddTest(TestAccept);
  AddTest(TestAdd);
  AddTest(TestAssign);
  AddTest(TestClear);
  AddTest(TestCount);
  AddTest(TestDelete);
  AddTest(TestCompare);
  AddTest(TestIndexOf);
  AddTest(TestInsert);
  AddTest(TestItems);
  AddTest(TestMove);
  AddTest(TestRemove);
  AddTest(TestValidateNode);
end;

procedure TDocumentNodesTest.InternalCreateFixture;
begin
  inherited;
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FNode := TTestNode.Create(nil);
  FParent := TTestNode.Create(nil);
{$WARNINGS ON}
  FNodes := TTestNodes.Create(nil);
  FNodes.Parent := FParent;
  FNodes.OnValidateNode := OnValidateNode;
end;

procedure TDocumentNodesTest.InternalDestroyFixture;
begin
  FreeAndNil(FNode);
  FreeAndNil(FNodes);
  FreeAndNil(FParent);
  FValidatedNode := nil;
  inherited;
end;

procedure TDocumentNodesTest.OnValidateNode(Node: TJvxNode);
begin
  FValidatedNode := Node;
end;

procedure TDocumentNodesTest.TestAccept;
var
  Visitor: TTestVisitor;
begin
  Visitor := TTestVisitor.Create;
  try
    FNodes.Accept(Visitor);
    Assert(Visitor.VisitCount = 0);
  finally
    Visitor.Free;
  end;
end;

procedure TDocumentNodesTest.TestAdd;
begin
  try
    FNodes.Add(FNode);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TDocumentNodesTest.TestAssign;
begin
  // Test that assigning one empy list to another doesn't cause exceptions
  FNodes.Assign(FNodes);

  // Test that assigning nil causes default AssignTo to raise an exception
  try
    FNodes.Assign(nil);
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
end;

procedure TDocumentNodesTest.TestClear;
begin
  FNodes.Clear;
end;

procedure TDocumentNodesTest.TestCount;
begin
  Assert(FNodes.Count = 0);
end;

procedure TDocumentNodesTest.TestDelete;
begin
  try
    FNodes.Delete(0);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TDocumentNodesTest.TestCompare;
begin
  Assert(not FNodes.Compare(nil));
  Assert(FNodes.Compare(FNodes));
end;

procedure TDocumentNodesTest.TestIndexOf;
begin
  Assert(FNodes.IndexOf(nil) = -1);
  Assert(FNodes.IndexOf(FNode) = -1);
end;

procedure TDocumentNodesTest.TestInsert;
begin
  try
    FNodes.Insert(0, FNode);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TDocumentNodesTest.TestItems;
begin
  try
    FNodes.Items[0];
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TDocumentNodesTest.TestMove;
begin
  try
    FNodes.Move(0, 1);
    Assert(EListError);
  except
    Assert(EListError);
  end;
end;

procedure TDocumentNodesTest.TestRemove;
begin
  Assert(FNodes.Remove(FNode) = -1);
end;

procedure TDocumentNodesTest.TestValidateNode;
var
  Composite: TJvxCompositeTextElement;
  Template: TJvxProtectedStringElement;
  CorrectElement: TJvxStringElement;
  IncorrectElement: TJvxIntegerElement;
begin
  try
    FNodes.ValidateNode(nil);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodes.ValidateNode(FNode);
  Assert(FValidatedNode = FNode);

  try
    FNodes.ValidateNode(FParent);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that after Template is assigned, only elements descending form
  // Template's class can be added to the composite element;
  Composite := TJvxCompositeTextElement.Create(nil);
  Template := TJvxProtectedStringElement.Create(nil);
  CorrectElement := TJvxStringElement.Create(nil);
  IncorrectElement := TJvxIntegerElement.Create(nil);
  try
    // Test that without template incorrect element can be added
    IncorrectElement.Parent := Composite;
    IncorrectElement.Parent := nil;

    Composite.Template := Template;

    // Test that element of a class descending from Template's class can be added
    CorrectElement.Parent := Composite;

    // Test that any othe element raises an exception
    try
      IncorrectElement.Parent := Composite;
      Assert(EInvalidOperation);
    except
      Assert(EInvalidOperation);
    end;
  finally
    IncorrectElement.Free;
    CorrectElement.Free;
    Template.Free;
    Composite.Free;
  end;

end;

{ TDocumentNodeListTest }

constructor TDocumentNodeListTest.Create(AOwner: TComponent);
begin
  inherited;
  AddTest(TestAccept);
  AddTest(TestActive);
  AddTest(TestAdd);
  AddTest(TestAssign);
  AddTest(TestClear);
  AddTest(TestCount);
  AddTest(TestDelete);
  AddTest(TestCompare);
  AddTest(TestFindNode);
  AddTest(TestIndexOf);
  AddTest(TestInsert);
  AddTest(TestItems);
  AddTest(TestMove);
  AddTest(TestRemove);
end;

procedure TDocumentNodeListTest.InternalCreateFixture;
begin
  inherited;
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FNotifier := TJvxNotifyComponent.Create(nil);
  FNotifier.OnNotify := Notification;

  FNode := TTestNode.Create(nil);
  FNode.NodeName := 'FNode';
  FNotifier.FreeNotification(FNode);

  FNode2 := TTestNode.Create(nil);
  FNode2.NodeName := 'FNode2';
  FNotifier.FreeNotification(FNode2);

  FNode3 := TTestNode.Create(nil);
  FNode3.NodeName := 'FNode3';
  FNotifier.FreeNotification(FNode3);

  FParent := TTestNode.Create(nil);
  FParent.NodeName := 'FParent';
  FNotifier.FreeNotification(FParent);

{$WARNINGS ON}
  FNodeList := TTestNodeList.Create(nil);
  FNodeList.Parent := FParent;
end;

procedure TDocumentNodeListTest.InternalDestroyFixture;
begin
  FreeAndNil(FNodeList);
  FreeAndNil(FNode);
  FreeAndNil(FNode2);
  FreeAndNil(FNode3);
  FreeAndNil(FParent);
  FreeAndNil(FNotifier);
  inherited;
end;

procedure TDocumentNodeListTest.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation <> opRemove then
    Exit;

  if AComponent = FNode then
    FNode := nil
  else if AComponent = FNode2 then
    FNode2 := nil
  else if AComponent = FNode3 then
    FNode3 := nil
  else if AComponent = FParent then
    FParent := nil;
end;

procedure TDocumentNodeListTest.TestAccept;
var
  Attribute: TJvxAttribute;
  Element: TJvxElement;
  Visitor: TTestVisitor;
begin
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  Attribute := TJvxAttribute.Create(nil);
  Element := TJvxElement.Create(nil);
{$WARNINGS ON}
  Visitor := TTestVisitor.Create;
  try
    FNodeList.Add(Attribute);
    FNodeList.Add(Element);
    FNodeList.Accept(Visitor);
    Assert(Visitor.VisitCount = 2);
    Assert(Visitor.VisitedAttribute = Attribute);
    Assert(Visitor.VisitedElement = Element);
  finally
    Element.Free;
    Attribute.Free;
    Visitor.Free;
  end;
end;

procedure TDocumentNodeListTest.TestAdd;
begin
  // Test that one node can be added to the list
  Assert(FNodeList.Add(FNode) = 0);
  Assert(FNodeList.IndexOf(FNode) = 0);
  Assert(FParent.InsertedNode = FNode);

  // Test that second node can be added without interfering with the first node
  Assert(FNodeList.Add(FNode2) = 1);
  Assert(FNodeList.IndexOf(FNode) = 0);
  Assert(FNodeList.IndexOf(FNode2) = 1);
  Assert(FParent.InsertedNode = FNode2);

  // Test that duplicates are not added to the list
  Assert(FNodeList.Add(FNode) = 0);
  Assert(FNodeList.IndexOf(FNode) = 0);
  Assert(FParent.InsertedNode = FNode2);
end;

procedure TDocumentNodeListTest.TestAssign;
var
  NodeList: TTestNodeList;
begin
  FNodeList.Add(FNode);
  FNodeList.Add(FNode2);
  FNodeList.Add(FNode3);

  NodeList := TTestNodeList.Create(nil);
  try
    // Test that assigning node list makes it equal to the original
    NodeList.Assign(FNodeList);
    Assert(NodeList.Compare(FNodeList));

    // Test that assigning node list creates copies of original nodes
    Assert(NodeList.IndexOf(FNode) < 0);
    Assert(NodeList.IndexOf(FNode2) < 0);
    Assert(NodeList.IndexOf(FNode3) < 0);
  finally
    NodeList.Free;
  end;
end;

procedure TDocumentNodeListTest.TestClear;
var
  Node: TJvxNode;
begin
  FNodeList.Add(FNode);
  Assert(FNodeList.Count = 1);
  Node := FNode;
  FNodeList.Clear;
  Assert(FNodeList.Count = 0);
  Assert(FNode = nil);
  Assert(FParent.RemovedNode = Node);
end;

procedure TDocumentNodeListTest.TestCount;
begin
  Assert(FNodeList.Count = 0);
  FNodeList.Add(FNode);
  Assert(FNodeList.Count = 1);
end;

procedure TDocumentNodeListTest.TestDelete;
begin
  FNodeList.Add(FNode);
  Assert(FNodeList.Count = 1);
  Assert(FParent.InsertedNode = FNode);

  FNodeList.Delete(0);
  Assert(FNodeList.Count = 0);
  Assert(FNode <> nil); // Item should not be destroyed when it was deleted
  Assert(FParent.RemovedNode = FNode);
end;

procedure TDocumentNodeListTest.TestCompare;
var
  NodeList: TTestNodeList;
begin
  FNodeList.Add(FNode);
  FNodeList.Add(FNode2);

  NodeList := TTestNodeList.Create(nil);
  try
    // Test that empty node list is not Compare to the one having items
    Assert(not FNodeList.Compare(NodeList));

    // Test that two node lists with the same items are equal
    NodeList.Add(FNode);
    NodeList.Add(FNode2);
    Assert(FNodeList.Compare(NodeList));

    // Test that order of items is important
    NodeList.Move(0, 1);
    Assert(not FNodeList.Compare(NodeList));
  finally
    NodeList.Free;
  end;
end;

procedure TDocumentNodeListTest.TestFindNode;
begin
  FNodeList.Add(FNode);
  FNodeList.Add(FNode2);
  FNodeList.Add(FNode3);
  Assert(FNodeList.FindNode('FNode') = FNode);
  Assert(FNodeList.FindNode('FNode2') = FNode2);
  Assert(FNodeList.FindNode('FNode3') = FNode3);
  Assert(FNodeList.FindNode('Test') = nil);
end;

procedure TDocumentNodeListTest.TestIndexOf;
begin
  FNodeList.Add(FNode);
  FNodeList.Add(FNode2);
  FNodeList.Add(FNode3);

  Assert(FNodeList.IndexOf(FNode) = 0);
  Assert(FNodeList.IndexOf(FNode2) = 1);
  Assert(FNodeList.IndexOf(FNode3) = 2);
end;

procedure TDocumentNodeListTest.TestInsert;
begin
  Assert(FNodeList.Count = 0);

  // Test that node can be inserted into an empty list
  Assert(FNodeList.Insert(0, FNode) = 0);
  Assert(FNodeList.Count = 1);
  Assert(FNodeList.IndexOf(FNode) = 0);
  Assert(FParent.InsertedNode = FNode);

  // Test that node can be inserted in the middle of a non-empty list
  Assert(FNodeList.Insert(0, FNode2) = 0);
  Assert(FNodeList.Count = 2);
  Assert(FNodeList.IndexOf(FNode2) = 0);
  Assert(FNodeList.IndexOf(FNode) = 1);
  Assert(FParent.InsertedNode = FNode2);

  // Test that node can be inserted in the end of a non-empty list
  Assert(FNodeList.Insert(2, FNode3) = 2);
  Assert(FNodeList.Count = 3);
  Assert(FNodeList.IndexOf(FNode2) = 0);
  Assert(FNodeList.IndexOf(FNode) = 1);
  Assert(FNodeList.IndexOf(FNode3) = 2);
  Assert(FParent.InsertedNode = FNode3);

  // Test that duplicates are ignored
  Assert(FNodeList.Insert(3, FNode) = 1);
  Assert(FNodeList.Count = 3);
  Assert(FNodeList.IndexOf(FNode) = 1);
  Assert(FParent.InsertedNode = FNode3);

  while FNodeList.Count > 0 do
    FNodeList.Delete(0);

  // Test that node cannot be inserted with an index less than 0
  try
    FNodeList.Insert(-1, FNode);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test that node cannot be inserted with an index greater than number of items
  try
    FNodeList.Insert(1, FNode);
    Assert(EListError);
  except
    Assert(EListError);
  end;
end;

procedure TDocumentNodeListTest.TestItems;
begin
  try
    FNodeList.Items[0];
    Assert(EListError);
  except
    Assert(EListError);
  end;

  FNodeList.Add(FNode);
  Assert(FNodeList.Items[0] = FNode);
end;

procedure TDocumentNodeListTest.TestMove;
begin
  FNodeList.Add(FNode);
  FNodeList.Add(FNode2);
  FNodeList.Add(FNode3);

  // Test that moving to invalid index raises an exception
  try
    FNodeList.Move(0, 3);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test moving from index and to index
  FNodeList.Move(0, 2);
  Assert(FNodeList.IndexOf(FNode) = 2);
  Assert(FNodeList.IndexOf(FNode2) = 0);
  Assert(FNodeList.IndexOf(FNode3) = 1);
end;

procedure TDocumentNodeListTest.TestRemove;
begin
  FNodeList.Add(FNode);
  Assert(FNodeList.Count = 1);
  Assert(FParent.InsertedNode = FNode);

  FNodeList.Remove(FNode);
  Assert(FNodeList.Count = 0);
  Assert(FNode <> nil); // Item should not be destroyed when it was removed
  Assert(FParent.RemovedNode = FNode);
end;

procedure TDocumentNodeListTest.TestActive;
begin
  // Test that TJvxListDataSource opens itself when created
  Assert(FNodeList.DataSource.Active);

  // Test that Active property compares new and old values
  FNodeList.DataSource.Active := True;

  // Test that Open checks Active status and doesn't do anything if data source
  // is already opened
  FNodeList.DataSource.Open;

  // Test that list data source cannot be closed
  try
    FNodeList.DataSource.Active := False;
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

{ TDocumentAttributeTest }

constructor TDocumentAttributeTest.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  AddTest(TestAccept);
  AddTest(TestAsBoolean);
  AddTest(TestAsDateTime);
  AddTest(TestAsFloat);
  AddTest(TestAsInteger);
  AddTest(TestAsString);
  AddTest(TestAsVariant);
  AddTest(TestDestroy);
  AddTest(TestSetParent);
end;

procedure TDocumentAttributeTest.InternalCreateFixture;
begin
  inherited;
  FElement := TTestElement.Create(nil);
  FElement2 := TTestElement.Create(nil);
  FAttribute := TTestAttribute.Create(nil);
end;

procedure TDocumentAttributeTest.InternalDestroyFixture;
begin
  FreeAndNil(FAttribute);
  FreeAndNil(FElement2);
  FreeAndNil(FElement);
  inherited;
end;

procedure TDocumentAttributeTest.TestAccept;
var
  Visitor: TTestVisitor;
begin
  Visitor := TTestVisitor.Create;
  try
    FAttribute.Accept(Visitor);
    Assert(Visitor.VisitCount = 1);
    Assert(Visitor.VisitedAttribute = FAttribute);
  finally
    Visitor.Free;
  end;
end;

procedure TDocumentAttributeTest.TestAsBoolean;
begin
  // Test that SetAsBoolean updates Assigned status and sends notification
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsBoolean := False;
  Assert(FAttribute.Assigned = True);
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that SetAsBoolean compares old and new values
  FAttribute.FChangedNode := nil;
  FAttribute.AsBoolean := False;
  Assert(FAttribute.ChangedNode = nil);
  FAttribute.AsBoolean := True;
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that GetAsBoolean always raises an exception for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsBoolean;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentAttributeTest.TestAsDateTime;
begin
  // Test that SetAsDateTime updates Assigned status and sends notification
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsDateTime := 0;
  Assert(FAttribute.Assigned = True);
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that SetAsDateTime compares old and new values
  FAttribute.FChangedNode := nil;
  FAttribute.AsDateTime := 0;
  Assert(FAttribute.ChangedNode = nil);
  FAttribute.AsDateTime := Now;
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that GetAsDateTime always raises an exception for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsDateTime;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentAttributeTest.TestAsFloat;
begin
  // Test that SetAsFloat updates Assigned status and sends notification
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsFloat := 0;
  Assert(FAttribute.Assigned = True);
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that SetAsFloat compares old and new values
  FAttribute.FChangedNode := nil;
  FAttribute.AsFloat := 0;
  Assert(FAttribute.ChangedNode = nil);
  FAttribute.AsFloat := 3.14;
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that GetAsFloat always returns 0 for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentAttributeTest.TestAsInteger;
begin
  // Test that SetAsInteger updates Assigned status and sends notification
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsInteger := 0;
  Assert(FAttribute.Assigned = True);
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that SetAsInteger compares old and new values
  FAttribute.FChangedNode := nil;
  FAttribute.AsInteger := 0;
  Assert(FAttribute.ChangedNode = nil);
  FAttribute.AsInteger := 314;
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that GetAsInteger always returns 0 for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsInteger;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TDocumentAttributeTest.TestAsString;
begin
  // Test that SetAsString compares old and new values with EmptyStr
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsString := 'Test';
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that SetAsString compares old and new values
  FAttribute.FChangedNode := nil;
  FAttribute.AsString := 'Test';
  Assert(FAttribute.ChangedNode = nil);

  // Test that GetAsString always returns an empty string for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigning an empty string doesn't call InternalSetAsString - only
  // InternalSetAssigned
  FAttribute.AsString := 'Test';
  FAttribute.AsString := EmptyStr;
  Assert(FAttribute.Assigned = False);
  Assert(FAttribute.InternalGetAsString = 'Test');
end;

procedure TDocumentAttributeTest.TestAsVariant;
begin
  // Test that SetAsVariant compares old and new values with null and unassigned
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FAttribute.AsVariant := Null;
  Assert(FAttribute.ChangedNode = nil);
  Assert(FAttribute.Assigned = False);

  // Test that SetAsVariant calls InternalSetAsVariant, InternalSetAssigned and
  // sends document change notification
  Assert(FAttribute.Assigned = False);
  FAttribute.AsVariant := '314';
  Assert(FAttribute.AsVariant = '314');
  Assert(Fattribute.Assigned = True);
  Assert(FAttribute.ChangedNode = FAttribute);

  // Test that GetAsVariant always returns an empty string for unnassigned nodes
  FAttribute.Assigned := False;
  try
    FAttribute.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that assigning Null doesn't call InternalSetAsVariant - it only calls
  // InternalSetAssigned
  FAttribute.AsVariant := 'Test';
  FAttribute.AsVariant := Null;
  Assert(FAttribute.Assigned = False);
  Assert(FAttribute.InternalGetAsString = 'Test');

  // Test that Unassigned raises an exception
  try
    FAttribute.AsVariant := Unassigned;
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TDocumentAttributeTest.TestDestroy;
begin
  // Test that attribute is removed from the list of it's parent when destroyed
  FElement.Attributes.Add(FAttribute);
  Assert(FElement.Attributes.Count = 2);
  FAttribute.Free;
  Assert(FElement.Attributes.Count = 1);
  Assert(FElement.RemovedNode = FAttribute);
  FAttribute := nil;
end;

procedure TDocumentAttributeTest.TestSetParent;
begin
  // Test that attribute is added to the list of it's parent
  Assert(FElement.Attributes.IndexOf(FAttribute) < 0);
  FAttribute.Parent := FElement;
  Assert(FElement.Attributes.IndexOf(FAttribute) >= 0);
  Assert(FElement.InsertedNode = FAttribute);
  Assert(FElement2.Attributes.IndexOf(FAttribute) < 0);

  // Test that attribute is removed from the list of it's previous parent
  FAttribute.Parent := FElement2;
  Assert(FElement.Attributes.IndexOf(FAttribute) < 0);
  Assert(FElement.RemovedNode = FAttribute);
  Assert(FElement2.Attributes.IndexOf(FAttribute) >= 0);
  Assert(FElement2.InsertedNode = FAttribute);

  // Test that attribute is removed from the list when Parent is assigned to nil
  FAttribute.Parent := nil;
  Assert(FElement2.Attributes.IndexOf(FAttribute) < 0);
  Assert(FElement2.RemovedNode = FAttribute);
end;

{ TDocumentElementTest }

constructor TDocumentElementTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAccept);
  AddTest(TestAssign);
  AddTest(TestAttributes);
  AddTest(TestCreateDefinedAttributes);
  AddTest(TestDestroy);
  AddTest(TestDestroyAttributes);
  AddTest(TestDocumentNotification);
  AddTest(TestCompare);
  AddTest(TestParent);
end;

procedure TDocumentElementTest.InternalCreateFixture;
begin
  inherited;
  FNotifier := TJvxNotifyComponent.Create(nil);
  FNotifier.OnNotify := Notification;

{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FComposite := TTestComposite.Create(nil);
  FNotifier.FreeNotification(FComposite);

  FComposite2 := TTestComposite.Create(nil);
  FNotifier.FreeNotification(FComposite2);

  FElement := TTestElement.Create(nil);
  FNotifier.FreeNotification(FElement);
{$WARNINGS ON}

  FAttribute := TTestAttribute.Create(nil);
  FNotifier.FreeNotification(FAttribute);
end;

procedure TDocumentElementTest.InternalDestroyFixture;
begin
  FAttribute.Free;
  FElement.Free;
  FComposite2.Free;
  FComposite.Free;
  FNotifier.Free;
  inherited;
end;

procedure TDocumentElementTest.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = FAttribute then
      FAttribute := nil
    else if AComponent = FElement then
      FElement := nil
    else if AComponent = FComposite then
      FComposite := nil
    else if AComponent = FComposite2 then
      FComposite2 := nil
  end;
end;

procedure TDocumentElementTest.TestAccept;
var
  Visitor: TTestVisitor;
begin
  Visitor := TTestVisitor.Create;
  try
    FElement.Accept(Visitor);
    Assert(Visitor.VisitCount = 1);
    Assert(Visitor.VisitedElement = FElement);
    Assert(Visitor.VisitedAttribute = nil);
  finally
    Visitor.Free;
  end;
end;

procedure TDocumentElementTest.TestAssign;
var
  Element: TTestElement;
begin
  FElement.Name := 'FTestElement';
  FElement.NodeName := 'test-element';
  FElement.Value := 'test element value';

  FAttribute.Name := 'FTestAttribute';
  FAttribute.NodeName := 'test-attribute';
  FAttribute.Value := 'test attribute value';

  FElement.Attributes.Add(FAttribute);

{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  Element := TTestElement.Create(nil);
{$WARNINGS ON}
  try
    Assert(not Element.Compare(FElement));
    Element.Assign(FElement);
    Assert(Element.Compare(FElement));
  finally
    Element.Free;
  end;
end;

procedure TDocumentElementTest.TestAttributes;
begin
  Assert(FElement.Attributes <> nil);
  Assert(FElement.Attributes is TJvxNodes);
  Assert(FElement.Attributes.Count = 1);
  FElement.Attributes.Add(FAttribute);
  Assert(FElement.Attributes.Count = 2);
end;

procedure TDocumentElementTest.TestCreateDefinedAttributes;
begin
  // Test that predefined attribute was created
  Assert(FElement.Attribute is TTestAttribute);
  Assert(FElement.Attribute.NodeName = 'TestAttribute');
end;

procedure TDocumentElementTest.TestDestroy;
var
  Attribute: TJvxNode;
begin
  // Test that element is removed from the list of it's parent when destroyed
  FElement.Attributes.Add(FAttribute);
  Assert(FElement.Attributes.Count = 2);
  Attribute := FAttribute;
  FAttribute.Free;
  Assert(FElement.Attributes.Count = 1);
  Assert(FElement.RemovedNode = Attribute);
end;

procedure TDocumentElementTest.TestDestroyAttributes;
begin
  // Test that attributes are destroyed with their parent
  FElement.Attributes.Add(FAttribute);
  FElement.Free;
  Assert(FAttribute = nil);
end;

procedure TDocumentElementTest.TestDocumentNotification;
var
  Node: TJvxNode;
begin
  // Test that element broadcasts notificaitons to its attributes
  FElement.Attributes.Insert(0, FAttribute);
  FElement.Notify(noChange);
  Assert(FAttribute.ChangedNode = FElement);

  // Test that attributes notify their parent
  Node := FAttribute;
  FElement.Attributes.Clear;
  Assert(FElement.RemovedNode = Node);
end;

procedure TDocumentElementTest.TestCompare;
var
  Attribute: TTestAttribute;
  Element: TTestElement;
begin
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  Element := TTestElement.Create(nil);
{$WARNINGS ON}
  Attribute := TTestAttribute.Create(nil);
  try
    Assert(Element.Compare(FElement));

    // Test that empty elements list is considered equal to nil
    FElement.Attributes.Add(FAttribute);
    FElement.Attributes.Remove(FAttribute);
    Assert(Element.Compare(FElement));

    // Test that Attributes are compared
    FElement.Attributes.Add(FAttribute);
    Assert(not Element.Compare(FElement));
    Element.Attributes.Add(Attribute);
    Assert(Element.Compare(FElement));

    // Test that each individual attribute is compared
    FAttribute.NodeName := 'test';
    Assert(not Element.Compare(FElement));
    Attribute.NodeName := 'test';
    Assert(Element.Compare(FElement));

    // Test that inherited Compare method is called
    FElement.NodeName := 'test';
    Assert(not Element.Compare(FElement));
    Element.NodeName := 'test';
    Assert(Element.Compare(FElement));

    FElement.Name := 'FElement';
    Assert(not Element.Compare(FElement));
    Element.Name := 'FElement';
    Assert(Element.Compare(FElement));

    // Test that value is compared
    FElement.Value := 'test value';
    Assert(not Element.Compare(FElement));
    Element.Value := 'test value';
    Assert(Element.Compare(FElement));
  finally
    Attribute.Free;
    Element.Free;
  end;
end;

procedure TDocumentElementTest.TestParent;
begin
  // Test that element is added to the list of it's parent
  Assert(FComposite.Elements.IndexOf(FElement) < 0);
  FElement.Parent := FComposite;
  Assert(FComposite.Elements.IndexOf(FElement) >= 0);
  Assert(FComposite2.Elements.IndexOf(FElement) < 0);
  Assert(FComposite.InsertedNode = FElement);

  // Test that element is removed from the list of it's previous parent
  FElement.Parent := FComposite2;
  Assert(FComposite.Elements.IndexOf(FElement) < 0);
  Assert(FComposite2.Elements.IndexOf(FElement) >= 0);
  Assert(FComposite.RemovedNode = FElement);
  Assert(FComposite2.InsertedNode = FElement);

  // Test that element is removed from the list when Parent is assigned to nil
  FElement.Parent := nil;
  Assert(FComposite.Elements.IndexOf(FElement) < 0);
  Assert(FComposite2.Elements.IndexOf(FElement) < 0);
  Assert(FComposite2.RemovedNode = FElement);
end;

{ TCompositeDocumentElementTest }

constructor TCompositeDocumentElementTest.Create(AOwner: TComponent);
begin
  inherited;
  AddTest(TestAccept);
  AddTest(TestAssign);
  AddTest(TestCreateDefinedElements);
  AddTest(TestDestroy);
  AddTest(TestDocumentNotification);
  AddTest(TestElements);
  AddTest(TestCompare);
  AddTest(TestHasAttributes);
  AddTest(TestHasElements);
  AddTest(TestNotification);
  AddTest(TestStreaming);
  AddTest(TestTemplate);
end;

procedure TCompositeDocumentElementTest.InternalCreateFixture;
begin
  inherited;
  FNotifier := TJvxNotifyComponent.Create(nil);
  FNotifier.OnNotify := Notification;

{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FParentComposite := TTestComposite.Create(nil);
  FNotifier.FreeNotification(FParentComposite);

  FComposite := TTestComposite.Create(FParentComposite);
  FNotifier.FreeNotification(FComposite);

  FElement := TTestElement.Create(FParentComposite);
  FNotifier.FreeNotification(FElement);
{$WARNINGS ON}

  FAttribute := TTestAttribute.Create(FParentComposite);
  FNotifier.FreeNotification(FAttribute);
end;

procedure TCompositeDocumentElementTest.InternalDestroyFixture;
begin
  FAttribute.Free;
  FElement.Free;
  FComposite.Free;
  FParentComposite.Free;
  FNotifier.Free;
  inherited;
end;

procedure TCompositeDocumentElementTest.Notification(
  AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = FParentComposite then
      FParentComposite := nil
    else if AComponent = FComposite then
      FComposite := nil
    else if AComponent = FElement then
      FElement := nil
    else if AComponent = FAttribute then
      FAttribute := nil;
  end;
end;

procedure TCompositeDocumentElementTest.TestAccept;
var
  Visitor: TTestVisitor;
begin
  Visitor := TTestVisitor.Create;
  try
    // Test that composite calls correct Visit method for itself and for its elements
    FComposite.Accept(Visitor);
    Assert(Visitor.VisitCount = 1);
    Assert(Visitor.VisitedElement = FComposite);
  finally
    Visitor.Free;
  end;
end;

procedure TCompositeDocumentElementTest.TestAssign;
var
  Composite: TTestComposite;
begin
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  Composite := TTestComposite.Create(nil);
{$WARNINGS ON}
  try
    FComposite.Elements.Add(FElement);
    Composite.Assign(FComposite);
    Assert(Composite.Compare(FComposite));
  finally
    Composite.Free;
  end;
end;

procedure TCompositeDocumentElementTest.TestCreateDefinedElements;
begin
  // Test that predefined element was created
  Assert(FComposite.Element is TTestElement);
  Assert(FComposite.Element.NodeName = 'TestElement');
end;

procedure TCompositeDocumentElementTest.TestDestroy;
begin
  // Test that composite destroys all its elements
  FComposite.Elements.Add(FElement);
  FComposite.Free;
  Assert(FElement = nil);
end;

procedure TCompositeDocumentElementTest.TestDocumentNotification;
var
  Node: TJvxNode;
begin
  // Test that composite broadcasts notificaitons to its elements
  FComposite.Elements.Insert(0, FElement);
  Assert(FElement.InsertedNode = FElement);
  FComposite.Notify(noChange);
  Assert(FElement.ChangedNode = FComposite);

  // Test that elements notify their parents
  Node := FElement;
  FComposite.Elements.Clear;
  Assert(FComposite.RemovedNode = Node);
end;

procedure TCompositeDocumentElementTest.TestElements;
begin
  Assert(FComposite.Elements <> nil);
  Assert(FComposite.Elements is TJvxNodes);
  FComposite.Elements.Add(FElement);
  Assert(FComposite.Elements.Count = 2);
  Assert(FComposite.InsertedNode = FElement);
end;

procedure TCompositeDocumentElementTest.TestCompare;
var
  Composite: TTestComposite;
  Element: TTestElement;
begin
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  Composite := TTestComposite.Create(nil);
  Element := TTestElement.Create(nil);
{$WARNINGS ON}
  try
    Assert(Composite.Compare(FComposite));

    // Test that empty elements list is considered equal to nil
    FComposite.Elements.Add(FElement);
    FComposite.Elements.Remove(FElement);
    Assert(Composite.Compare(FComposite));

    // Test that elements are compared
    FComposite.Elements.Add(FElement);
    Assert(not Composite.Compare(FComposite));
    Composite.Elements.Add(Element);
    Assert(Composite.Compare(FComposite));

    // Test that every element in the list is compared
    FElement.NodeName := 'test';
    Assert(not Composite.Compare(FComposite));
    Element.NodeName := 'test';
    Assert(Composite.Compare(FComposite));
  finally
    Element.Free;
    Composite.Free;
  end;
end;

procedure TCompositeDocumentElementTest.TestHasAttributes;
begin
  // Test that notification clears attributes collection
  Assert(FElement.HasAttributes);
  FElement.Attributes.Free;
  Assert(not FElement.HasAttributes);
end;

procedure TCompositeDocumentElementTest.TestHasElements;
begin
  // Test that notification clears elements collection
  Assert(FComposite.HasElements);
  FComposite.Elements.Free;
  Assert(not FComposite.HasElements);
end;

procedure TCompositeDocumentElementTest.TestNotification;
begin
  // Test that notification clears predefined attributes
  Assert(FElement.Attribute <> nil);
  FElement.Attribute.Free;
  Assert(FElement.Attribute = nil);

  // Test that notification clears predefined elements
  Assert(FComposite.Element <> nil);
  FComposite.Element.Free;
  ASsert(FComposite.Element = nil);
end;

procedure TCompositeDocumentElementTest.TestStreaming;

  procedure DFMStreamToFile(Stream: TStream; FileName: string);
  var
    FileStream: TFileStream;
  begin
    FileStream := TFileStream.Create(FileName, fmCreate);
    try
      Stream.Seek(0, soFromBeginning);
      ObjectBinaryToText(Stream, FileStream);
    finally
      FileStream.Free;
    end;
  end;

var
  Composite: TTestComposite;
  DoubleCheckComposite: TTestComposite;
  DoubleCheckStream: TMemoryStream;
  Stream: TMemoryStream;
begin
  // Prepare parent
  FParentComposite.Name := 'FTestParentComposite';
  FParentComposite.NodeName := 'TestParentComposite';
//  FParentComposite.Element.Value := 'New Value of Predefined Element';
  FParentComposite.Element.Assigned := False;
  FParentComposite.Element.Attribute.Value := 'New Value of Predefined Attribute';
  FParentComposite.Text.Add('FTestParentComposite line 1');
  FParentComposite.Text.Add('FTestParentComposite line 2');
  FParentComposite.Text.Add('FTestParentComposite line 3');

  // Prepare composite
  FComposite.Name := 'FTestComposite';
  FComposite.NodeName := 'TestComposite';
//  FComposite.Element.Value := 'New Value of Predefined Element';
  FComposite.Element.Assigned := False;
  FComposite.Text.Add('FComposite line 1');
  FComposite.Text.Add('FComposite line 2');
  FComposite.Text.Add('FComposite line 3');
  FComposite.Parent := FParentComposite;

  // Prepare element
  FElement.Name := 'FTestElement';
  FElement.NodeName := 'TestElement';
  FElement.Attribute.Value := 'New Value of Predefined Attribute';
  FElement.OnNotify := FParentComposite.DocumentNotification;
  FElement.Parent := FComposite;

  // Prepare attribute
  FAttribute.Name := 'FTestAttribute';
  FAttribute.NodeName := 'TestAttribute';
  FAttribute.Value := 'TestAttributeValue';
  FAttribute.OnNotify := FParentComposite.DocumentNotification;
  FAttribute.Parent := FElement;

  // Write parent composite to a stream, read it back and compare
  Composite := nil;
  DoubleCheckComposite := nil;
  Stream := TMemoryStream.Create;
  DoubleCheckStream := TMemoryStream.Create;
  try
    Stream.WriteComponent(FParentComposite);
    DFMStreamToFile(Stream, 'd:\stream.dfm');
    Stream.Seek(0, soFromBeginning);
    Composite := Stream.ReadComponent(nil) as TTestComposite;
    Assert(Composite.Compare(FParentComposite));

    // Double check
    DoubleCheckStream.WriteComponent(Composite);
    DFMStreamToFile(DoubleCheckStream, 'd:\stream2.dfm');
    DoubleCheckStream.Seek(0, soFromBeginning);
    DoubleCheckComposite := DoubleCheckStream.ReadComponent(nil) as TTestComposite;
    Assert(DoubleCheckComposite.Compare(Composite));
    Assert(DoubleCheckComposite.Compare(FParentComposite));
    Assert(Stream.Size = DoubleCheckStream.Size);
    Assert(SysUtils.CompareMem(Stream.Memory, DoubleCheckStream.Memory, Stream.Size));
  finally
    Composite.Free;
    DoubleCheckComposite.Free;
    Stream.Free;
    DoubleCheckStream.Free;
  end;
end;

procedure TCompositeDocumentElementTest.TestTemplate;
var
  Template: TJvxStringElement;
begin
  Template := TJvxStringElement.Create(nil);
  try
    // Test that template cannot be assigned while there are child elements
    try
      FComposite.Template := Template;
      Assert(EInvalidOperation);
    except
      Assert(EInvalidOperation);
    end;

    // Test that template can be assigned if the elements list is clear
    FComposite.Elements.Clear;
    FComposite.Template := Template;
  finally
    Template.Free;
  end;

  // Test that template property is cleared when the object is destroyed
  Assert(FComposite.Template = nil);

  // Test that template is freed if it belongs to the composite
  FComposite.Template := FElement;
  FElement.Owner.RemoveComponent(FElement);
  FComposite.InsertComponent(FElement);
  FComposite.Free;
  Assert(FElement = nil);
end;

{ TNodeFieldTest }

constructor TNodeFieldTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAccessError);
  AddTest(TestAssigned);
  AddTest(TestChanged);
  AddTest(TestGetAs);
  AddTest(TestSetDataSource);
end;

procedure TNodeFieldTest.InternalCreateFixture;
begin
  inherited;
  FAttribute := TTestAttribute.Create(nil);
{$warnings off} // turn off "constructing instance of a class containing abstract methods"
  FNodeField := TJvxDataField.Create(nil);
{$warnings on}
  FTestNodeField := TTestNodeField.Create(nil);
  FDataSource := TTestDataSource.Create(nil);
end;

procedure TNodeFieldTest.InternalDestroyFixture;
begin
  FDataSource.Free;
  FAttribute.Free;
  FNodeField.Free;
  FTestNodeField.Free;
  inherited;
end;

procedure TNodeFieldTest.TestAccessError;
var
  E: EInvalidOperation;
begin
  E := JvxAccessError(TypeInfo(Boolean));
  try
    Assert(E is EInvalidOperation);
    Assert(Pos('Boolean', E.Message) > 0);
  finally
    E.Free;
  end;
end;

procedure TNodeFieldTest.TestAssigned;
begin
  try
    FNodeField.Assigned;
    Assert(EAbstractError);
  except
    Assert(EAbstractError);
  end;
end;

procedure TNodeFieldTest.TestChanged;
begin
  // Test that calling Changed sends notification to the node
  FAttribute.DataField := FTestNodeField;
  Assert(FAttribute.Assigned = False);
  FAttribute.FChangedNode := nil;
  FTestNodeField.Value := 284;
  Assert(FAttribute.AsVariant = 284);
  Assert(FAttribute.ChangedNode = FAttribute);
end;

procedure TNodeFieldTest.TestGetAs;
begin
  // Test that GetAsFloat raises an exception if value is unassigned
  try
    FTestNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that GetAsOrdinal raises an exception if value is unassigned
  try
    FTestNodeField.AsOrdinal;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that GetAsString raises an exception if value is unassigned
  try
    FTestNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  // Test that GetAsVariant raises an exception if value is unassigned
  try
    FTestNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TNodeFieldTest.TestSetDataSource;
begin
  // Test that setting node set adds/removes field to/from TJvxDataSource.Fields
  Assert(FDataSource.Fields.IndexOf(FTestNodeField) < 0);
  FTestNodeField.FieldName := 'test';
  FTestNodeField.DataSource := FDataSource;
  Assert(FDataSource.Fields.IndexOf(FTestNodeField) >= 0);
  FTestNodeField.DataSource := nil;
  Assert(FDataSource.Fields.IndexOf(FTestNodeField) < 0);
end;

{ TNodeFieldsTest }

constructor TNodeFieldsTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAdd);
  AddTest(TestClear);
  AddTest(TestCount);
  AddTest(TestDelete);
  AddTest(TestDestroy);
  AddTest(TestFieldByName);
  AddTest(TestFieldDestroy);
  AddTest(TestFindField);
  AddTest(TestGetItem);
  AddTest(TestIndexOf);
  AddTest(TestInsert);
  AddTest(TestRemove);
  AddTest(TestValidation);
end;

procedure TNodeFieldsTest.InternalCreateFixture;
begin
  inherited;
  FNotifier := TJvxNotifyComponent.Create(nil);
  FNotifier.OnNotify := Notification;

  FNodeField := TTestNodeField.Create(nil);
  FNodeField.FieldName := 'test';
  FNodeField.FreeNotification(FNotifier);

  FDataSource := TTestDataSource.Create(nil);
  FDataSource.FreeNotification(FNotifier);
end;

procedure TNodeFieldsTest.InternalDestroyFixture;
begin
  FreeAndNil(FNodeField);
  FreeAndNil(FDataSource);
  FreeAndNil(FNotifier);
  inherited;
end;

procedure TNodeFieldsTest.Notification(Component: TComponent;
  Operation: TOperation);
begin
  if Operation = opRemove then
  begin
    if Component = FDataSource then
      FDataSource := nil
    else if Component = FNodeField then
      FNodeField := nil;
  end;
end;

procedure TNodeFieldsTest.TestAdd;
begin
  Assert(FDataSource.Fields.Count = 0);
  Assert(FDataSource.Fields.Add(FNodeField) = 0);
  Assert(FDataSource.Fields.Count = 1);
end;

procedure TNodeFieldsTest.TestClear;
begin
  // Test that Clear deletes and destroys all items
  FDataSource.Fields.Add(FNodeField);
  FDataSource.Fields.Clear;
  Assert(FDataSource.Fields.Count = 0);
  Assert(FNodeField = nil);
end;

procedure TNodeFieldsTest.TestCount;
begin
  Assert(FDataSource.Fields.Count = 0);
  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.Fields.Count = 1);
end;

procedure TNodeFieldsTest.TestDelete;
begin
  FDataSource.Fields.Add(FNodeField);

  // Test Delete with Index >= Count
  try
    FDataSource.Fields.Delete(1);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test Delete with Index < 0
  try
    FDataSource.Fields.Delete(-1);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test Delete with valid parameters
  Assert(FDataSource.Fields.Delete(0) = FNodeField);
  Assert(FNodeField.DataSource = nil);
  Assert(FDataSource.Fields.Count = 0);
end;

procedure TNodeFieldsTest.TestDestroy;
var
  Fields: TJvxDataFields;
begin
  Fields := FDataSource.Fields;
  Fields.Add(FNodeField);
  Fields.Free;

  // Test that TJvxDataFields destroys its fields when destroyed
  Assert(FNodeField = nil);
  // Test that TJvxDataFields clears FFields of TJvxDataSource when destroyed
  // so that next time Fields property is invoked the object gets recreated
  Assert(not JvxValidObject(Fields));
  Assert(JvxValidObject(FDataSource.Fields));
end;

procedure TNodeFieldsTest.TestFieldByName;
begin
  FNodeField.FieldName := 'test';
  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.Fields.FieldByName('test') = FNodeField);

  // test that FieldByName raises an exception if field not found
  try
    FDataSource.Fields.FieldByName('invalidfield');
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

procedure TNodeFieldsTest.TestFieldDestroy;
begin
  FDataSource.Fields.Add(FNodeField);
  FreeAndNil(FNodeField);
  Assert(FDataSource.Fields.Count = 0);
end;

procedure TNodeFieldsTest.TestFindField;
begin
  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);

  // Test that FindField is not case sensitive
  Assert(FDataSource.Fields.FindField('Field1') <> nil);
  Assert(FDataSource.Fields.FindField('FIELD1') = FDataSource.Fields.FindField('Field1'));

  // Test that FindField returns nil if field could not be found
  Assert(FDataSource.Fields.FindField('InvalidField') = nil);
end;

procedure TNodeFieldsTest.TestGetItem;
begin
  // Test that GetItem raises EListError even if the internal list is not created
  try
    FDataSource.Fields.Items[0];
    Assert(EListError);
  except
    Assert(EListError);
  end;

  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.Fields[0] = FNodeField);
end;

procedure TNodeFieldsTest.TestIndexOf;
begin
  // Test that with empty internal list IndexOf still returns -1
  Assert(FDataSource.Fields.IndexOf(FNodeField) = -1);

  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);

  Assert(FDataSource.Fields.IndexOf(FNodeField) = -1);
  Assert(FDataSource.Fields.Add(FNodeField) = FDataSource.Fields.IndexOf(FNodeField));
end;

procedure TNodeFieldsTest.TestInsert;
begin
  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);

  // Test that Index > Count raises an exception
  try
    FDataSource.Fields.Insert(4, FNodeField);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test that Index < 0 raises an exception
  try
    FDataSource.Fields.Insert(-1, FNodeField);
    Assert(EListError);
  except
    Assert(EListError);
  end;

  // Test execution of Insert with valid parameters
  FDataSource.Fields.Insert(2, FNodeField);
  Assert(FDataSource.Fields.Count = 4);
  Assert(FDataSource.Fields.IndexOf(FNodeField) = 2);
  Assert(FDataSource.FValidatedField = FNodeField);
  Assert(FNodeField.DataSource = FDataSource);
end;

procedure TNodeFieldsTest.TestRemove;
begin
  // Test that removing invalid field raises EListError
  Assert(FDataSource.Fields.Remove(FNodeField) = -1);

  // Test Remove with valid parameters
  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.Fields.Remove(FNodeField) = 0);
end;

procedure TNodeFieldsTest.TestValidation;
begin
  // Test that adding nil raises an exception
  try
    FDataSource.Fields.Add(nil);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that adding the same field twice will raise an exception
  FNodeField.FieldName := 'Test';
  FDataSource.Fields.Add(FNodeField);
  try
    FDataSource.Fields.Add(FNodeField);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that duplicate field names are not allowed
  FDataSource.Fields.Remove(FNodeField);
  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);
  FNodeField.FieldName := 'Field1';
  try
    FDataSource.Fields.Add(FNodeField);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that empty field names are not allowed
  FNodeField.FieldName := EmptyStr;
  try
    FDataSource.Fields.Add(FNodeField);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that node set validates fields
  FDataSource.FValidatedField := nil;
  FNodeField.FieldName := 'test';
  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.FValidatedField = FNodeField);

  // Test that renaming field will validate its name
  try
    FNodeField.FieldName := 'Field1';
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that when changing field's name it is validated only if it is
  // different from previous, case insensitive
  Assert(FNodeField.FieldName = 'test');
  FNodeField.FieldName := 'TEST';
  Assert(FNodeField.FieldName = 'TEST'); 
end;

{ TTypedNodeTest }

procedure TTypedNodeTest.AssignNodeValue;
begin
  Assert(Node.Assigned);
end;

constructor TTypedNodeTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAsBoolean);
  AddTest(TestAsDateTime);
  AddTest(TestAsFloat);
  AddTest(TestAsInteger);
  AddTest(TestAssign);
  AddTest(TestAssigned);
  AddTest(TestAsString);
  AddTest(TestAsVariant);
  AddTest(TestLoadData);
end;

procedure TTypedNodeTest.DataLoaded(Sender: TJvxNode; DataField: TJvxDataField;
  var Done: Boolean);
begin
  FLoadedNode := Sender;
end;

procedure TTypedNodeTest.DocumentNotification(Sender: TJvxNode;
  Operation: TJvxNodeOperation);
begin
  FNotificationOperation := Operation;
  FNotificationSender := Sender;
end;

procedure TTypedNodeTest.InternalCreateFixture;
begin
  inherited;
  FNode := FNodeClass.Create(nil);
  TTestNode(FNode).OnLoadData := DataLoaded;
  FNodeField := TTestNodeField.Create(nil);
  TTestNode(FNode).DataField := FNodeField;
  FNode.OnNotifyEvents.Add(DocumentNotification);
end;

procedure TTypedNodeTest.InternalDestroyFixture;
begin
  FreeAndNil(FNode);
  FreeAndNil(FNodeField);
  inherited;
end;

procedure TTypedNodeTest.LoadData(Sender: TJvxNode;
  DataField: TJvxDataField; var Done: Boolean);
begin
  Assert(FNode = Sender);
  Assert(FNodeField = DataField);
  Assert(Done = False);
  FLoadedNode := Sender;
  Done := True;
end;

procedure TTypedNodeTest.TestAsBoolean;
begin
  AssignNodeValue;

  try
    Node.AsBoolean;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsBoolean := True;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsBoolean(True);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAsDateTime;
begin
  AssignNodeValue;

  try
    Node.AsDateTime;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsDateTime := Now;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsDateTime(Now);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAsFloat;
begin
  AssignNodeValue;

  try
    Node.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsFloat := 0;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsFloat(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAsInteger;
begin
  AssignNodeValue;

  try
    Node.AsInteger;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsInteger := 0;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsInteger(0);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAssign;
var
  NewNode: TJvxNode;
begin
  NewNode := TJvxNodeClass(Node.ClassType).Create(nil);
  try
    Assert(NewNode.Compare(Node));
    AssignNodeValue;
    Assert(not NewNode.Compare(Node));
    NewNode.Assign(Node);
    Assert(NewNode.Compare(Node));
  finally
    NewNode.Free;
  end;
end;

procedure TTypedNodeTest.TestAssigned;
begin
  // Test that by default all nodes are not assigned
  Assert(Node.Assigned = False);

  // Test that by default assigning a node causes an exception
  try
    Node.Assigned := True;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAsString;
begin
  AssignNodeValue;

  try
    Node.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsString := EmptyStr;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsString(EmptyStr);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestAsVariant;
begin
  AssignNodeValue;

  try
    Node.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    Node.AsVariant := Null;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  try
    TTestNode(Node).InternalSetAsVariant(Null);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TTypedNodeTest.TestLoadData;
begin
  Assert(FLoadedNode = Node);

  // Test that assigning OnLoadData handler makes node reload it's data
  FLoadedNode := nil;
  TTestNode(FNode).OnLoadData := LoadData;
  Assert(FLoadedNode = Node);

  // Test that if OnLoadEvent handler sets Done to True, then default action is cancelled
  FNode.Assigned := False;
  Assert(FNodeField.Assigned);
  TTestNode(FNode).LoadData(FNodeField);
  Assert(not FNode.Assigned);
end;

{ TBooleanNodeTest }

procedure TBooleanNodeTest.AssignNodeValue;
begin
  Node.AsBoolean := True;
  inherited;
end;

procedure TBooleanNodeTest.TestAsBoolean;
begin
  Assert(Node.Assigned = False);
  Node.AsBoolean := True;
  Assert(Node.AsBoolean = True);
end;

procedure TBooleanNodeTest.TestAsInteger;
begin
  Assert(Node.Assigned = False);
  Node.AsInteger := Ord(True);
  Assert(Node.AsBoolean = True);
  Node.AsInteger := Ord(False);
  Assert(Node.AsBoolean = False);
  
  try
    Node.AsInteger := Ord(True) + 1;
    Assert(ERangeError);
  except
    Assert(ERangeError);
  end;

  try
    Node.AsInteger := Ord(False) - 1;
    Assert(ERangeError);
  except
    Assert(ERangeError);
  end;
end;

procedure TBooleanNodeTest.TestAssigned;
begin
  inherited;
  Assert(not Node.Assigned);
  Node.AsBoolean := False;
  Assert(Node.Assigned = True);
end;

procedure TBooleanNodeTest.TestAsString;
begin
  Assert(Node.Assigned = False);
  Node.AsString := 'True';
  Assert(Node.AsBoolean = True);
  Node.AsBoolean := False;
  Assert(Node.AsString = 'False');
end;

procedure TBooleanNodeTest.TestAsVariant;
begin
  Assert(Node.Assigned = False);
  Node.AsVariant := True;
  Assert(Node.AsBoolean = True);
  Assert(Node.AsVariant = True);
  Assert(VarType(Node.AsVariant) = varBoolean);
end;

procedure TBooleanNodeTest.TestLoadData;
begin
  Assert(Node.Assigned = False);
  DataField.Value := True;
  Assert(Node.AsBoolean = True);
  inherited;
end;

{ TDateTimeNodeTest }

procedure TDateTimeNodeTest.AssignNodeValue;
begin
  Node.AsDateTime := Date;
  inherited;
end;

procedure TDateTimeNodeTest.TestAsDateTime;
begin
  Assert(Node.Assigned = False);
  Node.AsDateTime := Date;
  Assert(Node.AsDateTime = Date);
end;

procedure TDateTimeNodeTest.TestAssigned;
begin
  inherited;
  Assert(not Node.Assigned);
  Node.AsDateTime := 0;
  Assert(Node.Assigned = True);
end;

procedure TDateTimeNodeTest.TestAsString;
const
  TestFormat = 'yyyymmddhhnnsszzz';
  FormatProperty = 'Format';
begin
  Assert(Node.Assigned = False);
  Node.AsString := DateTimeToStr(Date);
  Assert(Node.AsDateTime = Date);

  // Test that by default Format is empty
  Assert(GetStrProp(Node, FormatProperty) = EmptyStr);

  // Test that default Format is the same as format used by DateTimeToStr
  Node.AsDateTime := Date;
  Assert(Node.AsString = DateTimeToStr(Date));

  // Test that changing Format sends notification
  FNotificationSender := nil;
  SetStrProp(Node, FormatProperty, TestFormat);
  Assert(FNotificationSender = Node);
  Assert(FNotificationOperation = noChange);

  // Test that Format property is used to format datetime as string
  Node.AsDateTime := Date;
  Assert(Node.AsString = FormatDateTime(TestFormat, Date));
end;

procedure TDateTimeNodeTest.TestAsVariant;
begin
  Assert(Node.Assigned = False);
  Node.AsVariant := Date;
  Assert(Node.AsDateTime = Date);
  Assert(Node.AsVariant = Date);
  Assert(VarType(Node.AsVariant) = varDate);
end;

procedure TDateTimeNodeTest.TestLoadData;
begin
  Assert(Node.Assigned = False);
  DataField.Value := Date;
  Assert(Node.AsDateTime = Date);
  inherited;
end;

{ TFloatNodeTest }

procedure TFloatNodeTest.AssignNodeValue;
begin
  Node.AsFloat := 3.14;
  inherited;
end;

procedure TFloatNodeTest.TestAsFloat;
begin
  Assert(Node.Assigned = False);
  Node.AsFloat := 3.14;
  Assert(Node.AsFloat = 3.14);
end;

procedure TFloatNodeTest.TestAsInteger;
begin
  Assert(Node.Assigned = False);
  Node.AsInteger := 284;
  Assert(Node.AsFloat = 284);
  Node.AsFloat := 285;
  Assert(Node.AsFloat = 285);
  Node.AsFloat := 3.14;
  try
    Node.AsInteger;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
end;

procedure TFloatNodeTest.TestAssigned;
begin
  inherited;
  Assert(Node.Assigned = False);
  Node.AsFloat := 0;
  Assert(Node.Assigned = True);
end;

procedure TFloatNodeTest.TestAsString;
const
  TestFormat = '"test"000000.00';
  FormatProperty = 'Format';
begin
  Assert(Node.Assigned = False);
  Node.AsString := FloatToStr(2.5);
  Assert(Node.AsFloat = 2.5);

  // Test that by default Format is empty
  Assert(GetStrProp(Node, FormatProperty) = EmptyStr);

  // Test that default Format is the same as format used by FloatToStr
  Node.AsFloat := Pi;
  Assert(Node.AsString = FloatToStr(Pi));

  // Test that changing Format sends notification
  FNotificationSender := nil;
  SetStrProp(Node, FormatProperty, TestFormat);
  Assert(FNotificationSender = Node);
  Assert(FNotificationOperation = noChange);

  // Test that Format property is used to format datetime as string
  Node.AsFloat := Pi;
  Assert(Node.AsString = FormatFloat(TestFormat, Pi));
end;

procedure TFloatNodeTest.TestAsVariant;
begin
  Assert(Node.Assigned = False);
  Node.AsVariant := 3.14;
  Assert(Node.AsFloat = 3.14);
  Assert((Node.AsVariant - 3.14) < 1e-6);
end;

procedure TFloatNodeTest.TestLoadData;
begin
  Assert(Node.Assigned = False);
  DataField.Value := 3.14;
  Assert(Node.AsFloat = 3.14);
  inherited;
end;

{ TIntegerNodeTest }

procedure TIntegerNodeTest.AssignNodeValue;
begin
  Node.AsInteger := 284;
  inherited;
end;

procedure TIntegerNodeTest.TestAsFloat;
begin
  Assert(Node.Assigned = False);
  Node.AsFloat := 284;
  Assert(Node.AsInteger = 284);
  Node.AsInteger := 285;
  Assert(Node.AsFloat = 285);
  try
    Node.AsFloat := 5.5;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
end;

procedure TIntegerNodeTest.TestAsInteger;
begin
  Assert(Node.Assigned = False);
  Node.AsInteger := 284;
  Assert(Node.AsInteger = 284);
end;

procedure TIntegerNodeTest.TestAssigned;
begin
  inherited;
  Assert(Node.Assigned = False);
  Node.AsInteger := 0;
  Assert(Node.Assigned = True);
end;

procedure TIntegerNodeTest.TestAsString;
begin
  Assert(Node.Assigned = False);
  Node.AsString := IntToStr(284);
  Assert(Node.AsInteger = 284);
  Node.AsInteger := 285;
  Assert(Node.AsString = IntToStr(285));
end;

procedure TIntegerNodeTest.TestAsVariant;
begin
  Assert(Node.Assigned = False);
  Node.AsVariant := 284;
  Assert(Node.AsInteger = 284);
  Assert(Node.AsVariant = 284);
end;

procedure TIntegerNodeTest.TestLoadData;
begin
  Assert(Node.Assigned = False);
  DataField.Value := 284;
  Assert(Node.AsInteger = 284);
  inherited;
end;

{ TStringNodeTest }

procedure TStringNodeTest.AssignNodeValue;
begin
  Node.AsString := 'test';
  inherited;
end;

procedure TStringNodeTest.TestAsBoolean;
begin
  Node.AsBoolean := False;
  Assert(Node.AsString = 'False');
  Node.AsString := 'Test';
  try
    Node.AsBoolean;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
  // Test that SetAsBoolean succedes even if GetAsBoolean raises EConvertError
  Node.AsBoolean := True;
  Assert(Node.AsString = 'True');
end;

procedure TStringNodeTest.TestAsDateTime;
begin
  Node.AsString := 'Test';
  try
    Node.AsDateTime;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
  // Test that SetAsDateTime succedes even if GetAsDateTime raises EConvertError
  Node.AsDateTime := Date;
  Assert(Node.AsString = DateToStr(Date));
end;

procedure TStringNodeTest.TestAsFloat;
begin
  Node.AsString := 'Test';
  try
    Node.AsFloat;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
  // Test that SetAsFloat succedes even if GetAsFloat raises EConvertError
  Node.AsFloat := 3.14;
  Assert(Node.AsString = FloatToStr(3.14));
end;

procedure TStringNodeTest.TestAsInteger;
begin
  Node.AsString := 'Test';
  try
    Node.AsInteger;
    Assert(EConvertError);
  except
    Assert(EConvertError);
  end;
  // Test that SetAsFloat succedes even if GetAsFloat raises EConvertError
  Node.AsInteger := 284;
  Assert(Node.AsString = IntToStr(284));
end;

procedure TStringNodeTest.TestAssigned;
begin
  inherited;
  Assert(Node.Assigned = False);
  Node.AsString := 'Test';
  Assert(Node.Assigned = True);
  Node.AsString := EmptyStr;
  Assert(Node.Assigned = False);
end;

procedure TStringNodeTest.TestAsString;
begin
  Node.AsString := 'True';
  Assert(Node.AsBoolean = True);
  Node.AsString := DateToStr(Date);
  Assert(Node.AsDateTime = Date);
  Node.AsString := '3.14';
  Assert(Node.AsFloat = 3.14);
  Node.AsString := '284';
  Assert(Node.AsInteger = 284);
  Assert(Node.AsVariant = '284');
end;

procedure TStringNodeTest.TestAsVariant;
begin
  Node.AsVariant := True;
  Assert(Node.AsBoolean = True);
  Assert(Node.AsVariant = 'True');
  Node.AsVariant := VarFromDateTime(Date);
  Assert(Node.AsDateTime = Date);
  Assert(Node.AsVariant = DateTimeToStr(Date));
  Node.AsVariant := 3.14;
  Assert(Node.AsFloat = 3.14);
  Assert(Node.AsVariant = 3.14);
  Node.AsVariant := 284;
  Assert(Node.AsInteger = 284);
  Assert(Node.AsVariant = 284);
  Node.AsVariant := 'Test';
  Assert(Node.AsString = 'Test');
  Assert(Node.AsVariant = 'Test');
end;

procedure TStringNodeTest.TestLoadData;
begin
  DataField.Value := 'Test';
  Assert(Node.AsString = 'Test');
  inherited;
end;

{ TGenericNodesTestSuite }

constructor TGenericNodesTestSuite.Create(Owner: TComponent);
var
  Test: TTypedNodeTest;
begin
  inherited;
  Test := TBooleanNodeTest.Create(nil);
  Test.NodeClass := TJvxBooleanAttribute;
  AddTest(Test);

  Test := TBooleanNodeTest.Create(nil);
  Test.NodeClass := TJvxBooleanElement;
  AddTest(Test);

  Test := TDateTimeNodeTest.Create(nil);
  Test.NodeClass := TJvxDateTimeAttribute;
  AddTest(Test);

  Test := TDateTimeNodeTest.Create(nil);
  Test.NodeClass := TJvxDateTimeElement;
  AddTest(Test);

  Test := TFloatNodeTest.Create(nil);
  Test.NodeClass := TJvxFloatAttribute;
  AddTest(Test);

  Test := TFloatNodeTest.Create(nil);
  Test.NodeClass := TJvxFloatElement;
  AddTest(Test);

  Test := TIntegerNodeTest.Create(nil);
  Test.NodeClass := TJvxIntegerAttribute;
  AddTest(Test);

  Test := TIntegerNodeTest.Create(nil);
  Test.NodeClass := TJvxIntegerElement;
  AddTest(Test);

  Test := TStringNodeTest.Create(nil);
  Test.NodeClass := TJvxStringAttribute;
  AddTest(Test);

  Test := TStringNodeTest.Create(nil);
  Test.NodeClass := TJvxStringElement;
  AddTest(Test);

  Test := TStringNodeTest.Create(nil);
  Test.NodeClass := TJvxCompositeTextElement;
  AddTest(Test);
end;

{ TTestNodeList }

constructor TTestNodeList.Create(Owner: TComponent);
begin
  inherited;
  FListDataSource := TJvxListDataSource.Create(Self);
  OnGetDataSource := GetDataSource;
end;

function TTestNodeList.GetDataSource: TJvxDataSource;
begin
  Result := FListDataSource;
end;

{ TTestNodes }

constructor TTestNodes.Create(Owner: TComponent);
begin
  inherited;
  FDataSource := TJvxDataSource.Create(Self);
  OnGetDataSource := GetDataSource;
end;

function TTestNodes.GetDataSource: TJvxDataSource;
begin
  Result := FDataSource;
end;

{ TTestDataSource }

procedure TTestDataSource.CreateDefaultFields;
begin
  inherited;
  FField1 := TTestNodeField.Create(Self);
  FField1.FieldName := 'Field1';
  Fields.Add(FField1);

  FField2 := TTestNodeField.Create(Self);
  FField2.FieldName := 'Field2';
  Fields.Add(FField2);

  FField3 := TTestNodeField.Create(Self);
  FField3.FieldName := 'Field3';
  Fields.Add(FField3);
end;

procedure TTestDataSource.InternalOpen;
begin
end;

procedure TTestDataSource.Notification(Component: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if Component = FField1 then
      FField1 := nil
    else if Component = FField2 then
      FField2 := nil
    else if Component = FField3 then
      FField3 := nil;
  end;
end;

procedure TTestDataSource.ValidateField(Field: TJvxDataField);
begin
  FValidatedField := Field;
end;

{ TDataSourceTest }

constructor TDataSourceTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestActive);
  AddTest(TestCount);
  AddTest(TestDefaultFields);
  AddTest(TestDelete);
  AddTest(TestGetNode);
  AddTest(TestIndexOf);
  AddTest(TestInsert);
end;

procedure TDataSourceTest.InternalCreateFixture;
begin
  inherited;
{$WARNINGS OFF} // Constructing instance of a class containing abstract methods
  FNode := TTestNode.Create(nil);
{$WARNINGS ON}
  FNodeField := TTestNodeField.Create(nil);
  FDataSource := TTestDataSource.Create(nil);
end;

procedure TDataSourceTest.InternalDestroyFixture;
begin
  FNode.Free;
  FNodeField.Free;
  FDataSource.Free;
  inherited;
end;

procedure TDataSourceTest.TestActive;
begin
  Assert(FDataSource.Active = False);
  Assert(FDataSource.Fields.Count = 0);

  // Test that Open activates dataset and creates default fields
  FDataSource.Open;
  Assert(FDataSource.Active = True);
  Assert(FDataSource.Fields.Count = 3);

  // Test that calling Open on active dataset doesn't do anything
  FDataSource.Open;
  Assert(FDataSource.Fields.Count = 3);

  // Test that Close deactivates dataset and destroys default fields
  FDataSource.Close;
  Assert(FDataSource.Active = False);
  Assert(FDataSource.Fields.Count = 0);

  // Test that Active has the same effect as Open
  FDataSource.Active := True;
  Assert(FDataSource.Active = True);
  Assert(FDataSource.Fields.Count = 3);

  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);

  FDataSource.Active := False;
  Assert(FDataSource.Active = False);
  Assert(FDataSource.Fields.Count = 0);
end;

procedure TDataSourceTest.TestCount;
begin
  // Test that count returns 0 when node set is inactive
  Assert(FDataSource.Count = 0);
  // Test that count returns 0 when node set is active
  FDataSource.Active := True;
  Assert(FDataSource.Count = 0);
end;

procedure TDataSourceTest.TestDefaultFields;
begin
  // Test that default fields are created when node set is activated
  Assert(FDataSource.Fields.Count = 0);
  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 3);
  Assert(FDataSource.FField1 <> nil);
  FDataSource.Active := False;
  Assert(FDataSource.Fields.Count = 0);
  Assert(FDataSource.FField1 = nil);

  // Test that default fields are not created if there are custom fields
  FNodeField.FieldName := 'test';
  FDataSource.Fields.Add(FNodeField);
  Assert(FDataSource.Fields.Count = 1);
  FDataSource.Active := True;
  Assert(FDataSource.Fields.Count = 1);
  FDataSource.Active := False;
  Assert(FDataSource.Fields.Count = 1);
end;

procedure TDataSourceTest.TestDelete;
begin
  // Test deleting when node set is inactive
  Assert(FDataSource.Active = False);
  try
    FDataSource.Delete(0);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test deleting when node set is active
  FDataSource.Active := True;
  try
    FDataSource.Delete(0);
    Assert(EListError);
  except
    Assert(EListError);
  end;
end;

procedure TDataSourceTest.TestGetNode;
begin
  // Test GetNode when node set is inactive
  Assert(FDataSource.Active = False);
  try
    FDataSource.GetNode(0, nil);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test GetNode when node set is active
  FDataSource.Active := True;
  try
    FDataSource.GetNode(0, nil);
    Assert(EListError);
  except
    Assert(EListError);
  end;
end;

procedure TDataSourceTest.TestIndexOf;
begin
  // Test IndexOf for inactive node sets
  Assert(FDataSource.Active = False);
  Assert(FDataSource.IndexOf(FNode) < 0);

  // Test IndexOf for active node sets
  FDataSource.Active := True;
  Assert(FDataSource.IndexOf(FNode) < 0);
end;

procedure TDataSourceTest.TestInsert;
begin
  // Test inserting into inactive node set
  Assert(FDataSource.Active = False);
  try
    FDataSource.Insert(0, FNode);
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test inserting into active node set
  FDataSource.Active := True;
  try
    FDataSource.Insert(0, FNode);
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

initialization
  JvxRegisterTestClass(TDocumentNodeTest);
  JvxRegisterTestClass(TDocumentNodesTest);
  JvxRegisterTestClass(TDocumentNodeListTest);
  JvxRegisterTestClass(TDocumentElementTest);
  JvxRegisterTestClass(TCompositeDocumentElementTest);
  JvxRegisterTestClass(TDocumentAttributeTest);
  JvxRegisterTestClass(TDocumentNotifyEventsTest);
  JvxRegisterTestClass(TNodeFieldTest);
  JvxRegisterTestClass(TNodeFieldsTest);
  JvxRegisterTestClass(TDataSourceTest);
  JvxRegisterTestClass(TGenericNodesTestSuite);

  RegisterClasses([TTestAttribute, TTestElement, TTestComposite,
    TJvxProtectedStringAttribute, TJvxProtectedStringElement]);
end.
