
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxDataSourcesTest.pas.                                }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxDataSourcesTest;

interface

uses
  Classes, DB, DBClient, JvxTestClasses, JvxClasses, JvxStdNodes, JvxUtils,
  JvxDataSources;

type

  { TDBNodeFieldTest }

  TDBNodeFieldTest = class(TJvxTestSuite)
  private
    FDataSource: TDataSource;
    FDataSet: TClientDataSet;
    FNodeDataField: TJvxDBDataField;
    FNotificationSender: TObject;
    procedure DataSourceChanged(Sender: TObject);
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAsBoolean;
    procedure TestAsDateTime;
    procedure TestAsFloat;
    procedure TestAsInteger;
    procedure TestAssigned;
    procedure TestAsString;
    procedure TestAsVariant;
    procedure TestDataField;
    procedure TestDataSource;
    procedure TestOnChange;
  end;

  { TDBNodeSetTest }

  TDBNodeSetTest = class(TJvxTestSuite)
  private
    FAddressField: TJvxDBDataField;
    FCompanyField: TJvxDBDataField;
    FComposite: TJvxCompositeTextElement;
    FDataSet: TClientDataSet;
    FDataSource: TDataSource;
    FNodeSet: TJvxDBDataSource;
    FRecordTemplate: TJvxCompositeTextElement;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Onwer: TComponent); override;
  published
    procedure TestCount;
    procedure TestDataSet;
    procedure TestDataSource;
    procedure TestDefaultFields;
    procedure TestGetNode;
  end;

  { TTestObject }

  TTestObject = class(TPersistent)
  private
    FBooleanProp: Boolean;
    FCharProp: Char;
    FFloatProp: Extended;
    FIntegerProp: Integer;
    FStringProp: string;
    FDateTimeProp: tdatetime;
    FVariantProp: Variant;
    FOrdinalProp: LongWord;
  public
    constructor Create;
  published
    // Don't change the case of property types
    property CharProp: char read FCharProp write FCharProp default #10;
    property BooleanProp: boolean read FBooleanProp write FBooleanProp default True;
    property DateTimeProp: tdatetime read FDateTimeProp write FDateTimeProp;
    property IntegerProp: integer read FIntegerProp write FIntegerProp default 5;
    property FloatProp: extended read FFloatProp write FFloatProp;
    property OrdinalProp: LongWord read FOrdinalProp write FOrdinalProp;
    property StringProp: String read FStringProp write FStringProp;
    property VariantProp: variant read FVariantProp write FVariantProp;
  end;

  { TPropertyNodeFieldTest }

  TPropertyNodeFieldTest = class(TJvxTestSuite)
  private
    FNodeField: TJvxPropertyDataField;
    FNotificationSender: TObject;
    FTestObject: TTestObject;
    procedure OnFieldChange(Sender: TObject);
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Onwer: TComponent); override;
  published
    procedure TestAsFloat;
    procedure TestAsOrdinal;
    procedure TestAssigned;
    procedure TestAsString;
    procedure TestAsVariant;
    procedure TestInstance;
    procedure TestPropertyName;
  end;

  { TFieldsNodeSetTest }

  TFieldsNodeSetTest = class(TJvxTestSuite)
  private
    FDataSet: TClientDataSet;
    FNodeSet: TJvxFieldsDataSource;
    FTemplate: TJvxCompositeTextElement;
    FAttribute: TJvxStringAttribute;
    FComposite: TJvxCompositeTextElement;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  public
    constructor Create(Onwer: TComponent); override;
  published
    procedure TestCount;
    procedure TestDataSet;
    procedure TestDefaultFields;
    procedure TestGetNode;
  end;

implementation

uses
  SysUtils;

{ TDBNodeFieldTest }

constructor TDBNodeFieldTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAsBoolean);
  AddTest(TestAsDateTime);
  AddTest(TestAsFloat);
  AddTest(TestAsInteger);
  AddTest(TestAssigned);
  AddTest(TestAsString);
// todo:  AddTest(TestAsVariant);
  AddTest(TestDataField);
  AddTest(TestDataSource);
  AddTest(TestOnChange);
end;

procedure TDBNodeFieldTest.DataSourceChanged(Sender: TObject);
begin
  FNotificationSender := Sender;
end;

procedure TDBNodeFieldTest.InternalCreateFixture;

  procedure InitializeDataSet;
  begin
    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Boolean';
      DataType := ftBoolean;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'DateTime';
      DataType := ftDateTime;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Float';
      DataType := ftFloat;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Integer';
      DataType := ftInteger;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'String';
      DataType := ftString;
      Size := 20;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Variant';
      DataType := ftString;
      Size := 20;
    end;

    FDataSet.CreateDataSet;
  end;

begin
  FDataSet := TClientDataSet.Create(nil);
  InitializeDataSet;

  FDataSource := TDataSource.Create(nil);
  FDataSource.DataSet := FDataSet;

  FNodeDataField := TJvxDBDataField.Create(nil);
  FNodeDataField.DataSource := FDataSource;
  FNodeDataField.OnChange := DataSourceChanged;
end;

procedure TDBNodeFieldTest.InternalDestroyFixture;
begin
  FreeAndNil(FDataSet);
  FreeAndNil(FDataSource);
  FreeAndNil(FNodeDataField);
  FNotificationSender := nil;
end;

procedure TDBNodeFieldTest.TestAsBoolean;
begin
  // Test with disconnected data source
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'Boolean';

  FNodeDataField.Field.AsBoolean := False;
  Assert(Boolean(FNodeDataField.AsOrdinal) = FNodeDataField.Field.AsBoolean);
  FNodeDataField.Field.AsBoolean := True;
  Assert(Boolean(FNodeDataField.AsOrdinal) = FNodeDataField.Field.AsBoolean);
end;

procedure TDBNodeFieldTest.TestAsDateTime;
begin
  // Test with disconnected data source
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'DateTime';

  FNodeDataField.Field.AsDateTime := Now;
  Assert(FNodeDataField.AsFloat = FNodeDataField.Field.AsDateTime);
end;

procedure TDBNodeFieldTest.TestAsFloat;
begin
  // Test with disconnected data source
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'Float';

  FNodeDataField.Field.AsFloat := Now;
  Assert(FNodeDataField.AsFloat = FNodeDataField.Field.AsFloat);
end;

procedure TDBNodeFieldTest.TestAsInteger;
begin
  // Test that disconnected data source has correct default value
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'Integer';

  FNodeDataField.Field.AsInteger := 314;
  Assert(FNodeDataField.AsOrdinal = FNodeDataField.Field.AsInteger);
end;

procedure TDBNodeFieldTest.TestAssigned;
begin
  // Test that Assigned is False if field is null
  FNodeDataField.DataField := 'Boolean';
  Assert(FNodeDataField.Field.IsNull);
  Assert(not FNodeDataField.Assigned);

  // Test that Assigned is False if data field is empty
  FDataSet.Edit;
  FNodeDataField.Field.AsBoolean := True;
  Assert(not FNodeDataField.Field.IsNull);
  FNodeDataField.DataField := EmptyStr;
  Assert(not FNodeDataField.Assigned);

  // Test that Assigned is True if data field is assigned and is not null
  FNodeDataField.DataField := 'Boolean';
  Assert(FNodeDataField.Assigned);
end;

procedure TDBNodeFieldTest.TestAsString;
begin
  // Test that disconnected data source has correct default value
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'String';

  FNodeDataField.Field.AsString := DateTimeToStr(Now);
  Assert(FNodeDataField.AsString = FNodeDataField.Field.AsString);
end;

procedure TDBNodeFieldTest.TestAsVariant;
begin
  // Test that disconnected data source has correct default value
  Assert(not FNodeDataField.Assigned);

  // Test that changing underlying dataset updates data source
  FDataSet.Edit;
  FNodeDataField.DataField := 'Variant';

  FNodeDataField.Field.AsVariant := 'Test';
  Assert(FNodeDataField.AsVariant = FNodeDataField.Field.AsVariant);
end;

procedure TDBNodeFieldTest.TestDataField;
begin
  // Test that assigning DataField affects Field property
  Assert(FNodeDataField.Field = nil);
  FNodeDataField.DataField := 'Boolean';
  Assert(FNodeDataField.DataField = 'Boolean');
  Assert(FNodeDataField.Field = FDataSet.FieldByName('Boolean'));
  FNodeDataField.DataField := EmptyStr;
  Assert(FNodeDataField.Field = nil);
end;

procedure TDBNodeFieldTest.TestDataSource;
begin
  // Test that when data source is freed - the property is cleared
  Assert(FNodeDataField.DataSource = FDataSource);
  FreeAndNil(FDataSource);
  Assert(FNodeDataField.DataSource = nil);
end;

procedure TDBNodeFieldTest.TestOnChange;
begin
  // Prepare test
  FNodeDataField.DataField := 'Boolean';
  FNodeDataField.DataSource := nil;

  FDataSet.Insert;
  FDataSet.FieldByName('Boolean').AsBoolean := True;
  FDataSet.Post;
  FDataSet.Append;
  FDataSet.Post;
  Assert(FDataSet.RecordCount = 2);

  // Test that notification fired when data set is activated
  FNotificationSender := nil;
  FNodeDataField.DataSource := FDataSource;
  Assert(FNotificationSender = FNodeDataField);
  Assert(not FNodeDataField.Assigned);

  // Test that notification is fired when data set is scrolled
  FNotificationSender := nil;
  FDataSet.Prior;
  Assert(FNotificationSender = FNodeDataField);
  Assert(FNodeDataField.Assigned);
  Assert(FNodeDataField.AsOrdinal = Ord(True));

  // Test that notification is fired when data field is changed
  FNotificationSender := nil;
  FNodeDataField.DataField := 'Integer';
  Assert(FNotificationSender = FNodeDataField);

  // Test that notification is fired when field value is changed
  FDataSet.Edit;
  FNotificationSender := nil;
  FNodeDataField.Field.AsInteger := 314;
  Assert(FNotificationSender = FNodeDataField);

  // Test that notification is fired whend data set is deactivated
  FNotificationSender := nil;
  FNodeDataField.DataSource := nil;
  Assert(FNotificationSender = FNodeDataField);
  Assert(not FNodeDataField.Assigned);
end;

{ TDBNodeSetTest }

constructor TDBNodeSetTest.Create(Onwer: TComponent);
begin
  inherited;
  AddTest(TestCount);
  AddTest(TestDataSet);
  AddTest(TestDataSource);
  AddTest(TestDefaultFields);
  AddTest(TestGetNode);
end;

procedure TDBNodeSetTest.InternalCreateFixture;

  procedure InitializeDataSet;
  var
    I: Integer;
  begin
    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Company';
      DataType := ftString;
      Size := 20;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Address';
      DataType := ftString;
      Size := 20;
    end;

    FDataSet.CreateDataSet;

    for I := 1 to 10 do
    begin
      FDataSet.Append;
      FDataSet.FieldByName('Company').AsString := 'Company' + IntToStr(I);
      FDataSet.FieldByName('Address').AsString := 'Address' + IntToStr(I);
      FDataSet.Post;
    end;
  end;

var
  Attribute: TJvxStringAttribute;
  Element: TJvxStringElement;
begin
  inherited;
  FDataSet := TClientDataSet.Create(nil);
  InitializeDataSet;

  FDataSource := TDataSource.Create(nil);
  FDataSource.DataSet := FDataSet;

  FAddressField := TJvxDBDataField.Create(nil);
  FAddressField.DataSource := FDataSource;
  FAddressField.DataField := 'Address';

  FCompanyField := TJvxDBDataField.Create(nil);
  FCompanyField.DataSource := FDataSource;
  FCompanyField.DataField := 'Company';

  FNodeSet := TJvxDBDataSource.Create(nil);
  FNodeSet.DataSource := FDataSource;
  FNodeSet.Active := True;

  FRecordTemplate := TJvxCompositeTextElement.Create(nil);
  FRecordTemplate.NodeName := 'row';

  Attribute := TJvxStringAttribute.Create(nil);
  Attribute.NodeName := 'address';
  Attribute.DataField := FAddressField;
  Attribute.Parent := FRecordTemplate;

  Element := TJvxStringElement.Create(nil);
  Element.NodeName := 'company';
  Element.DataField := FCompanyField;
  Element.Parent := FRecordTemplate;

  FComposite := TJvxCompositeTextElement.Create(nil);
  FComposite.NodeName := 'rows';
  FComposite.Template := FRecordTemplate;
  FComposite.DataSource := FNodeSet;
end;

procedure TDBNodeSetTest.InternalDestroyFixture;
begin
  FreeAndNil(FDataSource);
  FreeAndNil(FDataSet);
  FreeAndNil(FAddressField);
  FreeAndNil(FCompanyField);
  FreeAndNil(FComposite);
  FreeAndNil(FNodeSet);
  FreeAndNil(FRecordTemplate);
  inherited;
end;

procedure TDBNodeSetTest.TestCount;
begin
  Assert(FDataSet.RecordCount > 0);
  Assert(FComposite.Elements.Count = FDataSet.RecordCount);
end;

procedure TDBNodeSetTest.TestDataSet;
begin
  Assert(FNodeSet.DataSet = FDataSet);
  FNodeSet.DataSource := nil;
  Assert(FNodeSet.DataSet = nil);

  // Test that opening node set with data source but without data set raises an exception
  FNodeSet.Active := False;
  FNodeSet.DataSource := FDataSource;
  FDataSource.DataSet := nil;
  try
    FNodeSet.Open;
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Todo: test disconnecting data set when node set is active
end;

procedure TDBNodeSetTest.TestDataSource;
begin
  Assert(FNodeSet.DataSource = FDataSource);
  FNodeSet.DataSource := nil;
  Assert(FNodeSet.DataSource = nil);

  // Test that opening node set without data source raises an exception
  FNodeSet.Active := False;
  try
    FNodeSet.Open;
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Todo: test disconnecting data source when node set is active
end;

procedure TDBNodeSetTest.TestDefaultFields;
var
  I: Integer;
begin
  FNodeSet.Active := True; 
  Assert(FNodeSet.Fields.Count = FDataSet.Fields.Count);
  for I := 0 to FDataSet.Fields.Count - 1 do
  begin
    Assert(FNodeSet.Fields[I].FieldName = FDataSet.Fields[I].FieldName);
    Assert(FNodeSet.Fields[I].AsString = FDataSet.Fields[I].AsString);
  end;
end;

procedure TDBNodeSetTest.TestGetNode;
var
  I: Integer;
begin
  for I := 0 to 9 do
  begin
    Assert(FComposite.Elements[I].Attributes[0].NodeName = 'address');
    Assert(FComposite.Elements[I].Attributes[0].AsString = 'Address' + IntToStr(I + 1));
    Assert(FComposite.Elements[I].Elements[0].NodeName = 'company');
    Assert(FComposite.Elements[I].Elements[0].AsString = 'Company' + IntToStr(I + 1));
  end;
end;

{ TTestObject }

constructor TTestObject.Create;
begin
  inherited;
  FCharProp := #10;
  FBooleanProp := True;
  FIntegerProp := 5;
  FStringProp := 'test';
  FVariantProp := 'variant';
end;

{ TPropertyNodeFieldTest }

constructor TPropertyNodeFieldTest.Create(Onwer: TComponent);
begin
  inherited;
  AddTest(TestAsFloat);
  AddTest(TestAsOrdinal);
  AddTest(TestAssigned);
  AddTest(TestAsString);
  AddTest(TestAsVariant);
  AddTest(TestInstance);
  AddTest(TestPropertyName);
end;

procedure TPropertyNodeFieldTest.InternalCreateFixture;
begin
  inherited;
  FTestObject := TTestObject.Create;
  FNodeField := TJvxPropertyDataField.Create(nil);
  FNodeField.OnChange := OnFieldChange;
end;

procedure TPropertyNodeFieldTest.InternalDestroyFixture;
begin
  FNodeField.Free;
  FTestObject.Free;
  inherited;
end;

procedure TPropertyNodeFieldTest.OnFieldChange(Sender: TObject);
begin
  FNotificationSender := Sender;
end;

procedure TPropertyNodeFieldTest.TestAsFloat;
begin
  // Test that you can use AsFloat with Extended property
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'FloatProp';

  // Test that when unassigned (default) AsFloat doesn't raise an exception
  Assert(FTestObject.FloatProp = 0); // default value
  Assert(FNodeField.AsFloat = 0);

  FTestObject.FloatProp := 3.14;
  Assert(FNodeField.AsFloat = FTestObject.FloatProp);

  // Test with other floating type properties
  FNodeField.PropertyName := 'DateTimeProp';
  FTestObject.DateTimeProp := Date;
  Assert(FNodeField.AsFloat = FTestObject.DateTimeProp);

  // Test that any other property type raises an exception
  FNodeField.PropertyName := 'BooleanProp';
  try
    FNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'CharProp';
  try
    FNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'IntegerProp';
  try
    FNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'StringProp';
  try
    FNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'VariantProp';
  try
    FNodeField.AsFloat;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TPropertyNodeFieldTest.TestAsOrdinal;
begin
  // Test that you can use AsOrdinal with Integer property
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'IntegerProp';

  // Test that when unassigned (default) AsInteger doesn't raise an exception
  Assert(FTestObject.IntegerProp = 5); // default value
  Assert(FNodeField.AsOrdinal = 5);

  FTestObject.IntegerProp := 284;
  Assert(FNodeField.AsOrdinal = FTestObject.IntegerProp);

  // Test that you can use AsOrdinal with Boolean property
  FNodeField.PropertyName := 'BooleanProp';
  FTestObject.BooleanProp := False;
  Assert(FNodeField.AsOrdinal = Ord(FTestObject.BooleanProp));

  // Test that you can use AsOrdinal with Char property
  FNodeField.PropertyName := 'CharProp';
  FTestObject.CharProp := #13;
  Assert(FNodeField.AsOrdinal = Ord(FTestObject.CharProp));

  // Test that any other property type raises an exception
  FNodeField.PropertyName := 'DateTimeProp';
  try
    FNodeField.AsOrdinal;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'FloatProp';
  try
    FNodeField.AsOrdinal;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'StringProp';
  try
    FNodeField.AsOrdinal;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'VariantProp';
  try
    FNodeField.AsOrdinal;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TPropertyNodeFieldTest.TestAssigned;
begin
  Assert(not FNodeField.Assigned);
  FNodeField.Instance := FTestObject;

  // Test JvxOrdinal properties (Char, Boolean, Integer)
  Assert(FTestObject.CharProp = #10); // default value
  FNodeField.PropertyName := 'CharProp';
  Assert(FNodeField.Assigned);
  FTestObject.CharProp := 'a';
  Assert(FNodeField.Assigned);

  Assert(FTestObject.BooleanProp = True); // default value
  FNodeField.PropertyName := 'BooleanProp';
  Assert(FNodeField.Assigned);
  FTestObject.BooleanProp := False;
  Assert(FNodeField.Assigned);

  Assert(FTestObject.IntegerProp = 5); // default value
  FNodeField.PropertyName := 'IntegerProp';
  Assert(FNodeField.Assigned);
  FTestObject.IntegerProp := 6;
  Assert(FNodeField.Assigned);

  Assert(FTestObject.OrdinalProp = 0);
  FNodeField.PropertyName := 'OrdinalProp';
  Assert(FNodeField.Assigned);
  FTestObject.OrdinalProp := $80000000; // test the "magic" RTTI default value
  Assert(FNodeField.Assigned);

  // Test float properties
  Assert(FTestObject.DateTimeProp = 0);
  FNodeField.PropertyName := 'DateTimeProp';
  Assert(FNodeField.Assigned);
  FTestObject.DateTimeProp := 1;
  Assert(FNodeField.Assigned);

  Assert(FTestObject.FloatProp = 0);
  FNodeField.PropertyName := 'FloatProp';
  Assert(FNodeField.Assigned);
  FTestObject.FloatProp := 1;
  Assert(FNodeField.Assigned);

  // Test string property
  FTestObject.StringProp := EmptyStr;
  FNodeField.PropertyName := 'StringProp';
  Assert(not FNodeField.Assigned);
  FTestObject.StringProp := 'test';
  Assert(FNodeField.Assigned);

  // Test variant property
  FTestObject.VariantProp := Null;
  FNodeField.PropertyName := 'VariantProp';
  Assert(not FNodeField.Assigned);
  FTestObject.VariantProp := Null;
  Assert(not FNodeField.Assigned);
  FTestObject.VariantProp := 'test';
  Assert(FNodeField.Assigned);
end;

procedure TPropertyNodeFieldTest.TestAsString;
begin
  // Test that you can use AsString with string property
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'StringProp';

  // Test that when unassigned (default) AsString returns EmptyStr
  FTestObject.StringProp := EmptyStr; // default value
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FTestObject.StringProp := 'test';
  Assert(FNodeField.AsString = FTestObject.StringProp);

  // Test that any other property type raises an exception
  FNodeField.PropertyName := 'CharProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'BooleanProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'DateTimeProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'FloatProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'IntegerProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'VariantProp';
  try
    FNodeField.AsString;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TPropertyNodeFieldTest.TestAsVariant;
begin
  // Test that you can use AsVariant with Variant property
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'VariantProp';

  // Test that when unassigned (default) AsVariant returns Null
  FTestObject.VariantProp := Null; // default value
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FTestObject.VariantProp := 'test';
  Assert(FNodeField.AsVariant = FTestObject.VariantProp);

  // Test that any other property type raises an exception
  FNodeField.PropertyName := 'CharProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'BooleanProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'DateTimeProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'FloatProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'IntegerProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;

  FNodeField.PropertyName := 'StringProp';
  try
    FNodeField.AsVariant;
    Assert(EInvalidOperation);
  except
    Assert(EInvalidOperation);
  end;
end;

procedure TPropertyNodeFieldTest.TestInstance;
begin
  // Test that changing Instance fires notification
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'IntegerProp';
  FNotificationSender := nil;
  FNodeField.Instance := nil;
  Assert(FNotificationSender = FNodeField);
end;

procedure TPropertyNodeFieldTest.TestPropertyName;
begin
  // Test that changing PropertyName fires notification
  FNodeField.Instance := FTestObject;
  FNodeField.PropertyName := 'IntegerProp';
  FNotificationSender := nil;
  FNodeField.PropertyName := EmptyStr;
  Assert(FNotificationSender = FNodeField);

  // Test that using non-existant property name raises an exception
  try
    FNodeField.PropertyName := 'InvalidPropName';
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;
end;

{ TFieldsNodeSetTest }

constructor TFieldsNodeSetTest.Create(Onwer: TComponent);
begin
  inherited;
  AddTest(TestCount);
  AddTest(TestDataSet);
  AddTest(TestDefaultFields);
  AddTest(TestGetNode);
end;

procedure TFieldsNodeSetTest.InternalCreateFixture;

  procedure InitializeDataSet;
  begin
    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Company';
      DataType := ftString;
      Size := 20;
    end;

    with FDataSet.FieldDefs.AddFieldDef do
    begin
      Name := 'Address';
      DataType := ftString;
      Size := 20;
    end;

    FDataSet.CreateDataSet;
  end;

begin
  inherited;

  FDataSet := TClientDataSet.Create(nil);
  InitializeDataSet;

  FNodeSet := TJvxFieldsDataSource.Create(nil);
  FNodeSet.DataSet := FDataSet;
  FNodeSet.Active := True;

  FTemplate := TJvxCompositeTextElement.Create(nil);
  FTemplate.NodeName := 'FieldDef';

  FAttribute := TJvxStringAttribute.Create(nil);
  FAttribute.NodeName := 'FieldName';
  FAttribute.Parent := FTemplate;

  FComposite := TJvxCompositeTextElement.Create(nil);
  FComposite.NodeName := 'FieldDefs';
  FComposite.Template := FTemplate;
  FComposite.DataSource := FNodeSet;
end;

procedure TFieldsNodeSetTest.InternalDestroyFixture;
begin
  FAttribute.Free;
  FTemplate.Free;
  FComposite.Free;
  FNodeSet.Free;
  FDataSet.Free;
  inherited;
end;

procedure TFieldsNodeSetTest.TestCount;
begin
  Assert(FComposite.Elements.Count = FDataSet.Fields.Count);
end;

procedure TFieldsNodeSetTest.TestDataSet;
begin
  // Test that opening node set with data source but without data set raises an exception
  FNodeSet.Active := False;
  FNodeSet.DataSet := nil;
  try
    FNodeSet.Open;
    Assert(EJvxError);
  except
    Assert(EJvxError);
  end;

  // Test that if DataSet is destroyed, the reference is cleared
  FNodeSet.DataSet := FDataSet;
  Assert(FNodeSet.DataSet = FDataSet);
  FreeAndNil(FDataSet);
  Assert(FNodeSet.DataSet = nil); 

  // Todo: test disconnecting data set when node set is active
end;

procedure TFieldsNodeSetTest.TestDefaultFields;
begin
  Assert(FNodeSet.Fields.FindField('FieldName') <> nil);
  Assert(FNodeSet.Fields.FindField('DisplayWidth') <> nil);
  Assert(FNodeSet.Fields.FindField('DisplayLabel') <> nil);
end;

procedure TFieldsNodeSetTest.TestGetNode;
var
  I: Integer;
begin
  FAttribute.DataField := FNodeSet.Fields.FieldByName('FieldName');
  for I := 0 to FDataSet.Fields.Count - 1 do
  begin
    Assert(FComposite.Elements[I].NodeName = 'FieldDef');
    Assert(FComposite.Elements[I].Attributes[0].NodeName = 'FieldName');
    Assert(FDataSet.Fields[I].FieldName = FComposite.Elements[I].Attributes[0].AsString);
  end;
end;

initialization
  JvxRegisterTestClass(TDBNodeFieldTest);
  JvxRegisterTestClass(TDBNodeSetTest);
  JvxRegisterTestClass(TPropertyNodeFieldTest);
  JvxRegisterTestClass(TFieldsNodeSetTest);

end.
