
{******************************************************************************}
{                                                                              }
{  The contents of this file are subject to the Mozilla Public License         }
{  (the "License"); you may not use this file except in compliance with        }
{  the License. You may obtain a copy of the License at                        }
{  http://www.mozilla.org/MPL/                                                 }
{                                                                              }
{  Software distributed under the License is distributed on an "AS IS"         }
{  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See         }
{  the License for the specific language governing rights and                  }
{  limitations under the License.                                              }
{                                                                              }
{  The Original Code is JvxHTMLClassesTest.pas.                                }
{                                                                              }
{  The Initial Developer of the Original Code is Oleg Sych <osych@hotmail.com> }
{  Portions created by Oleg Sych are Copyright (C) 2000 Oleg Sych.             }
{  All Rights Reserved.                                                        }
{                                                                              }
{  You may retrieve the latest version of this file at XVCL home page, located }
{  at http://go.to/xvcl                                                        }
{                                                                              }
{  Contributor(s):                                                             }
{                                                                              }
{******************************************************************************}

unit JvxHTMLClassesTest;

interface

uses
  Classes, JvxTestClasses, JvxClasses, JvxStdNodes, JvxHTMLClasses;

type

  { TJvxCustomNodeTest }

  TJvxCustomNodeTest = class(TJvxTestSuite)
  protected
    procedure TestNode(Nodes: TJvxNodes; Node: TJvxNode; const NodeName: string;
      NodeClass: TJvxNodeClass);
    procedure TestValidChildNode(Nodes: TJvxNodes; NodeClass: TJvxNodeClass);
    procedure TestInvalidChildNode(Nodes: TJvxNodes; NodeClass: TJvxNodeClass);
  public
    constructor Create(Owner: TComponent); override;
  published
    procedure TestAttributes; virtual; abstract;
    procedure TestElements; virtual; abstract;
    procedure TestNodeName; virtual; abstract;
  end;

  { TJvxHTMLTest }

  TJvxHTMLTest = class(TJvxCustomNodeTest)
  protected
    procedure TestAlignAttribute(Align: TJvxHTMLAlignAttribute; ValidValues: TJvxHTMLAligns);
  end;

  { TJvxHTMLCustomElementTest }

  TJvxHTMLCustomElementTest = class(TJvxHTMLTest)
  private
    FElement: TJvxHTMLCustomElement;
  protected
    procedure InternalDestroyFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
  end;

  { TJvxHTMLCustomBodyElementTest }

  TJvxHTMLCustomBodyElementTest = class(TJvxHTMLCustomElementTest)
  published
    procedure TestAttributes; override;
  end;

  { TJvxHTMLBodyTest }

  TJvxHTMLBodyTest = class(TJvxHTMLCustomBodyElementTest)
  protected
    procedure InternalCreateFixture; override;
  published
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLTitleTest }

  TJvxHTMLTitleTest = class(TJvxHTMLTest)
  private
    FElement: TJvxHTMLTitle;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLHeadTest }

  TJvxHTMLHeadTest = class(TJvxHTMLTest)
  private
    FElement: TJvxHTMLHead;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLCustomRootElementTest }

  TJvxHTMLCustomRootElementTest = class(TJvxHTMLTest)
  private
    FElement: TJvxHTMLCustomRootElement;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLElementTest }

  TJvxHTMLElementTest = class(TJvxHTMLCustomRootElementTest)
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  published
    procedure TestElements; override;
  end;

  { TJvxHTMLLegendTest }

  TJvxHTMLLegendTest = class(TJvxHTMLCustomElementTest)
  protected
    procedure InternalCreateFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLFieldSetTest }

  TJvxHTMLFieldSetTest = class(TJvxHTMLCustomElementTest)
  protected
    procedure InternalCreateFixture; override;
  published
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLCustomFocusableElementTest }

  TJvxHTMLCustomFocusableElementTest = class(TJvxHTMLCustomElementTest)
  published
    procedure TestAttributes; override;
  end;

  { TJvxHTMLCustomFormElementTest }

  TJvxHTMLCustomFormElementTest = class(TJvxHTMLCustomFocusableElementTest);

  { TJvxHTMLLabelTest }

  TJvxHTMLLabelTest = class(TJvxHTMLCustomFormElementTest)
  protected
    procedure InternalCreateFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLSelectTest }

  TJvxHTMLSelectTest = class(TJvxHTMLTest)
  private
    FElement: TJvxHTMLSelect;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

  { TJvxHTMLImportInstructionTest }

  TJvxHTMLImportInstructionTest = class(TJvxCustomNodeTest)
  private
    FInstruction: TJvxHTMLImportInstruction;
  protected
    procedure InternalCreateFixture; override;
    procedure InternalDestroyFixture; override;
  published
    procedure TestAttributes; override;
    procedure TestElements; override;
    procedure TestNodeName; override;
  end;

implementation

uses
  JvxDebugMem;

{ TJvxCustomNodeTest }

constructor TJvxCustomNodeTest.Create(Owner: TComponent);
begin
  inherited;
  AddTest(TestAttributes);
  AddTest(TestElements);
  AddTest(TestNodeName);
end;

procedure TJvxCustomNodeTest.TestInvalidChildNode(Nodes: TJvxNodes;
  NodeClass: TJvxNodeClass);
var
  Node: TJvxNode;
begin
  Node := NodeClass.Create(nil);
  try
    try
      Nodes.Add(Node);
      Assert(EInvalidOperation);
    except
      Assert(EInvalidOperation);
    end;
  finally
    Node.Free;
  end;
end;

procedure TJvxCustomNodeTest.TestNode(Nodes: TJvxNodes;
  Node: TJvxNode; const NodeName: string; NodeClass: TJvxNodeClass);
begin
  Assert(JvxValidObject(Node, NodeClass));
  Assert(Node.NodeName = NodeName);
  Assert(Nodes.IndexOf(Node) >= 0);
end;

procedure TJvxCustomNodeTest.TestValidChildNode(Nodes: TJvxNodes;
  NodeClass: TJvxNodeClass);
var
  Node: TJvxNode;
begin
  Node := NodeClass.Create(nil);
  try
    Nodes.Add(Node);
  finally
    Node.Free;
  end;
end;

{ TJvxHTMLTest }

procedure TJvxHTMLTest.TestAlignAttribute(Align: TJvxHTMLAlignAttribute;
  ValidValues: TJvxHTMLAligns);
var
  I: TJvxHTMLAlign;
begin
  for I := Low(TJvxHTMLAlign) to High(TJvxHTMLAlign) do
    try
      Align.Value := I;
      Assert(I in ValidValues);
    except
      Assert(not (I in ValidValues));
      Assert(EJvxError);
    end;
end;

{ TJvxHTMLCustomElementTest }

type
  TJvxHTMLExposedCustomElement = class(TJvxHTMLCustomElement);

procedure TJvxHTMLCustomElementTest.InternalDestroyFixture;
begin
  FElement.Free;
  inherited;
end;

procedure TJvxHTMLCustomElementTest.TestAttributes;
begin
  Assert(FElement.HasAttributes);
  with TJvxHTMLExposedCustomElement(FElement) do
  begin
    TestNode(Attributes, HTMLClass, 'class', TJvxProtectedStringAttribute);
    TestNode(Attributes, ID, 'id', TJvxProtectedStringAttribute);
    TestNode(Attributes, OnClick, 'onclick', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnDblClick, 'ondblclick', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnKeyDown, 'onkeydown', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnKeyPress, 'onkeypress', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnKeyUp, 'onkeyup', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnMouseDown, 'onmousedown', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnMouseMove, 'onmousemove', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnMouseOut, 'onmouseout', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnMouseOver, 'onmouseover', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnMouseUp, 'onmouseup', TJvxHTMLScriptAttribute);
    TestNode(Attributes, Style, 'style', TJvxProtectedStringAttribute);
    TestNode(Attributes, Title, 'title', TJvxProtectedStringAttribute);
  end;
end;

procedure TJvxHTMLCustomElementTest.TestElements;
begin
  Assert(FElement.HasElements = False);
end;

{ TJvxHTMLCustomBodyElementTest }

type
  TJvxHTMLExposedCustomBodyElement = class(TJvxHTMLCustomBodyElement);

procedure TJvxHTMLCustomBodyElementTest.TestAttributes;
begin
  inherited;
  with TJvxHTMLExposedCustomBodyElement(FElement) do
  begin
    TestNode(Attributes, OnLoad, 'onload', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnUnload, 'onunload', TJvxHTMLScriptAttribute);
  end;
end;

{ TJvxHTMLBodyTest }

procedure TJvxHTMLBodyTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLBody.Create(nil);
end;

procedure TJvxHTMLBodyTest.TestElements;
begin
  inherited;
  // Test that script element can be a child of body element
  TestValidChildNode(FElement.Elements, TJvxHTMLScript);

  // Test that block element can be a child of body element
  TestValidChildNode(FElement.Elements, TJvxHTMLDiv);

  // Test that inline elemen cannot be a child of body element
  TestInvalidChildNode(FElement.Elements, TJvxHTMLSpan);
end;

procedure TJvxHTMLBodyTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'body');
end;

{ TJvxHTMLTitleTest }

procedure TJvxHTMLTitleTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLTitle.Create(nil);
end;

procedure TJvxHTMLTitleTest.InternalDestroyFixture;
begin
  FElement.Free;
  inherited;
end;

procedure TJvxHTMLTitleTest.TestAttributes;
begin
  Assert(FElement.HasAttributes = False);
end;

procedure TJvxHTMLTitleTest.TestElements;
begin
  Assert(FElement.HasElements = False);
end;

procedure TJvxHTMLTitleTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'title');
end;

{ TJvxHTMLHeadTest }

procedure TJvxHTMLHeadTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLHead.Create(nil);
end;

procedure TJvxHTMLHeadTest.InternalDestroyFixture;
begin
  FElement.Free;
  inherited;
end;

procedure TJvxHTMLHeadTest.TestAttributes;
begin
  Assert(FElement.HasAttributes);
  Assert(FElement.Attributes.Count = 1);

  // Test Profile attribute
  TestNode(FElement.Attributes, FElement.Profile, 'profile', TJvxHTMLURIAttribute);
end;

procedure TJvxHTMLHeadTest.TestElements;
begin
  Assert(FElement.HasElements);
  Assert(FElement.Elements.Count = 1);

  // Test title element
  TestNode(FElement.Elements, FElement.Title, 'title', TJvxHTMLTitle);
end;

procedure TJvxHTMLHeadTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'head');
end;

{ TJvxHTMLCustomRootElementTest }

type
  TJvxHTMLExposedCustomRootElement = class(TJvxHTMLCustomRootElement);

procedure TJvxHTMLCustomRootElementTest.TestAttributes;
begin
  Assert(FElement.HasAttributes = False);
end;

procedure TJvxHTMLCustomRootElementTest.TestElements;
begin
  with TJvxHTMLExposedCustomRootElement(FElement) do
  begin
    // Test Head element
    TestNode(Elements, Head, 'head', TJvxHTMLHead);
  end;
end;

procedure TJvxHTMLCustomRootElementTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'html');
end;

{ TJvxHTMLElementTest }

procedure TJvxHTMLElementTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLElement.Create(nil);
end;

procedure TJvxHTMLElementTest.InternalDestroyFixture;
begin
  FElement.Free;
  inherited;
end;

procedure TJvxHTMLElementTest.TestElements;
begin
  inherited;
  with TJvxHTMLElement(FElement) do
  begin
    // Test Body element
    TestNode(Elements, Body, 'body', TJvxHTMLBody);
  end;
end;

{ TJvxHTMLLegendTest }

procedure TJvxHTMLLegendTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLLegend.Create(nil);
end;

procedure TJvxHTMLLegendTest.TestAttributes;
begin
  inherited;
  with TJvxHTMLLegend(FElement) do
  begin
    TestNode(Attributes, AccessKey, 'accesskey', TJvxProtectedStringAttribute);
    TestNode(Attributes, Align, 'align', TJvxHTMLAlignAttribute);
    TestAlignAttribute(Align, [haTop, haBottom, haLeft, haRight]);
  end;
end;

procedure TJvxHTMLLegendTest.TestElements;
begin
  inherited;
  // Test that block content is not allowed within legend
  TestInvalidChildNode(FElement.Elements, TJvxHTMLDiv);
  // Test that inline content is allowed within legend
  TestValidChildNode(FElement.Elements, TJvxHTMLSpan);
end;

procedure TJvxHTMLLegendTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'legend');
end;

{ TJvxHTMLFieldSetTest }

procedure TJvxHTMLFieldSetTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLFieldSet.Create(nil);
end;

procedure TJvxHTMLFieldSetTest.TestElements;
begin
  // inherited; is not called because it asserts that Element.HasElements = False
  with FElement as TJvxHTMLFieldSet do
  begin
    // Test predefined legend element
    TestNode(Elements, Legend, 'legend', TJvxHTMLLegend);

    // Test that second legend element is not allowed
    TestInvalidChildNode(Elements, TJvxHTMLLegend);

    // Test allowed child elements
    TestValidChildNode(Elements, TJvxHTMLForm);
    TestValidChildNode(Elements, TJvxHTMLScript);
    TestValidChildNode(Elements, TJvxHTMLSpan);
    TestValidChildNode(Elements, TJvxHTMLDiv);
  end;
end;

procedure TJvxHTMLFieldSetTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'fieldset');
end;

{ TJvxHTMLCustomFocusableElementTest }

type
  TJvxHTMLExposedFocusableElement = class(TJvxHTMLCustomFocusableElement);

procedure TJvxHTMLCustomFocusableElementTest.TestAttributes;
begin
  inherited;
  with TJvxHTMLExposedFocusableElement(FElement) do
  begin
    TestNode(Attributes, AccessKey, 'accesskey', TJvxProtectedStringAttribute);
    TestNode(Attributes, OnBlur, 'onblur', TJvxHTMLScriptAttribute);
    TestNode(Attributes, OnFocus, 'onfocus', TJvxHTMLScriptAttribute);
    TestNode(Attributes, TabIndex, 'tabindex', TJvxProtectedIntegerAttribute);
  end;
end;

{ TJvxHTMLLabelTest }

procedure TJvxHTMLLabelTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLLabel.Create(nil);
end;

procedure TJvxHTMLLabelTest.TestAttributes;
begin
  inherited;
  with FElement as TJvxHTMLLabel do
  begin
    TestNode(Attributes, FocusControl, 'for', TJvxHTMLIdReferenceAttribute);
  end;
end;

procedure TJvxHTMLLabelTest.TestElements;
begin
  inherited;
  // Test that nested labels are not allowed
  TestInvalidChildNode(FElement.Elements, TJvxHTMLLabel);
  // Test that any inline elements are allowed
  TestValidChildNode(FElement.Elements, TJvxHTMLSpan);
end;

procedure TJvxHTMLLabelTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'label');
end;

{ TJvxHTMLSelectTest }

procedure TJvxHTMLSelectTest.InternalCreateFixture;
begin
  inherited;
  FElement := TJvxHTMLSelect.Create(nil);
end;

procedure TJvxHTMLSelectTest.InternalDestroyFixture;
begin
  FElement.Free;
  inherited;
end;

procedure TJvxHTMLSelectTest.TestAttributes;
begin
  // todo: finish TJvxHTMLSelectTest.TestAttributes;
end;

procedure TJvxHTMLSelectTest.TestElements;
begin
  // todo: finish TJvxHTMLSelectTest.TestElements;
end;

procedure TJvxHTMLSelectTest.TestNodeName;
begin
  Assert(FElement.NodeName = 'select');
end;

{ TJvxHTMLImportInstructionTest }

procedure TJvxHTMLImportInstructionTest.InternalCreateFixture;
begin
  inherited;
  FInstruction := TJvxHTMLImportInstruction.Create(nil);
end;

procedure TJvxHTMLImportInstructionTest.InternalDestroyFixture;
begin
  FInstruction.Free;
  inherited;
end;

procedure TJvxHTMLImportInstructionTest.TestAttributes;
begin
  with FInstruction do
  begin
    TestNode(Attributes, ElementImplementation, 'implementation',
      TJvxHTMLURIAttribute);
    TestNode(Attributes, NameSpace, 'namespace', TJvxProtectedStringAttribute);
  end;
end;

procedure TJvxHTMLImportInstructionTest.TestElements;
begin
  Assert(FInstruction.HasElements = False);
end;

procedure TJvxHTMLImportInstructionTest.TestNodeName;
begin
  Assert(FInstruction.NodeName = 'import');
end;

initialization
  JvxRegisterTestClass(TJvxHTMLTitleTest);
  JvxRegisterTestClass(TJvxHTMLHeadTest);
  JvxRegisterTestClass(TJvxHTMLBodyTest);
  JvxRegisterTestClass(TJvxHTMLSelectTest);
  JvxRegisterTestClass(TJvxHTMLElementTest);
  JvxRegisterTestClass(TJvxHTMLLegendTest);
  JvxRegisterTestClass(TJvxHTMLFieldSetTest);
  JvxRegisterTestClass(TJvxHTMLLabelTest);
  JvxRegisterTestClass(TJvxHTMLImportInstructionTest);

end.
